#nullable enable
using Canon.Core.Abstractions;
using Canon.Core.GrammarParser;
using Canon.Core.Enums;
namespace Canon.Tests.GeneratedParserTests;

public class GeneratedTransformer : ITransformer
{
    private IDictionary<TerminatorBase, string> _shiftPointers;

    public string Name { get; }

    public IDictionary<Terminator, ReduceInformation> ReduceTable { get; }

    public IDictionary<TerminatorBase, ITransformer> ShiftTable { get; }

    public GeneratedTransformer(Dictionary<TerminatorBase, string> shiftTable,
        Dictionary<Terminator, ReduceInformation> reduceTable, string name)
    {
        ReduceTable = reduceTable;
        ShiftTable = new Dictionary<TerminatorBase, ITransformer>();
        _shiftPointers = shiftTable;
        Name = name;
    }

    public GeneratedTransformer()
    {
        ReduceTable = new Dictionary<Terminator, ReduceInformation>();
        ShiftTable = new Dictionary<TerminatorBase, ITransformer>();
        _shiftPointers = new Dictionary<TerminatorBase, string>();
        Name = Guid.NewGuid().ToString();
    }

    public void ConstructShiftTable(Dictionary<string, GeneratedTransformer> transformers)
    {
        foreach (KeyValuePair<TerminatorBase,string> pair in _shiftPointers)
        {
            ShiftTable.Add(pair.Key, transformers[pair.Value]);
        }
    }

    public override bool Equals(object? obj)
    {
        if (obj is not GeneratedTransformer other)
        {
            return false;
        }

        return Name == other.Name;
    }

    public override int GetHashCode() => Name.GetHashCode();
}
public class GeneratedGrammarParser : IGrammarParser
{
	private static readonly Dictionary<string, GeneratedTransformer> s_transformers = new()
	{
		{ "e697e771-c2e7-47fc-96a0-7070dbb2ce33", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.ProgramStruct), "80e3bd16-4b4a-4501-94f9-cc27526e43b3"}, { new NonTerminator(NonTerminatorType.ProgramHead), "8c5721c6-3439-460e-be9b-b2d8bbc8eb0f"}, { new Terminator(KeywordType.Program), "9b115af8-5d5b-4bbb-a4a3-f578256d2284"},}, new Dictionary<Terminator, ReduceInformation>{ }, "e697e771-c2e7-47fc-96a0-7070dbb2ce33") },
		{ "80e3bd16-4b4a-4501-94f9-cc27526e43b3", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { Terminator.EndTerminator, new ReduceInformation(1, new NonTerminator(NonTerminatorType.StartNonTerminator))}, }, "80e3bd16-4b4a-4501-94f9-cc27526e43b3") },
		{ "8c5721c6-3439-460e-be9b-b2d8bbc8eb0f", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.Semicolon), "c76033a8-d295-435a-a71f-b72698fc31fa"},}, new Dictionary<Terminator, ReduceInformation>{ }, "8c5721c6-3439-460e-be9b-b2d8bbc8eb0f") },
		{ "9b115af8-5d5b-4bbb-a4a3-f578256d2284", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { Terminator.IdentifierTerminator, "a10409a9-cb2b-4b1e-be74-207e472169b4"},}, new Dictionary<Terminator, ReduceInformation>{ }, "9b115af8-5d5b-4bbb-a4a3-f578256d2284") },
		{ "c76033a8-d295-435a-a71f-b72698fc31fa", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.ProgramBody), "dcb982ef-f1db-4cec-9c5a-bd2c76a5ac5d"}, { new NonTerminator(NonTerminatorType.ConstDeclarations), "ce9dbf6f-f9f4-48a4-909e-a9fe4fb01444"}, { new Terminator(KeywordType.Const), "dc7ac06d-67a1-4ad2-916e-db32fb93745e"},}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.Begin), new ReduceInformation(0, new NonTerminator(NonTerminatorType.ConstDeclarations))}, { new Terminator(KeywordType.Procedure), new ReduceInformation(0, new NonTerminator(NonTerminatorType.ConstDeclarations))}, { new Terminator(KeywordType.Function), new ReduceInformation(0, new NonTerminator(NonTerminatorType.ConstDeclarations))}, { new Terminator(KeywordType.Var), new ReduceInformation(0, new NonTerminator(NonTerminatorType.ConstDeclarations))}, }, "c76033a8-d295-435a-a71f-b72698fc31fa") },
		{ "a10409a9-cb2b-4b1e-be74-207e472169b4", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.LeftParenthesis), "12e85a6c-053c-4405-b2ae-2e6516c40443"},}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.Semicolon), new ReduceInformation(2, new NonTerminator(NonTerminatorType.ProgramHead))}, }, "a10409a9-cb2b-4b1e-be74-207e472169b4") },
		{ "dcb982ef-f1db-4cec-9c5a-bd2c76a5ac5d", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.Period), "95cca640-4098-4c26-b126-6ca078d6eafd"},}, new Dictionary<Terminator, ReduceInformation>{ }, "dcb982ef-f1db-4cec-9c5a-bd2c76a5ac5d") },
		{ "ce9dbf6f-f9f4-48a4-909e-a9fe4fb01444", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.VarDeclarations), "5b42918b-0da5-4b64-99ff-edd13126a49c"}, { new Terminator(KeywordType.Var), "c5062536-3d53-47d9-a3a1-6fc865e4bac1"},}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.Begin), new ReduceInformation(0, new NonTerminator(NonTerminatorType.VarDeclarations))}, { new Terminator(KeywordType.Procedure), new ReduceInformation(0, new NonTerminator(NonTerminatorType.VarDeclarations))}, { new Terminator(KeywordType.Function), new ReduceInformation(0, new NonTerminator(NonTerminatorType.VarDeclarations))}, }, "ce9dbf6f-f9f4-48a4-909e-a9fe4fb01444") },
		{ "dc7ac06d-67a1-4ad2-916e-db32fb93745e", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.ConstDeclaration), "3f81753a-9038-4fbd-b5e0-53a1f81b7b24"}, { Terminator.IdentifierTerminator, "bf1692f6-9e37-4ad3-a5bf-48e0657ab9a0"},}, new Dictionary<Terminator, ReduceInformation>{ }, "dc7ac06d-67a1-4ad2-916e-db32fb93745e") },
		{ "12e85a6c-053c-4405-b2ae-2e6516c40443", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.IdentifierList), "6cedc276-38e3-4725-be04-3f4ea2e244b7"}, { Terminator.IdentifierTerminator, "db1b1397-6337-48e4-96b1-d5c0b645ca6f"},}, new Dictionary<Terminator, ReduceInformation>{ }, "12e85a6c-053c-4405-b2ae-2e6516c40443") },
		{ "95cca640-4098-4c26-b126-6ca078d6eafd", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { Terminator.EndTerminator, new ReduceInformation(4, new NonTerminator(NonTerminatorType.ProgramStruct))}, }, "95cca640-4098-4c26-b126-6ca078d6eafd") },
		{ "5b42918b-0da5-4b64-99ff-edd13126a49c", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.SubprogramDeclarations), "f538fd8d-4dca-4557-81e9-4e89c0a7ccd2"},}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.Begin), new ReduceInformation(0, new NonTerminator(NonTerminatorType.SubprogramDeclarations))}, { new Terminator(KeywordType.Procedure), new ReduceInformation(0, new NonTerminator(NonTerminatorType.SubprogramDeclarations))}, { new Terminator(KeywordType.Function), new ReduceInformation(0, new NonTerminator(NonTerminatorType.SubprogramDeclarations))}, }, "5b42918b-0da5-4b64-99ff-edd13126a49c") },
		{ "c5062536-3d53-47d9-a3a1-6fc865e4bac1", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.VarDeclaration), "a0445b23-37d0-4051-aa35-be5f65c89dd5"}, { new NonTerminator(NonTerminatorType.IdentifierList), "8c027315-eabd-4555-bf71-ae4e3c7cda0c"}, { Terminator.IdentifierTerminator, "c50004c4-fac8-4812-a2e2-6467f2005495"},}, new Dictionary<Terminator, ReduceInformation>{ }, "c5062536-3d53-47d9-a3a1-6fc865e4bac1") },
		{ "3f81753a-9038-4fbd-b5e0-53a1f81b7b24", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.Semicolon), "a25f42bc-1526-4fd1-9013-37acc9d2768a"},}, new Dictionary<Terminator, ReduceInformation>{ }, "3f81753a-9038-4fbd-b5e0-53a1f81b7b24") },
		{ "bf1692f6-9e37-4ad3-a5bf-48e0657ab9a0", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(OperatorType.Equal), "f26c47de-c5ea-49f6-b38f-2dfdf2162095"},}, new Dictionary<Terminator, ReduceInformation>{ }, "bf1692f6-9e37-4ad3-a5bf-48e0657ab9a0") },
		{ "6cedc276-38e3-4725-be04-3f4ea2e244b7", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.RightParenthesis), "9ebbc40a-483f-4bdf-91b6-992f1e231129"}, { new Terminator(DelimiterType.Comma), "cc6bb2c4-812a-4abb-8fe2-8f01d7051976"},}, new Dictionary<Terminator, ReduceInformation>{ }, "6cedc276-38e3-4725-be04-3f4ea2e244b7") },
		{ "db1b1397-6337-48e4-96b1-d5c0b645ca6f", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightParenthesis), new ReduceInformation(1, new NonTerminator(NonTerminatorType.IdentifierList))}, { new Terminator(DelimiterType.Comma), new ReduceInformation(1, new NonTerminator(NonTerminatorType.IdentifierList))}, }, "db1b1397-6337-48e4-96b1-d5c0b645ca6f") },
		{ "f538fd8d-4dca-4557-81e9-4e89c0a7ccd2", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.CompoundStatement), "4e47e44a-24eb-4e39-a057-5508719d254d"}, { new Terminator(KeywordType.Begin), "c89026ea-27e5-43f5-8ff0-c93c5902c585"}, { new NonTerminator(NonTerminatorType.Subprogram), "c2002ab4-3f72-4b2b-8cd5-27d38dd53e5d"}, { new NonTerminator(NonTerminatorType.SubprogramHead), "0a3c4ede-a007-43b2-9112-a40c50248245"}, { new Terminator(KeywordType.Procedure), "4da611ae-9f99-4beb-abb1-38172ffd8315"}, { new Terminator(KeywordType.Function), "968acde1-5a05-45e8-96e9-2d70dd8c3204"},}, new Dictionary<Terminator, ReduceInformation>{ }, "f538fd8d-4dca-4557-81e9-4e89c0a7ccd2") },
		{ "a0445b23-37d0-4051-aa35-be5f65c89dd5", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.Semicolon), "5ad81cc8-f47a-404a-9f49-e0fa66418513"},}, new Dictionary<Terminator, ReduceInformation>{ }, "a0445b23-37d0-4051-aa35-be5f65c89dd5") },
		{ "8c027315-eabd-4555-bf71-ae4e3c7cda0c", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.Colon), "05c25427-6a5a-4857-bc55-30ab6b7d75cc"}, { new Terminator(DelimiterType.Comma), "15e52f16-d101-4097-a288-bb0e1585a9ff"},}, new Dictionary<Terminator, ReduceInformation>{ }, "8c027315-eabd-4555-bf71-ae4e3c7cda0c") },
		{ "c50004c4-fac8-4812-a2e2-6467f2005495", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.Colon), new ReduceInformation(1, new NonTerminator(NonTerminatorType.IdentifierList))}, { new Terminator(DelimiterType.Comma), new ReduceInformation(1, new NonTerminator(NonTerminatorType.IdentifierList))}, }, "c50004c4-fac8-4812-a2e2-6467f2005495") },
		{ "a25f42bc-1526-4fd1-9013-37acc9d2768a", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { Terminator.IdentifierTerminator, "5e84e22e-943e-4d5b-bda9-89e059814265"},}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.Begin), new ReduceInformation(3, new NonTerminator(NonTerminatorType.ConstDeclarations))}, { new Terminator(KeywordType.Procedure), new ReduceInformation(3, new NonTerminator(NonTerminatorType.ConstDeclarations))}, { new Terminator(KeywordType.Function), new ReduceInformation(3, new NonTerminator(NonTerminatorType.ConstDeclarations))}, { new Terminator(KeywordType.Var), new ReduceInformation(3, new NonTerminator(NonTerminatorType.ConstDeclarations))}, }, "a25f42bc-1526-4fd1-9013-37acc9d2768a") },
		{ "f26c47de-c5ea-49f6-b38f-2dfdf2162095", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.ConstValue), "f822a3e5-8be4-4970-9da9-bab0a9269cca"}, { new Terminator(OperatorType.Plus), "a571965c-2926-4726-9ebc-41392af100de"}, { new Terminator(OperatorType.Minus), "68c9ee0a-e63f-48b4-be47-10ce8fa20e05"}, { Terminator.NumberTerminator, "e6e75567-82f1-4b31-87e3-d9fb13d16b4a"}, { Terminator.CharacterTerminator, "b5ce8073-d716-491f-aea1-70e2b6f0e2fe"},}, new Dictionary<Terminator, ReduceInformation>{ }, "f26c47de-c5ea-49f6-b38f-2dfdf2162095") },
		{ "9ebbc40a-483f-4bdf-91b6-992f1e231129", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.Semicolon), new ReduceInformation(5, new NonTerminator(NonTerminatorType.ProgramHead))}, }, "9ebbc40a-483f-4bdf-91b6-992f1e231129") },
		{ "cc6bb2c4-812a-4abb-8fe2-8f01d7051976", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { Terminator.IdentifierTerminator, "fe33991d-d7f5-439a-bf2b-87c94dabc945"},}, new Dictionary<Terminator, ReduceInformation>{ }, "cc6bb2c4-812a-4abb-8fe2-8f01d7051976") },
		{ "4e47e44a-24eb-4e39-a057-5508719d254d", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.Period), new ReduceInformation(4, new NonTerminator(NonTerminatorType.ProgramBody))}, }, "4e47e44a-24eb-4e39-a057-5508719d254d") },
		{ "c89026ea-27e5-43f5-8ff0-c93c5902c585", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.StatementList), "f4b5d432-8cad-4783-b580-824236ab9d50"}, { new NonTerminator(NonTerminatorType.Statement), "5d302683-f6d0-4c7f-8a86-9311c20ee9f5"}, { new NonTerminator(NonTerminatorType.Variable), "168b14a3-c5c2-4d3b-85d6-8401a5d72d3f"}, { Terminator.IdentifierTerminator, "a9bcbf2a-3d77-41a8-a4cb-35695590c1a7"}, { new NonTerminator(NonTerminatorType.ProcedureCall), "ee7bd04c-ba1b-4c05-9f5a-92dcbd37e67b"}, { new NonTerminator(NonTerminatorType.CompoundStatement), "372dfbe4-160d-4966-bc33-aafc60a96c43"}, { new Terminator(KeywordType.If), "67ff610b-d0c2-4e7d-9505-4dceffaa4ee1"}, { new Terminator(KeywordType.For), "3bee9f71-bf83-4ee3-a95e-aeefaa9e9f45"}, { new Terminator(KeywordType.Begin), "de7a627a-1e9b-41f4-835e-c30359b633fd"},}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(0, new NonTerminator(NonTerminatorType.Statement))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(0, new NonTerminator(NonTerminatorType.Statement))}, }, "c89026ea-27e5-43f5-8ff0-c93c5902c585") },
		{ "c2002ab4-3f72-4b2b-8cd5-27d38dd53e5d", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.Semicolon), "443983bd-14c2-48f5-9a52-ead4b422aae1"},}, new Dictionary<Terminator, ReduceInformation>{ }, "c2002ab4-3f72-4b2b-8cd5-27d38dd53e5d") },
		{ "0a3c4ede-a007-43b2-9112-a40c50248245", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.Semicolon), "e45c51cc-cdb7-4d7c-abb0-11e00b5dd0a2"},}, new Dictionary<Terminator, ReduceInformation>{ }, "0a3c4ede-a007-43b2-9112-a40c50248245") },
		{ "4da611ae-9f99-4beb-abb1-38172ffd8315", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { Terminator.IdentifierTerminator, "7819577f-f637-4d9f-bac6-d2fafcdfadf4"},}, new Dictionary<Terminator, ReduceInformation>{ }, "4da611ae-9f99-4beb-abb1-38172ffd8315") },
		{ "968acde1-5a05-45e8-96e9-2d70dd8c3204", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { Terminator.IdentifierTerminator, "daac83c0-9e88-42e3-9a3e-5642f9040d4a"},}, new Dictionary<Terminator, ReduceInformation>{ }, "968acde1-5a05-45e8-96e9-2d70dd8c3204") },
		{ "5ad81cc8-f47a-404a-9f49-e0fa66418513", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.IdentifierList), "7080b182-243c-4083-8e43-08fb4cd8954a"}, { Terminator.IdentifierTerminator, "c50004c4-fac8-4812-a2e2-6467f2005495"},}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.Begin), new ReduceInformation(3, new NonTerminator(NonTerminatorType.VarDeclarations))}, { new Terminator(KeywordType.Procedure), new ReduceInformation(3, new NonTerminator(NonTerminatorType.VarDeclarations))}, { new Terminator(KeywordType.Function), new ReduceInformation(3, new NonTerminator(NonTerminatorType.VarDeclarations))}, }, "5ad81cc8-f47a-404a-9f49-e0fa66418513") },
		{ "05c25427-6a5a-4857-bc55-30ab6b7d75cc", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Type), "c4de0eaf-cdfd-406d-a7a8-c2b167a10c3f"}, { new NonTerminator(NonTerminatorType.BasicType), "3c45f039-db62-46ee-a41d-23dacfb8d0f0"}, { new Terminator(KeywordType.Array), "07f694b1-c082-40f3-8ea0-c6eda8e73b83"}, { new Terminator(KeywordType.Integer), "7bb70cf0-2067-4971-ada1-ccf973097e8b"}, { new Terminator(KeywordType.Real), "1023abfc-7be1-4c20-890e-3ff30b2c89fc"}, { new Terminator(KeywordType.Boolean), "7430a84c-7617-405a-9992-d202dcbc8adc"}, { new Terminator(KeywordType.Character), "c1bdaca1-017b-4856-84d6-4cd20b5911a3"},}, new Dictionary<Terminator, ReduceInformation>{ }, "05c25427-6a5a-4857-bc55-30ab6b7d75cc") },
		{ "15e52f16-d101-4097-a288-bb0e1585a9ff", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { Terminator.IdentifierTerminator, "969ef315-f76e-4882-b7f8-49a680227e5f"},}, new Dictionary<Terminator, ReduceInformation>{ }, "15e52f16-d101-4097-a288-bb0e1585a9ff") },
		{ "5e84e22e-943e-4d5b-bda9-89e059814265", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(OperatorType.Equal), "42e39a9d-60ab-4074-838c-39bb015766dc"},}, new Dictionary<Terminator, ReduceInformation>{ }, "5e84e22e-943e-4d5b-bda9-89e059814265") },
		{ "f822a3e5-8be4-4970-9da9-bab0a9269cca", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.Semicolon), new ReduceInformation(3, new NonTerminator(NonTerminatorType.ConstDeclaration))}, }, "f822a3e5-8be4-4970-9da9-bab0a9269cca") },
		{ "a571965c-2926-4726-9ebc-41392af100de", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { Terminator.NumberTerminator, "499e9d93-5c6f-431e-8697-ff85804d2b81"},}, new Dictionary<Terminator, ReduceInformation>{ }, "a571965c-2926-4726-9ebc-41392af100de") },
		{ "68c9ee0a-e63f-48b4-be47-10ce8fa20e05", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { Terminator.NumberTerminator, "bf65c993-a148-4aff-9a75-4fd7fb171dd7"},}, new Dictionary<Terminator, ReduceInformation>{ }, "68c9ee0a-e63f-48b4-be47-10ce8fa20e05") },
		{ "e6e75567-82f1-4b31-87e3-d9fb13d16b4a", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.Semicolon), new ReduceInformation(1, new NonTerminator(NonTerminatorType.ConstValue))}, }, "e6e75567-82f1-4b31-87e3-d9fb13d16b4a") },
		{ "b5ce8073-d716-491f-aea1-70e2b6f0e2fe", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.Semicolon), new ReduceInformation(1, new NonTerminator(NonTerminatorType.ConstValue))}, }, "b5ce8073-d716-491f-aea1-70e2b6f0e2fe") },
		{ "fe33991d-d7f5-439a-bf2b-87c94dabc945", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightParenthesis), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdentifierList))}, { new Terminator(DelimiterType.Comma), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdentifierList))}, }, "fe33991d-d7f5-439a-bf2b-87c94dabc945") },
		{ "f4b5d432-8cad-4783-b580-824236ab9d50", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(KeywordType.End), "e5bf326f-c375-4cbc-b3e0-61ef788b5f69"}, { new Terminator(DelimiterType.Semicolon), "41bdccbb-3f79-4d74-9b06-5a48b781a2c2"},}, new Dictionary<Terminator, ReduceInformation>{ }, "f4b5d432-8cad-4783-b580-824236ab9d50") },
		{ "5d302683-f6d0-4c7f-8a86-9311c20ee9f5", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(1, new NonTerminator(NonTerminatorType.StatementList))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(1, new NonTerminator(NonTerminatorType.StatementList))}, }, "5d302683-f6d0-4c7f-8a86-9311c20ee9f5") },
		{ "168b14a3-c5c2-4d3b-85d6-8401a5d72d3f", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(OperatorType.Assign), "39240a36-120e-4b21-b021-3ea33c914ee5"},}, new Dictionary<Terminator, ReduceInformation>{ }, "168b14a3-c5c2-4d3b-85d6-8401a5d72d3f") },
		{ "a9bcbf2a-3d77-41a8-a4cb-35695590c1a7", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(OperatorType.Assign), "2a5ebf4f-7ea9-446b-bc3c-71dc7123d81d"}, { new NonTerminator(NonTerminatorType.IdVarPart), "a4f48ffb-2f04-4d1c-a0cd-03d46da54477"}, { new Terminator(DelimiterType.LeftSquareBracket), "d00f55eb-4b09-4fe5-9eab-efff3126957e"}, { new Terminator(DelimiterType.LeftParenthesis), "0ca508e1-6f0b-4065-8613-a0133a8c5789"},}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(OperatorType.Assign), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.End), new ReduceInformation(1, new NonTerminator(NonTerminatorType.ProcedureCall))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(1, new NonTerminator(NonTerminatorType.ProcedureCall))}, }, "a9bcbf2a-3d77-41a8-a4cb-35695590c1a7") },
		{ "ee7bd04c-ba1b-4c05-9f5a-92dcbd37e67b", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Statement))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Statement))}, }, "ee7bd04c-ba1b-4c05-9f5a-92dcbd37e67b") },
		{ "372dfbe4-160d-4966-bc33-aafc60a96c43", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Statement))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Statement))}, }, "372dfbe4-160d-4966-bc33-aafc60a96c43") },
		{ "67ff610b-d0c2-4e7d-9505-4dceffaa4ee1", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Expression), "590378dd-f4c6-488c-9668-ed26e4e8f03f"}, { new NonTerminator(NonTerminatorType.SimpleExpression), "c709baca-481b-49c4-90c0-916900b02b3e"}, { new NonTerminator(NonTerminatorType.Term), "f5df4190-3621-4ffd-abac-15a525352e5c"}, { new NonTerminator(NonTerminatorType.Factor), "aa2a69ba-536a-4716-9726-fd4bca6a2ecb"}, { Terminator.NumberTerminator, "f2e280ec-cc53-4dd5-9e78-4d96496b2e43"}, { new NonTerminator(NonTerminatorType.Variable), "61692e3e-df15-4f2e-bbda-0e43f17752df"}, { new Terminator(DelimiterType.LeftParenthesis), "77ef6b8f-5b0a-42d4-9397-9bee38e59991"}, { Terminator.IdentifierTerminator, "43a0e8af-43b0-436c-bf72-119f9adb1db1"}, { new Terminator(KeywordType.Not), "bf6cd66a-c610-4c98-8ec1-fce070c9ea7a"}, { new Terminator(OperatorType.Minus), "5a485bf7-252f-452b-a035-e99007748568"},}, new Dictionary<Terminator, ReduceInformation>{ }, "67ff610b-d0c2-4e7d-9505-4dceffaa4ee1") },
		{ "3bee9f71-bf83-4ee3-a95e-aeefaa9e9f45", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { Terminator.IdentifierTerminator, "0a9e63ba-4e0d-4c77-8581-7fc661746091"},}, new Dictionary<Terminator, ReduceInformation>{ }, "3bee9f71-bf83-4ee3-a95e-aeefaa9e9f45") },
		{ "de7a627a-1e9b-41f4-835e-c30359b633fd", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.StatementList), "c9a81728-5d86-417f-8eb5-1f8592ffe2a8"}, { new NonTerminator(NonTerminatorType.Statement), "5d302683-f6d0-4c7f-8a86-9311c20ee9f5"}, { new NonTerminator(NonTerminatorType.Variable), "168b14a3-c5c2-4d3b-85d6-8401a5d72d3f"}, { Terminator.IdentifierTerminator, "a9bcbf2a-3d77-41a8-a4cb-35695590c1a7"}, { new NonTerminator(NonTerminatorType.ProcedureCall), "ee7bd04c-ba1b-4c05-9f5a-92dcbd37e67b"}, { new NonTerminator(NonTerminatorType.CompoundStatement), "372dfbe4-160d-4966-bc33-aafc60a96c43"}, { new Terminator(KeywordType.If), "67ff610b-d0c2-4e7d-9505-4dceffaa4ee1"}, { new Terminator(KeywordType.For), "3bee9f71-bf83-4ee3-a95e-aeefaa9e9f45"}, { new Terminator(KeywordType.Begin), "de7a627a-1e9b-41f4-835e-c30359b633fd"},}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(0, new NonTerminator(NonTerminatorType.Statement))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(0, new NonTerminator(NonTerminatorType.Statement))}, }, "de7a627a-1e9b-41f4-835e-c30359b633fd") },
		{ "443983bd-14c2-48f5-9a52-ead4b422aae1", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.Begin), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SubprogramDeclarations))}, { new Terminator(KeywordType.Procedure), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SubprogramDeclarations))}, { new Terminator(KeywordType.Function), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SubprogramDeclarations))}, }, "443983bd-14c2-48f5-9a52-ead4b422aae1") },
		{ "e45c51cc-cdb7-4d7c-abb0-11e00b5dd0a2", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.SubprogramBody), "10aad22b-4cdf-43d3-806c-010b4743b507"}, { new NonTerminator(NonTerminatorType.ConstDeclarations), "8f9dff69-539c-488d-a4bd-527b6798ac27"}, { new Terminator(KeywordType.Const), "72afbb5c-a7fc-4979-9495-90a0a736c705"},}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.Begin), new ReduceInformation(0, new NonTerminator(NonTerminatorType.ConstDeclarations))}, { new Terminator(KeywordType.Var), new ReduceInformation(0, new NonTerminator(NonTerminatorType.ConstDeclarations))}, }, "e45c51cc-cdb7-4d7c-abb0-11e00b5dd0a2") },
		{ "7819577f-f637-4d9f-bac6-d2fafcdfadf4", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.FormalParameter), "deab9d75-c09a-421f-b37b-22d5434ef1ca"}, { new Terminator(DelimiterType.LeftParenthesis), "6b68a8b9-a28e-4b48-8e23-a6817d5e7b00"},}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.Semicolon), new ReduceInformation(0, new NonTerminator(NonTerminatorType.FormalParameter))}, }, "7819577f-f637-4d9f-bac6-d2fafcdfadf4") },
		{ "daac83c0-9e88-42e3-9a3e-5642f9040d4a", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.FormalParameter), "f6c54cdd-7e5a-4714-950b-2374191c9aad"}, { new Terminator(DelimiterType.LeftParenthesis), "6f4c1ec2-4545-4575-a956-8ba95a98dddc"},}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.Colon), new ReduceInformation(0, new NonTerminator(NonTerminatorType.FormalParameter))}, }, "daac83c0-9e88-42e3-9a3e-5642f9040d4a") },
		{ "7080b182-243c-4083-8e43-08fb4cd8954a", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.Colon), "651b45b5-da7d-4f35-a03c-afd82195fb68"}, { new Terminator(DelimiterType.Comma), "15e52f16-d101-4097-a288-bb0e1585a9ff"},}, new Dictionary<Terminator, ReduceInformation>{ }, "7080b182-243c-4083-8e43-08fb4cd8954a") },
		{ "c4de0eaf-cdfd-406d-a7a8-c2b167a10c3f", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.Semicolon), new ReduceInformation(3, new NonTerminator(NonTerminatorType.VarDeclaration))}, }, "c4de0eaf-cdfd-406d-a7a8-c2b167a10c3f") },
		{ "3c45f039-db62-46ee-a41d-23dacfb8d0f0", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.Semicolon), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Type))}, }, "3c45f039-db62-46ee-a41d-23dacfb8d0f0") },
		{ "07f694b1-c082-40f3-8ea0-c6eda8e73b83", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.LeftSquareBracket), "92206523-e417-495f-94aa-812bc43665f3"},}, new Dictionary<Terminator, ReduceInformation>{ }, "07f694b1-c082-40f3-8ea0-c6eda8e73b83") },
		{ "7bb70cf0-2067-4971-ada1-ccf973097e8b", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.Semicolon), new ReduceInformation(1, new NonTerminator(NonTerminatorType.BasicType))}, }, "7bb70cf0-2067-4971-ada1-ccf973097e8b") },
		{ "1023abfc-7be1-4c20-890e-3ff30b2c89fc", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.Semicolon), new ReduceInformation(1, new NonTerminator(NonTerminatorType.BasicType))}, }, "1023abfc-7be1-4c20-890e-3ff30b2c89fc") },
		{ "7430a84c-7617-405a-9992-d202dcbc8adc", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.Semicolon), new ReduceInformation(1, new NonTerminator(NonTerminatorType.BasicType))}, }, "7430a84c-7617-405a-9992-d202dcbc8adc") },
		{ "c1bdaca1-017b-4856-84d6-4cd20b5911a3", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.Semicolon), new ReduceInformation(1, new NonTerminator(NonTerminatorType.BasicType))}, }, "c1bdaca1-017b-4856-84d6-4cd20b5911a3") },
		{ "969ef315-f76e-4882-b7f8-49a680227e5f", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.Colon), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdentifierList))}, { new Terminator(DelimiterType.Comma), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdentifierList))}, }, "969ef315-f76e-4882-b7f8-49a680227e5f") },
		{ "42e39a9d-60ab-4074-838c-39bb015766dc", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.ConstValue), "035c7652-7a28-43f7-ba44-1031844761e2"}, { new Terminator(OperatorType.Plus), "a571965c-2926-4726-9ebc-41392af100de"}, { new Terminator(OperatorType.Minus), "68c9ee0a-e63f-48b4-be47-10ce8fa20e05"}, { Terminator.NumberTerminator, "e6e75567-82f1-4b31-87e3-d9fb13d16b4a"}, { Terminator.CharacterTerminator, "b5ce8073-d716-491f-aea1-70e2b6f0e2fe"},}, new Dictionary<Terminator, ReduceInformation>{ }, "42e39a9d-60ab-4074-838c-39bb015766dc") },
		{ "499e9d93-5c6f-431e-8697-ff85804d2b81", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.Semicolon), new ReduceInformation(2, new NonTerminator(NonTerminatorType.ConstValue))}, }, "499e9d93-5c6f-431e-8697-ff85804d2b81") },
		{ "bf65c993-a148-4aff-9a75-4fd7fb171dd7", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.Semicolon), new ReduceInformation(2, new NonTerminator(NonTerminatorType.ConstValue))}, }, "bf65c993-a148-4aff-9a75-4fd7fb171dd7") },
		{ "e5bf326f-c375-4cbc-b3e0-61ef788b5f69", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.Period), new ReduceInformation(3, new NonTerminator(NonTerminatorType.CompoundStatement))}, }, "e5bf326f-c375-4cbc-b3e0-61ef788b5f69") },
		{ "41bdccbb-3f79-4d74-9b06-5a48b781a2c2", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Statement), "33d07881-003f-4441-ace4-327f287a6dde"}, { new NonTerminator(NonTerminatorType.Variable), "168b14a3-c5c2-4d3b-85d6-8401a5d72d3f"}, { Terminator.IdentifierTerminator, "a9bcbf2a-3d77-41a8-a4cb-35695590c1a7"}, { new NonTerminator(NonTerminatorType.ProcedureCall), "ee7bd04c-ba1b-4c05-9f5a-92dcbd37e67b"}, { new NonTerminator(NonTerminatorType.CompoundStatement), "372dfbe4-160d-4966-bc33-aafc60a96c43"}, { new Terminator(KeywordType.If), "67ff610b-d0c2-4e7d-9505-4dceffaa4ee1"}, { new Terminator(KeywordType.For), "3bee9f71-bf83-4ee3-a95e-aeefaa9e9f45"}, { new Terminator(KeywordType.Begin), "de7a627a-1e9b-41f4-835e-c30359b633fd"},}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(0, new NonTerminator(NonTerminatorType.Statement))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(0, new NonTerminator(NonTerminatorType.Statement))}, }, "41bdccbb-3f79-4d74-9b06-5a48b781a2c2") },
		{ "39240a36-120e-4b21-b021-3ea33c914ee5", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Expression), "c7ad577c-c184-4e56-90b3-ca6a2ea19d19"}, { new NonTerminator(NonTerminatorType.SimpleExpression), "64715301-bb45-40e5-9af2-b421dc9e1b8c"}, { new NonTerminator(NonTerminatorType.Term), "47e35174-231c-4a16-9387-25615446322b"}, { new NonTerminator(NonTerminatorType.Factor), "7a25b1d8-c432-47b5-b5d6-4b9d4ec99976"}, { Terminator.NumberTerminator, "51e160d6-1614-4bea-878b-b9200a78d134"}, { new NonTerminator(NonTerminatorType.Variable), "3ef7029f-efcc-4dfc-bbef-9de3a66266b7"}, { new Terminator(DelimiterType.LeftParenthesis), "271a6714-8230-4423-9ba9-b983289c29e4"}, { Terminator.IdentifierTerminator, "a21cd4d2-38d8-44d9-a8cc-f43a72534d65"}, { new Terminator(KeywordType.Not), "c4efdd9d-271b-4565-84a7-a0f9bae205fe"}, { new Terminator(OperatorType.Minus), "9ecf3833-38d5-4e56-84a4-74c622293d59"},}, new Dictionary<Terminator, ReduceInformation>{ }, "39240a36-120e-4b21-b021-3ea33c914ee5") },
		{ "2a5ebf4f-7ea9-446b-bc3c-71dc7123d81d", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Expression), "82678949-b5e6-4d69-85b7-9b77982dc2ef"}, { new NonTerminator(NonTerminatorType.SimpleExpression), "64715301-bb45-40e5-9af2-b421dc9e1b8c"}, { new NonTerminator(NonTerminatorType.Term), "47e35174-231c-4a16-9387-25615446322b"}, { new NonTerminator(NonTerminatorType.Factor), "7a25b1d8-c432-47b5-b5d6-4b9d4ec99976"}, { Terminator.NumberTerminator, "51e160d6-1614-4bea-878b-b9200a78d134"}, { new NonTerminator(NonTerminatorType.Variable), "3ef7029f-efcc-4dfc-bbef-9de3a66266b7"}, { new Terminator(DelimiterType.LeftParenthesis), "271a6714-8230-4423-9ba9-b983289c29e4"}, { Terminator.IdentifierTerminator, "a21cd4d2-38d8-44d9-a8cc-f43a72534d65"}, { new Terminator(KeywordType.Not), "c4efdd9d-271b-4565-84a7-a0f9bae205fe"}, { new Terminator(OperatorType.Minus), "9ecf3833-38d5-4e56-84a4-74c622293d59"},}, new Dictionary<Terminator, ReduceInformation>{ }, "2a5ebf4f-7ea9-446b-bc3c-71dc7123d81d") },
		{ "a4f48ffb-2f04-4d1c-a0cd-03d46da54477", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(OperatorType.Assign), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, }, "a4f48ffb-2f04-4d1c-a0cd-03d46da54477") },
		{ "d00f55eb-4b09-4fe5-9eab-efff3126957e", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.ExpressionList), "84afed73-d773-4a6a-931e-2b6448c2dca3"}, { new NonTerminator(NonTerminatorType.Expression), "a0ba5247-cbd6-46f6-b1fd-574979eb2b27"}, { new NonTerminator(NonTerminatorType.SimpleExpression), "e8b0426e-8437-45f8-a02f-3eebdfb48e84"}, { new NonTerminator(NonTerminatorType.Term), "cd4bfe7a-5a2e-41f1-a891-6359311b9403"}, { new NonTerminator(NonTerminatorType.Factor), "ceb0b15a-7ff9-4265-8766-716485412510"}, { Terminator.NumberTerminator, "06d206a2-8c81-4399-8790-c903a8acfe21"}, { new NonTerminator(NonTerminatorType.Variable), "422866b2-29d6-4004-b762-cb9ad9ddc5e7"}, { new Terminator(DelimiterType.LeftParenthesis), "4f42a0a3-3c4b-46a5-8dcc-b91007f93d5c"}, { Terminator.IdentifierTerminator, "7339bcec-b0ae-4611-83cd-08345e44324f"}, { new Terminator(KeywordType.Not), "bfdaacfe-9460-4db8-9469-712b6674efb5"}, { new Terminator(OperatorType.Minus), "eac356e4-6098-43be-9b50-3ff1a637a898"},}, new Dictionary<Terminator, ReduceInformation>{ }, "d00f55eb-4b09-4fe5-9eab-efff3126957e") },
		{ "0ca508e1-6f0b-4065-8613-a0133a8c5789", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.ExpressionList), "171d1827-4aaf-4774-bb69-b43842cda52a"}, { new NonTerminator(NonTerminatorType.Expression), "4f59631a-5553-49da-9f0d-dda48b0d65b2"}, { new NonTerminator(NonTerminatorType.SimpleExpression), "e2a2e2c4-61e9-496f-8a60-ac108404247f"}, { new NonTerminator(NonTerminatorType.Term), "83d0456c-4817-49a5-bd20-c0d8a1c5b696"}, { new NonTerminator(NonTerminatorType.Factor), "f1db3e46-b446-4adc-93ae-5dfed0964c99"}, { Terminator.NumberTerminator, "0abfb7c0-083d-4242-a9a7-06183f7230f7"}, { new NonTerminator(NonTerminatorType.Variable), "a8c27dec-213a-4917-8b05-b0422260d1c1"}, { new Terminator(DelimiterType.LeftParenthesis), "45c4069f-f3cf-48f7-af29-8eb8303c47d6"}, { Terminator.IdentifierTerminator, "5aabfbf6-09f5-484f-9f95-ac0fa52c26c4"}, { new Terminator(KeywordType.Not), "253ff382-06bc-47a7-9645-cc04c48e1d0a"}, { new Terminator(OperatorType.Minus), "5708e399-73e5-4bcb-9aeb-f25d1e06c35c"},}, new Dictionary<Terminator, ReduceInformation>{ }, "0ca508e1-6f0b-4065-8613-a0133a8c5789") },
		{ "590378dd-f4c6-488c-9668-ed26e4e8f03f", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(KeywordType.Then), "36034f1f-1d4c-4218-9176-291bc5da5fc3"},}, new Dictionary<Terminator, ReduceInformation>{ }, "590378dd-f4c6-488c-9668-ed26e4e8f03f") },
		{ "c709baca-481b-49c4-90c0-916900b02b3e", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.RelationOperator), "6a445aae-de55-4ebc-8144-36b6def60bd6"}, { new Terminator(OperatorType.Equal), "1b49be29-a9d8-403a-b434-5c116300d15c"}, { new Terminator(OperatorType.NotEqual), "fb7de677-df97-4431-b5b7-086a8ded26ff"}, { new Terminator(OperatorType.Less), "8db4bd06-8fe0-4476-924a-3354772a5ac3"}, { new Terminator(OperatorType.LessEqual), "68ce8cad-2c90-4f71-822f-885e193cc394"}, { new Terminator(OperatorType.Greater), "f863fa1a-f90f-430f-8ee8-4cb33053ee45"}, { new Terminator(OperatorType.GreaterEqual), "d79c0f7f-9997-421d-bed4-1e0b2517774c"}, { new NonTerminator(NonTerminatorType.AddOperator), "2122fdcb-bbda-4be2-9f58-e4f3c12ea687"}, { new Terminator(OperatorType.Plus), "b2a223d6-6905-4b72-a67b-96d8604fe688"}, { new Terminator(OperatorType.Minus), "569b7d42-5c9a-4ba6-b124-8dacd7f387f4"}, { new Terminator(KeywordType.Or), "37aaac08-c0a9-420d-bfd1-1ad8615a7513"},}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.Then), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Expression))}, }, "c709baca-481b-49c4-90c0-916900b02b3e") },
		{ "f5df4190-3621-4ffd-abac-15a525352e5c", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.MultiplyOperator), "4221e2a4-55fa-4449-b6e3-fe531d22ac26"}, { new Terminator(OperatorType.Multiply), "e576c109-5246-4139-8248-415b8e3e68de"}, { new Terminator(OperatorType.Divide), "6a567f32-04a0-49b8-bf8e-42f55ee0600f"}, { new Terminator(KeywordType.Divide), "86be9c46-a085-4183-9264-b12d73fde46e"}, { new Terminator(KeywordType.Mod), "d7708b97-7da8-492a-a64f-6196ae9527f7"}, { new Terminator(KeywordType.And), "271d9f46-4be0-495c-a90b-ae8f19ee928b"},}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.Then), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Equal), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Less), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Greater), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Plus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Minus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(KeywordType.Or), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, }, "f5df4190-3621-4ffd-abac-15a525352e5c") },
		{ "aa2a69ba-536a-4716-9726-fd4bca6a2ecb", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.Then), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Equal), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Less), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Greater), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Mod), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.And), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Plus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Minus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Or), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, }, "aa2a69ba-536a-4716-9726-fd4bca6a2ecb") },
		{ "f2e280ec-cc53-4dd5-9e78-4d96496b2e43", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.Then), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Equal), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Less), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Greater), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, }, "f2e280ec-cc53-4dd5-9e78-4d96496b2e43") },
		{ "61692e3e-df15-4f2e-bbda-0e43f17752df", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.Then), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Equal), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Less), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Greater), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, }, "61692e3e-df15-4f2e-bbda-0e43f17752df") },
		{ "77ef6b8f-5b0a-42d4-9397-9bee38e59991", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Expression), "3f3b4ebb-4308-40b1-9c77-4c7dc69c4ae5"}, { new NonTerminator(NonTerminatorType.SimpleExpression), "e566a9ad-f17c-46ff-981f-ee4fa81d2d3d"}, { new NonTerminator(NonTerminatorType.Term), "1f1e4a05-7d30-44f4-aa5f-5caacfbce4b2"}, { new NonTerminator(NonTerminatorType.Factor), "ff72d68f-ac20-46e8-94ed-090e841128c1"}, { Terminator.NumberTerminator, "79939c29-990e-4743-8a8a-5fccd2ec415d"}, { new NonTerminator(NonTerminatorType.Variable), "28f501cb-0b97-40c3-8a64-d214832a6789"}, { new Terminator(DelimiterType.LeftParenthesis), "f1464d64-129c-4d68-a61e-43296e963ee1"}, { Terminator.IdentifierTerminator, "9dd45908-639c-4e69-97e4-2471cdf5e189"}, { new Terminator(KeywordType.Not), "303776ee-584f-4977-b76a-9449d6a4e1ad"}, { new Terminator(OperatorType.Minus), "2401b03b-4e43-4409-989c-a4471708a795"},}, new Dictionary<Terminator, ReduceInformation>{ }, "77ef6b8f-5b0a-42d4-9397-9bee38e59991") },
		{ "43a0e8af-43b0-436c-bf72-119f9adb1db1", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.LeftParenthesis), "e8e3778c-2637-497d-8220-1ad1da0361e9"}, { new NonTerminator(NonTerminatorType.IdVarPart), "54b1c17e-914a-427a-be92-e4713c545e91"}, { new Terminator(DelimiterType.LeftSquareBracket), "8b173d7f-1010-4129-8ec7-38968bd4309c"},}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.Then), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Equal), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Less), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Greater), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Divide), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Divide), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Mod), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.And), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Plus), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Minus), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Or), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, }, "43a0e8af-43b0-436c-bf72-119f9adb1db1") },
		{ "bf6cd66a-c610-4c98-8ec1-fce070c9ea7a", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Factor), "04b41bb6-4af4-4542-998c-65289d824970"}, { Terminator.NumberTerminator, "f2e280ec-cc53-4dd5-9e78-4d96496b2e43"}, { new NonTerminator(NonTerminatorType.Variable), "61692e3e-df15-4f2e-bbda-0e43f17752df"}, { new Terminator(DelimiterType.LeftParenthesis), "77ef6b8f-5b0a-42d4-9397-9bee38e59991"}, { Terminator.IdentifierTerminator, "43a0e8af-43b0-436c-bf72-119f9adb1db1"}, { new Terminator(KeywordType.Not), "bf6cd66a-c610-4c98-8ec1-fce070c9ea7a"}, { new Terminator(OperatorType.Minus), "5a485bf7-252f-452b-a035-e99007748568"},}, new Dictionary<Terminator, ReduceInformation>{ }, "bf6cd66a-c610-4c98-8ec1-fce070c9ea7a") },
		{ "5a485bf7-252f-452b-a035-e99007748568", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Factor), "34ad805f-ae0b-4119-8ba0-5df56a2828d7"}, { Terminator.NumberTerminator, "f2e280ec-cc53-4dd5-9e78-4d96496b2e43"}, { new NonTerminator(NonTerminatorType.Variable), "61692e3e-df15-4f2e-bbda-0e43f17752df"}, { new Terminator(DelimiterType.LeftParenthesis), "77ef6b8f-5b0a-42d4-9397-9bee38e59991"}, { Terminator.IdentifierTerminator, "43a0e8af-43b0-436c-bf72-119f9adb1db1"}, { new Terminator(KeywordType.Not), "bf6cd66a-c610-4c98-8ec1-fce070c9ea7a"}, { new Terminator(OperatorType.Minus), "5a485bf7-252f-452b-a035-e99007748568"},}, new Dictionary<Terminator, ReduceInformation>{ }, "5a485bf7-252f-452b-a035-e99007748568") },
		{ "0a9e63ba-4e0d-4c77-8581-7fc661746091", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(OperatorType.Assign), "054f5564-12c5-415f-93da-fc890622f35f"},}, new Dictionary<Terminator, ReduceInformation>{ }, "0a9e63ba-4e0d-4c77-8581-7fc661746091") },
		{ "c9a81728-5d86-417f-8eb5-1f8592ffe2a8", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(KeywordType.End), "f758521d-4774-4528-bf36-57ba44f80c2c"}, { new Terminator(DelimiterType.Semicolon), "41bdccbb-3f79-4d74-9b06-5a48b781a2c2"},}, new Dictionary<Terminator, ReduceInformation>{ }, "c9a81728-5d86-417f-8eb5-1f8592ffe2a8") },
		{ "10aad22b-4cdf-43d3-806c-010b4743b507", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.Semicolon), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Subprogram))}, }, "10aad22b-4cdf-43d3-806c-010b4743b507") },
		{ "8f9dff69-539c-488d-a4bd-527b6798ac27", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.VarDeclarations), "f55f44f9-867a-45a5-85c7-e032d9310084"}, { new Terminator(KeywordType.Var), "ad2c8046-f4a8-4148-907b-90b9ae3d5720"},}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.Begin), new ReduceInformation(0, new NonTerminator(NonTerminatorType.VarDeclarations))}, }, "8f9dff69-539c-488d-a4bd-527b6798ac27") },
		{ "72afbb5c-a7fc-4979-9495-90a0a736c705", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.ConstDeclaration), "945dbf84-e151-47ce-a3d3-f6d2b0942e46"}, { Terminator.IdentifierTerminator, "bf1692f6-9e37-4ad3-a5bf-48e0657ab9a0"},}, new Dictionary<Terminator, ReduceInformation>{ }, "72afbb5c-a7fc-4979-9495-90a0a736c705") },
		{ "deab9d75-c09a-421f-b37b-22d5434ef1ca", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.Semicolon), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SubprogramHead))}, }, "deab9d75-c09a-421f-b37b-22d5434ef1ca") },
		{ "6b68a8b9-a28e-4b48-8e23-a6817d5e7b00", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.ParameterList), "228ea2cd-dcd1-4cc4-9e60-a9e8a5973e75"}, { new NonTerminator(NonTerminatorType.Parameter), "ac87171c-34b2-465b-91d8-68d97e9befec"}, { new NonTerminator(NonTerminatorType.VarParameter), "d610ed1d-74c4-4cdb-9cf2-48380c72c98d"}, { new NonTerminator(NonTerminatorType.ValueParameter), "aa35680f-188d-43a6-b147-3fc855c4a46e"}, { new Terminator(KeywordType.Var), "0375c645-e2b3-4040-bde4-504155d42326"}, { new NonTerminator(NonTerminatorType.IdentifierList), "aa9a1909-68d6-41a2-a737-f3cd97f97b1e"}, { Terminator.IdentifierTerminator, "c50004c4-fac8-4812-a2e2-6467f2005495"},}, new Dictionary<Terminator, ReduceInformation>{ }, "6b68a8b9-a28e-4b48-8e23-a6817d5e7b00") },
		{ "f6c54cdd-7e5a-4714-950b-2374191c9aad", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.Colon), "8d27c6b3-8c45-49f5-90f6-6d7ca86204ee"},}, new Dictionary<Terminator, ReduceInformation>{ }, "f6c54cdd-7e5a-4714-950b-2374191c9aad") },
		{ "6f4c1ec2-4545-4575-a956-8ba95a98dddc", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.ParameterList), "5b8f927a-5497-434a-ba5d-afabbd00fd19"}, { new NonTerminator(NonTerminatorType.Parameter), "ac87171c-34b2-465b-91d8-68d97e9befec"}, { new NonTerminator(NonTerminatorType.VarParameter), "d610ed1d-74c4-4cdb-9cf2-48380c72c98d"}, { new NonTerminator(NonTerminatorType.ValueParameter), "aa35680f-188d-43a6-b147-3fc855c4a46e"}, { new Terminator(KeywordType.Var), "0375c645-e2b3-4040-bde4-504155d42326"}, { new NonTerminator(NonTerminatorType.IdentifierList), "aa9a1909-68d6-41a2-a737-f3cd97f97b1e"}, { Terminator.IdentifierTerminator, "c50004c4-fac8-4812-a2e2-6467f2005495"},}, new Dictionary<Terminator, ReduceInformation>{ }, "6f4c1ec2-4545-4575-a956-8ba95a98dddc") },
		{ "651b45b5-da7d-4f35-a03c-afd82195fb68", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Type), "ef2f6f7a-6bb2-4379-9e30-d82a840ae22d"}, { new NonTerminator(NonTerminatorType.BasicType), "3c45f039-db62-46ee-a41d-23dacfb8d0f0"}, { new Terminator(KeywordType.Array), "07f694b1-c082-40f3-8ea0-c6eda8e73b83"}, { new Terminator(KeywordType.Integer), "7bb70cf0-2067-4971-ada1-ccf973097e8b"}, { new Terminator(KeywordType.Real), "1023abfc-7be1-4c20-890e-3ff30b2c89fc"}, { new Terminator(KeywordType.Boolean), "7430a84c-7617-405a-9992-d202dcbc8adc"}, { new Terminator(KeywordType.Character), "c1bdaca1-017b-4856-84d6-4cd20b5911a3"},}, new Dictionary<Terminator, ReduceInformation>{ }, "651b45b5-da7d-4f35-a03c-afd82195fb68") },
		{ "92206523-e417-495f-94aa-812bc43665f3", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Period), "4599a572-dd97-46c6-9c12-feb2224ee0a7"}, { Terminator.NumberTerminator, "fdbf1b7d-4afc-4ef5-ba8b-2a174df24c40"},}, new Dictionary<Terminator, ReduceInformation>{ }, "92206523-e417-495f-94aa-812bc43665f3") },
		{ "035c7652-7a28-43f7-ba44-1031844761e2", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.Semicolon), new ReduceInformation(5, new NonTerminator(NonTerminatorType.ConstDeclaration))}, }, "035c7652-7a28-43f7-ba44-1031844761e2") },
		{ "33d07881-003f-4441-ace4-327f287a6dde", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(3, new NonTerminator(NonTerminatorType.StatementList))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(3, new NonTerminator(NonTerminatorType.StatementList))}, }, "33d07881-003f-4441-ace4-327f287a6dde") },
		{ "c7ad577c-c184-4e56-90b3-ca6a2ea19d19", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Statement))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Statement))}, }, "c7ad577c-c184-4e56-90b3-ca6a2ea19d19") },
		{ "64715301-bb45-40e5-9af2-b421dc9e1b8c", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.RelationOperator), "57fa87e6-c9d4-47d0-b838-2294a07701be"}, { new Terminator(OperatorType.Equal), "1b49be29-a9d8-403a-b434-5c116300d15c"}, { new Terminator(OperatorType.NotEqual), "fb7de677-df97-4431-b5b7-086a8ded26ff"}, { new Terminator(OperatorType.Less), "8db4bd06-8fe0-4476-924a-3354772a5ac3"}, { new Terminator(OperatorType.LessEqual), "68ce8cad-2c90-4f71-822f-885e193cc394"}, { new Terminator(OperatorType.Greater), "f863fa1a-f90f-430f-8ee8-4cb33053ee45"}, { new Terminator(OperatorType.GreaterEqual), "d79c0f7f-9997-421d-bed4-1e0b2517774c"}, { new NonTerminator(NonTerminatorType.AddOperator), "e85467ab-5ef0-408e-9322-b3f27a0bb07e"}, { new Terminator(OperatorType.Plus), "b2a223d6-6905-4b72-a67b-96d8604fe688"}, { new Terminator(OperatorType.Minus), "569b7d42-5c9a-4ba6-b124-8dacd7f387f4"}, { new Terminator(KeywordType.Or), "37aaac08-c0a9-420d-bfd1-1ad8615a7513"},}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Expression))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Expression))}, }, "64715301-bb45-40e5-9af2-b421dc9e1b8c") },
		{ "47e35174-231c-4a16-9387-25615446322b", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.MultiplyOperator), "8c35e188-2fab-48b5-a1f4-538ee6d2a5bf"}, { new Terminator(OperatorType.Multiply), "e576c109-5246-4139-8248-415b8e3e68de"}, { new Terminator(OperatorType.Divide), "6a567f32-04a0-49b8-bf8e-42f55ee0600f"}, { new Terminator(KeywordType.Divide), "86be9c46-a085-4183-9264-b12d73fde46e"}, { new Terminator(KeywordType.Mod), "d7708b97-7da8-492a-a64f-6196ae9527f7"}, { new Terminator(KeywordType.And), "271d9f46-4be0-495c-a90b-ae8f19ee928b"},}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Equal), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Less), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Greater), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Plus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Minus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(KeywordType.Or), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, }, "47e35174-231c-4a16-9387-25615446322b") },
		{ "7a25b1d8-c432-47b5-b5d6-4b9d4ec99976", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Equal), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Less), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Greater), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Mod), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.And), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Plus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Minus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Or), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, }, "7a25b1d8-c432-47b5-b5d6-4b9d4ec99976") },
		{ "51e160d6-1614-4bea-878b-b9200a78d134", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Equal), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Less), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Greater), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, }, "51e160d6-1614-4bea-878b-b9200a78d134") },
		{ "3ef7029f-efcc-4dfc-bbef-9de3a66266b7", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Equal), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Less), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Greater), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, }, "3ef7029f-efcc-4dfc-bbef-9de3a66266b7") },
		{ "271a6714-8230-4423-9ba9-b983289c29e4", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Expression), "4d73edab-fe7c-48d4-a431-8f2315f61cfe"}, { new NonTerminator(NonTerminatorType.SimpleExpression), "e566a9ad-f17c-46ff-981f-ee4fa81d2d3d"}, { new NonTerminator(NonTerminatorType.Term), "1f1e4a05-7d30-44f4-aa5f-5caacfbce4b2"}, { new NonTerminator(NonTerminatorType.Factor), "ff72d68f-ac20-46e8-94ed-090e841128c1"}, { Terminator.NumberTerminator, "79939c29-990e-4743-8a8a-5fccd2ec415d"}, { new NonTerminator(NonTerminatorType.Variable), "28f501cb-0b97-40c3-8a64-d214832a6789"}, { new Terminator(DelimiterType.LeftParenthesis), "f1464d64-129c-4d68-a61e-43296e963ee1"}, { Terminator.IdentifierTerminator, "9dd45908-639c-4e69-97e4-2471cdf5e189"}, { new Terminator(KeywordType.Not), "303776ee-584f-4977-b76a-9449d6a4e1ad"}, { new Terminator(OperatorType.Minus), "2401b03b-4e43-4409-989c-a4471708a795"},}, new Dictionary<Terminator, ReduceInformation>{ }, "271a6714-8230-4423-9ba9-b983289c29e4") },
		{ "a21cd4d2-38d8-44d9-a8cc-f43a72534d65", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.LeftParenthesis), "d9202fc2-c54d-4a37-8ece-deab98ca67f7"}, { new NonTerminator(NonTerminatorType.IdVarPart), "2acc352f-a616-4da2-95de-b2e571c47ace"}, { new Terminator(DelimiterType.LeftSquareBracket), "71f2a7de-322f-4c2c-99f6-1ab754fe6cfa"},}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Equal), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Less), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Greater), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Divide), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Divide), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Mod), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.And), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Plus), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Minus), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Or), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, }, "a21cd4d2-38d8-44d9-a8cc-f43a72534d65") },
		{ "c4efdd9d-271b-4565-84a7-a0f9bae205fe", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Factor), "75201f13-168d-46bc-bf6e-c2bae2776382"}, { Terminator.NumberTerminator, "51e160d6-1614-4bea-878b-b9200a78d134"}, { new NonTerminator(NonTerminatorType.Variable), "3ef7029f-efcc-4dfc-bbef-9de3a66266b7"}, { new Terminator(DelimiterType.LeftParenthesis), "271a6714-8230-4423-9ba9-b983289c29e4"}, { Terminator.IdentifierTerminator, "a21cd4d2-38d8-44d9-a8cc-f43a72534d65"}, { new Terminator(KeywordType.Not), "c4efdd9d-271b-4565-84a7-a0f9bae205fe"}, { new Terminator(OperatorType.Minus), "9ecf3833-38d5-4e56-84a4-74c622293d59"},}, new Dictionary<Terminator, ReduceInformation>{ }, "c4efdd9d-271b-4565-84a7-a0f9bae205fe") },
		{ "9ecf3833-38d5-4e56-84a4-74c622293d59", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Factor), "c1ae23a9-136a-4b7e-905d-16545bbb0514"}, { Terminator.NumberTerminator, "51e160d6-1614-4bea-878b-b9200a78d134"}, { new NonTerminator(NonTerminatorType.Variable), "3ef7029f-efcc-4dfc-bbef-9de3a66266b7"}, { new Terminator(DelimiterType.LeftParenthesis), "271a6714-8230-4423-9ba9-b983289c29e4"}, { Terminator.IdentifierTerminator, "a21cd4d2-38d8-44d9-a8cc-f43a72534d65"}, { new Terminator(KeywordType.Not), "c4efdd9d-271b-4565-84a7-a0f9bae205fe"}, { new Terminator(OperatorType.Minus), "9ecf3833-38d5-4e56-84a4-74c622293d59"},}, new Dictionary<Terminator, ReduceInformation>{ }, "9ecf3833-38d5-4e56-84a4-74c622293d59") },
		{ "82678949-b5e6-4d69-85b7-9b77982dc2ef", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Statement))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Statement))}, }, "82678949-b5e6-4d69-85b7-9b77982dc2ef") },
		{ "84afed73-d773-4a6a-931e-2b6448c2dca3", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.RightSquareBracket), "0aca3c7f-d427-4a07-9482-6be4a300ff1c"}, { new Terminator(DelimiterType.Comma), "082dc96a-aa2d-444e-b2b0-8a6bca7f6ba2"},}, new Dictionary<Terminator, ReduceInformation>{ }, "84afed73-d773-4a6a-931e-2b6448c2dca3") },
		{ "a0ba5247-cbd6-46f6-b1fd-574979eb2b27", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightSquareBracket), new ReduceInformation(1, new NonTerminator(NonTerminatorType.ExpressionList))}, { new Terminator(DelimiterType.Comma), new ReduceInformation(1, new NonTerminator(NonTerminatorType.ExpressionList))}, }, "a0ba5247-cbd6-46f6-b1fd-574979eb2b27") },
		{ "e8b0426e-8437-45f8-a02f-3eebdfb48e84", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.RelationOperator), "bb12941d-5825-4f3f-a62a-8c6682ff512e"}, { new Terminator(OperatorType.Equal), "1b49be29-a9d8-403a-b434-5c116300d15c"}, { new Terminator(OperatorType.NotEqual), "fb7de677-df97-4431-b5b7-086a8ded26ff"}, { new Terminator(OperatorType.Less), "8db4bd06-8fe0-4476-924a-3354772a5ac3"}, { new Terminator(OperatorType.LessEqual), "68ce8cad-2c90-4f71-822f-885e193cc394"}, { new Terminator(OperatorType.Greater), "f863fa1a-f90f-430f-8ee8-4cb33053ee45"}, { new Terminator(OperatorType.GreaterEqual), "d79c0f7f-9997-421d-bed4-1e0b2517774c"}, { new NonTerminator(NonTerminatorType.AddOperator), "4c209434-ee62-4c84-9b2f-4f4007bc4c3a"}, { new Terminator(OperatorType.Plus), "b2a223d6-6905-4b72-a67b-96d8604fe688"}, { new Terminator(OperatorType.Minus), "569b7d42-5c9a-4ba6-b124-8dacd7f387f4"}, { new Terminator(KeywordType.Or), "37aaac08-c0a9-420d-bfd1-1ad8615a7513"},}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightSquareBracket), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Expression))}, { new Terminator(DelimiterType.Comma), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Expression))}, }, "e8b0426e-8437-45f8-a02f-3eebdfb48e84") },
		{ "cd4bfe7a-5a2e-41f1-a891-6359311b9403", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.MultiplyOperator), "2941ea9b-6c72-4779-8d5b-54e620856301"}, { new Terminator(OperatorType.Multiply), "e576c109-5246-4139-8248-415b8e3e68de"}, { new Terminator(OperatorType.Divide), "6a567f32-04a0-49b8-bf8e-42f55ee0600f"}, { new Terminator(KeywordType.Divide), "86be9c46-a085-4183-9264-b12d73fde46e"}, { new Terminator(KeywordType.Mod), "d7708b97-7da8-492a-a64f-6196ae9527f7"}, { new Terminator(KeywordType.And), "271d9f46-4be0-495c-a90b-ae8f19ee928b"},}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightSquareBracket), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Equal), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Less), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Greater), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Plus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Minus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(KeywordType.Or), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(DelimiterType.Comma), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, }, "cd4bfe7a-5a2e-41f1-a891-6359311b9403") },
		{ "ceb0b15a-7ff9-4265-8766-716485412510", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightSquareBracket), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Equal), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Less), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Greater), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Mod), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.And), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Plus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Minus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Or), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(DelimiterType.Comma), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, }, "ceb0b15a-7ff9-4265-8766-716485412510") },
		{ "06d206a2-8c81-4399-8790-c903a8acfe21", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightSquareBracket), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Equal), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Less), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Greater), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(DelimiterType.Comma), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, }, "06d206a2-8c81-4399-8790-c903a8acfe21") },
		{ "422866b2-29d6-4004-b762-cb9ad9ddc5e7", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightSquareBracket), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Equal), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Less), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Greater), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(DelimiterType.Comma), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, }, "422866b2-29d6-4004-b762-cb9ad9ddc5e7") },
		{ "4f42a0a3-3c4b-46a5-8dcc-b91007f93d5c", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Expression), "35b2a70a-6d3e-4b34-aa2d-477ff691c677"}, { new NonTerminator(NonTerminatorType.SimpleExpression), "e566a9ad-f17c-46ff-981f-ee4fa81d2d3d"}, { new NonTerminator(NonTerminatorType.Term), "1f1e4a05-7d30-44f4-aa5f-5caacfbce4b2"}, { new NonTerminator(NonTerminatorType.Factor), "ff72d68f-ac20-46e8-94ed-090e841128c1"}, { Terminator.NumberTerminator, "79939c29-990e-4743-8a8a-5fccd2ec415d"}, { new NonTerminator(NonTerminatorType.Variable), "28f501cb-0b97-40c3-8a64-d214832a6789"}, { new Terminator(DelimiterType.LeftParenthesis), "f1464d64-129c-4d68-a61e-43296e963ee1"}, { Terminator.IdentifierTerminator, "9dd45908-639c-4e69-97e4-2471cdf5e189"}, { new Terminator(KeywordType.Not), "303776ee-584f-4977-b76a-9449d6a4e1ad"}, { new Terminator(OperatorType.Minus), "2401b03b-4e43-4409-989c-a4471708a795"},}, new Dictionary<Terminator, ReduceInformation>{ }, "4f42a0a3-3c4b-46a5-8dcc-b91007f93d5c") },
		{ "7339bcec-b0ae-4611-83cd-08345e44324f", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.LeftParenthesis), "13ad14c5-3184-4246-892c-e5c8b1aa8ab0"}, { new NonTerminator(NonTerminatorType.IdVarPart), "3d2d3751-03bf-4a08-88e2-2ac7aa664671"}, { new Terminator(DelimiterType.LeftSquareBracket), "c06874e3-04d1-4be1-ac5d-f77ab188ede7"},}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightSquareBracket), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Equal), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Less), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Greater), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Divide), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Divide), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Mod), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.And), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Plus), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Minus), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Or), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(DelimiterType.Comma), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, }, "7339bcec-b0ae-4611-83cd-08345e44324f") },
		{ "bfdaacfe-9460-4db8-9469-712b6674efb5", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Factor), "871776e4-b741-4664-a3d1-c82844ec1a17"}, { Terminator.NumberTerminator, "06d206a2-8c81-4399-8790-c903a8acfe21"}, { new NonTerminator(NonTerminatorType.Variable), "422866b2-29d6-4004-b762-cb9ad9ddc5e7"}, { new Terminator(DelimiterType.LeftParenthesis), "4f42a0a3-3c4b-46a5-8dcc-b91007f93d5c"}, { Terminator.IdentifierTerminator, "7339bcec-b0ae-4611-83cd-08345e44324f"}, { new Terminator(KeywordType.Not), "bfdaacfe-9460-4db8-9469-712b6674efb5"}, { new Terminator(OperatorType.Minus), "eac356e4-6098-43be-9b50-3ff1a637a898"},}, new Dictionary<Terminator, ReduceInformation>{ }, "bfdaacfe-9460-4db8-9469-712b6674efb5") },
		{ "eac356e4-6098-43be-9b50-3ff1a637a898", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Factor), "af4bc17c-8325-48dc-b52b-92f8d164c0d7"}, { Terminator.NumberTerminator, "06d206a2-8c81-4399-8790-c903a8acfe21"}, { new NonTerminator(NonTerminatorType.Variable), "422866b2-29d6-4004-b762-cb9ad9ddc5e7"}, { new Terminator(DelimiterType.LeftParenthesis), "4f42a0a3-3c4b-46a5-8dcc-b91007f93d5c"}, { Terminator.IdentifierTerminator, "7339bcec-b0ae-4611-83cd-08345e44324f"}, { new Terminator(KeywordType.Not), "bfdaacfe-9460-4db8-9469-712b6674efb5"}, { new Terminator(OperatorType.Minus), "eac356e4-6098-43be-9b50-3ff1a637a898"},}, new Dictionary<Terminator, ReduceInformation>{ }, "eac356e4-6098-43be-9b50-3ff1a637a898") },
		{ "171d1827-4aaf-4774-bb69-b43842cda52a", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.RightParenthesis), "bb8ace84-3c8a-497f-83e3-979ed4529545"}, { new Terminator(DelimiterType.Comma), "077bd83e-2dfc-4686-bf99-ec6992c05b60"},}, new Dictionary<Terminator, ReduceInformation>{ }, "171d1827-4aaf-4774-bb69-b43842cda52a") },
		{ "4f59631a-5553-49da-9f0d-dda48b0d65b2", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightParenthesis), new ReduceInformation(1, new NonTerminator(NonTerminatorType.ExpressionList))}, { new Terminator(DelimiterType.Comma), new ReduceInformation(1, new NonTerminator(NonTerminatorType.ExpressionList))}, }, "4f59631a-5553-49da-9f0d-dda48b0d65b2") },
		{ "e2a2e2c4-61e9-496f-8a60-ac108404247f", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.RelationOperator), "5b323743-f503-403e-be13-7cd5cdb76bd8"}, { new Terminator(OperatorType.Equal), "1b49be29-a9d8-403a-b434-5c116300d15c"}, { new Terminator(OperatorType.NotEqual), "fb7de677-df97-4431-b5b7-086a8ded26ff"}, { new Terminator(OperatorType.Less), "8db4bd06-8fe0-4476-924a-3354772a5ac3"}, { new Terminator(OperatorType.LessEqual), "68ce8cad-2c90-4f71-822f-885e193cc394"}, { new Terminator(OperatorType.Greater), "f863fa1a-f90f-430f-8ee8-4cb33053ee45"}, { new Terminator(OperatorType.GreaterEqual), "d79c0f7f-9997-421d-bed4-1e0b2517774c"}, { new NonTerminator(NonTerminatorType.AddOperator), "38426e6b-4d67-4d75-b310-309e6ea86212"}, { new Terminator(OperatorType.Plus), "b2a223d6-6905-4b72-a67b-96d8604fe688"}, { new Terminator(OperatorType.Minus), "569b7d42-5c9a-4ba6-b124-8dacd7f387f4"}, { new Terminator(KeywordType.Or), "37aaac08-c0a9-420d-bfd1-1ad8615a7513"},}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightParenthesis), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Expression))}, { new Terminator(DelimiterType.Comma), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Expression))}, }, "e2a2e2c4-61e9-496f-8a60-ac108404247f") },
		{ "83d0456c-4817-49a5-bd20-c0d8a1c5b696", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.MultiplyOperator), "37d17fe6-f232-4c2e-8d64-2ffb041dd55d"}, { new Terminator(OperatorType.Multiply), "e576c109-5246-4139-8248-415b8e3e68de"}, { new Terminator(OperatorType.Divide), "6a567f32-04a0-49b8-bf8e-42f55ee0600f"}, { new Terminator(KeywordType.Divide), "86be9c46-a085-4183-9264-b12d73fde46e"}, { new Terminator(KeywordType.Mod), "d7708b97-7da8-492a-a64f-6196ae9527f7"}, { new Terminator(KeywordType.And), "271d9f46-4be0-495c-a90b-ae8f19ee928b"},}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightParenthesis), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Equal), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Less), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Greater), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Plus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Minus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(KeywordType.Or), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(DelimiterType.Comma), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, }, "83d0456c-4817-49a5-bd20-c0d8a1c5b696") },
		{ "f1db3e46-b446-4adc-93ae-5dfed0964c99", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightParenthesis), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Equal), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Less), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Greater), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Mod), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.And), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Plus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Minus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Or), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(DelimiterType.Comma), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, }, "f1db3e46-b446-4adc-93ae-5dfed0964c99") },
		{ "0abfb7c0-083d-4242-a9a7-06183f7230f7", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightParenthesis), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Equal), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Less), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Greater), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(DelimiterType.Comma), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, }, "0abfb7c0-083d-4242-a9a7-06183f7230f7") },
		{ "a8c27dec-213a-4917-8b05-b0422260d1c1", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightParenthesis), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Equal), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Less), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Greater), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(DelimiterType.Comma), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, }, "a8c27dec-213a-4917-8b05-b0422260d1c1") },
		{ "45c4069f-f3cf-48f7-af29-8eb8303c47d6", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Expression), "7eebe5e3-6af0-439d-977f-db6e51804ba1"}, { new NonTerminator(NonTerminatorType.SimpleExpression), "e566a9ad-f17c-46ff-981f-ee4fa81d2d3d"}, { new NonTerminator(NonTerminatorType.Term), "1f1e4a05-7d30-44f4-aa5f-5caacfbce4b2"}, { new NonTerminator(NonTerminatorType.Factor), "ff72d68f-ac20-46e8-94ed-090e841128c1"}, { Terminator.NumberTerminator, "79939c29-990e-4743-8a8a-5fccd2ec415d"}, { new NonTerminator(NonTerminatorType.Variable), "28f501cb-0b97-40c3-8a64-d214832a6789"}, { new Terminator(DelimiterType.LeftParenthesis), "f1464d64-129c-4d68-a61e-43296e963ee1"}, { Terminator.IdentifierTerminator, "9dd45908-639c-4e69-97e4-2471cdf5e189"}, { new Terminator(KeywordType.Not), "303776ee-584f-4977-b76a-9449d6a4e1ad"}, { new Terminator(OperatorType.Minus), "2401b03b-4e43-4409-989c-a4471708a795"},}, new Dictionary<Terminator, ReduceInformation>{ }, "45c4069f-f3cf-48f7-af29-8eb8303c47d6") },
		{ "5aabfbf6-09f5-484f-9f95-ac0fa52c26c4", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.LeftParenthesis), "48d6264a-4895-469d-8a3d-303a6be14a00"}, { new NonTerminator(NonTerminatorType.IdVarPart), "0ba414d6-6370-42f8-a088-5f7b8e9fd42b"}, { new Terminator(DelimiterType.LeftSquareBracket), "315084fc-3051-4edf-b8f8-4955459fecff"},}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightParenthesis), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Equal), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Less), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Greater), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Divide), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Divide), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Mod), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.And), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Plus), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Minus), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Or), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(DelimiterType.Comma), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, }, "5aabfbf6-09f5-484f-9f95-ac0fa52c26c4") },
		{ "253ff382-06bc-47a7-9645-cc04c48e1d0a", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Factor), "70ee78ef-1325-49e3-bba6-b20c44383937"}, { Terminator.NumberTerminator, "0abfb7c0-083d-4242-a9a7-06183f7230f7"}, { new NonTerminator(NonTerminatorType.Variable), "a8c27dec-213a-4917-8b05-b0422260d1c1"}, { new Terminator(DelimiterType.LeftParenthesis), "45c4069f-f3cf-48f7-af29-8eb8303c47d6"}, { Terminator.IdentifierTerminator, "5aabfbf6-09f5-484f-9f95-ac0fa52c26c4"}, { new Terminator(KeywordType.Not), "253ff382-06bc-47a7-9645-cc04c48e1d0a"}, { new Terminator(OperatorType.Minus), "5708e399-73e5-4bcb-9aeb-f25d1e06c35c"},}, new Dictionary<Terminator, ReduceInformation>{ }, "253ff382-06bc-47a7-9645-cc04c48e1d0a") },
		{ "5708e399-73e5-4bcb-9aeb-f25d1e06c35c", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Factor), "649e70ee-496e-40c8-b504-178b9a7d99ef"}, { Terminator.NumberTerminator, "0abfb7c0-083d-4242-a9a7-06183f7230f7"}, { new NonTerminator(NonTerminatorType.Variable), "a8c27dec-213a-4917-8b05-b0422260d1c1"}, { new Terminator(DelimiterType.LeftParenthesis), "45c4069f-f3cf-48f7-af29-8eb8303c47d6"}, { Terminator.IdentifierTerminator, "5aabfbf6-09f5-484f-9f95-ac0fa52c26c4"}, { new Terminator(KeywordType.Not), "253ff382-06bc-47a7-9645-cc04c48e1d0a"}, { new Terminator(OperatorType.Minus), "5708e399-73e5-4bcb-9aeb-f25d1e06c35c"},}, new Dictionary<Terminator, ReduceInformation>{ }, "5708e399-73e5-4bcb-9aeb-f25d1e06c35c") },
		{ "36034f1f-1d4c-4218-9176-291bc5da5fc3", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Statement), "ea81cccb-cba2-4edd-ad23-836b2c152424"}, { new NonTerminator(NonTerminatorType.Variable), "f7849288-2cd0-4c68-aeee-abb4f910fa32"}, { Terminator.IdentifierTerminator, "403840ce-3751-4324-a4da-a2b0a7653957"}, { new NonTerminator(NonTerminatorType.ProcedureCall), "504c43d8-8335-43de-8838-5c3205046f5d"}, { new NonTerminator(NonTerminatorType.CompoundStatement), "15b27fe6-dc50-4e57-8246-bc2e8a5eb05a"}, { new Terminator(KeywordType.If), "1e290bf0-8ecb-4a2d-b5dc-bd2aa7ff6505"}, { new Terminator(KeywordType.For), "83bfcb23-a73e-4ba5-ac65-68e5a019cc6a"}, { new Terminator(KeywordType.Begin), "1fd17dc7-14a7-4500-8bc1-e768a6969eae"},}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(0, new NonTerminator(NonTerminatorType.Statement))}, { new Terminator(KeywordType.Else), new ReduceInformation(0, new NonTerminator(NonTerminatorType.Statement))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(0, new NonTerminator(NonTerminatorType.Statement))}, }, "36034f1f-1d4c-4218-9176-291bc5da5fc3") },
		{ "6a445aae-de55-4ebc-8144-36b6def60bd6", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.SimpleExpression), "9f283a60-47f9-425f-9e6e-1d4df9415f95"}, { new NonTerminator(NonTerminatorType.Term), "70c7c6ca-23db-4dc4-b18f-b7fec5c12330"}, { new NonTerminator(NonTerminatorType.Factor), "4c6ca0be-90cb-4569-8441-0ffc0c0a22a4"}, { Terminator.NumberTerminator, "f9789d84-d480-461d-a4a2-0404654a02e1"}, { new NonTerminator(NonTerminatorType.Variable), "4056346c-1389-4501-9fa7-8d71820cf943"}, { new Terminator(DelimiterType.LeftParenthesis), "141303a8-ff5b-48d6-9e48-10252b617d09"}, { Terminator.IdentifierTerminator, "5b888b7a-23af-4ebe-a292-bafb94fabb7b"}, { new Terminator(KeywordType.Not), "df7787b9-6d11-4bb2-b89d-3527a0e1b4fb"}, { new Terminator(OperatorType.Minus), "3404318e-e4c9-45b1-9569-32aeb85c2397"},}, new Dictionary<Terminator, ReduceInformation>{ }, "6a445aae-de55-4ebc-8144-36b6def60bd6") },
		{ "1b49be29-a9d8-403a-b434-5c116300d15c", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { Terminator.NumberTerminator, new ReduceInformation(1, new NonTerminator(NonTerminatorType.RelationOperator))}, { Terminator.IdentifierTerminator, new ReduceInformation(1, new NonTerminator(NonTerminatorType.RelationOperator))}, { new Terminator(DelimiterType.LeftParenthesis), new ReduceInformation(1, new NonTerminator(NonTerminatorType.RelationOperator))}, { new Terminator(KeywordType.Not), new ReduceInformation(1, new NonTerminator(NonTerminatorType.RelationOperator))}, { new Terminator(OperatorType.Minus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.RelationOperator))}, }, "1b49be29-a9d8-403a-b434-5c116300d15c") },
		{ "fb7de677-df97-4431-b5b7-086a8ded26ff", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { Terminator.NumberTerminator, new ReduceInformation(1, new NonTerminator(NonTerminatorType.RelationOperator))}, { Terminator.IdentifierTerminator, new ReduceInformation(1, new NonTerminator(NonTerminatorType.RelationOperator))}, { new Terminator(DelimiterType.LeftParenthesis), new ReduceInformation(1, new NonTerminator(NonTerminatorType.RelationOperator))}, { new Terminator(KeywordType.Not), new ReduceInformation(1, new NonTerminator(NonTerminatorType.RelationOperator))}, { new Terminator(OperatorType.Minus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.RelationOperator))}, }, "fb7de677-df97-4431-b5b7-086a8ded26ff") },
		{ "8db4bd06-8fe0-4476-924a-3354772a5ac3", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { Terminator.NumberTerminator, new ReduceInformation(1, new NonTerminator(NonTerminatorType.RelationOperator))}, { Terminator.IdentifierTerminator, new ReduceInformation(1, new NonTerminator(NonTerminatorType.RelationOperator))}, { new Terminator(DelimiterType.LeftParenthesis), new ReduceInformation(1, new NonTerminator(NonTerminatorType.RelationOperator))}, { new Terminator(KeywordType.Not), new ReduceInformation(1, new NonTerminator(NonTerminatorType.RelationOperator))}, { new Terminator(OperatorType.Minus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.RelationOperator))}, }, "8db4bd06-8fe0-4476-924a-3354772a5ac3") },
		{ "68ce8cad-2c90-4f71-822f-885e193cc394", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { Terminator.NumberTerminator, new ReduceInformation(1, new NonTerminator(NonTerminatorType.RelationOperator))}, { Terminator.IdentifierTerminator, new ReduceInformation(1, new NonTerminator(NonTerminatorType.RelationOperator))}, { new Terminator(DelimiterType.LeftParenthesis), new ReduceInformation(1, new NonTerminator(NonTerminatorType.RelationOperator))}, { new Terminator(KeywordType.Not), new ReduceInformation(1, new NonTerminator(NonTerminatorType.RelationOperator))}, { new Terminator(OperatorType.Minus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.RelationOperator))}, }, "68ce8cad-2c90-4f71-822f-885e193cc394") },
		{ "f863fa1a-f90f-430f-8ee8-4cb33053ee45", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { Terminator.NumberTerminator, new ReduceInformation(1, new NonTerminator(NonTerminatorType.RelationOperator))}, { Terminator.IdentifierTerminator, new ReduceInformation(1, new NonTerminator(NonTerminatorType.RelationOperator))}, { new Terminator(DelimiterType.LeftParenthesis), new ReduceInformation(1, new NonTerminator(NonTerminatorType.RelationOperator))}, { new Terminator(KeywordType.Not), new ReduceInformation(1, new NonTerminator(NonTerminatorType.RelationOperator))}, { new Terminator(OperatorType.Minus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.RelationOperator))}, }, "f863fa1a-f90f-430f-8ee8-4cb33053ee45") },
		{ "d79c0f7f-9997-421d-bed4-1e0b2517774c", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { Terminator.NumberTerminator, new ReduceInformation(1, new NonTerminator(NonTerminatorType.RelationOperator))}, { Terminator.IdentifierTerminator, new ReduceInformation(1, new NonTerminator(NonTerminatorType.RelationOperator))}, { new Terminator(DelimiterType.LeftParenthesis), new ReduceInformation(1, new NonTerminator(NonTerminatorType.RelationOperator))}, { new Terminator(KeywordType.Not), new ReduceInformation(1, new NonTerminator(NonTerminatorType.RelationOperator))}, { new Terminator(OperatorType.Minus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.RelationOperator))}, }, "d79c0f7f-9997-421d-bed4-1e0b2517774c") },
		{ "2122fdcb-bbda-4be2-9f58-e4f3c12ea687", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Term), "8b8539ff-6425-420b-bc8a-71286ffe917d"}, { new NonTerminator(NonTerminatorType.Factor), "aa2a69ba-536a-4716-9726-fd4bca6a2ecb"}, { Terminator.NumberTerminator, "f2e280ec-cc53-4dd5-9e78-4d96496b2e43"}, { new NonTerminator(NonTerminatorType.Variable), "61692e3e-df15-4f2e-bbda-0e43f17752df"}, { new Terminator(DelimiterType.LeftParenthesis), "77ef6b8f-5b0a-42d4-9397-9bee38e59991"}, { Terminator.IdentifierTerminator, "43a0e8af-43b0-436c-bf72-119f9adb1db1"}, { new Terminator(KeywordType.Not), "bf6cd66a-c610-4c98-8ec1-fce070c9ea7a"}, { new Terminator(OperatorType.Minus), "5a485bf7-252f-452b-a035-e99007748568"},}, new Dictionary<Terminator, ReduceInformation>{ }, "2122fdcb-bbda-4be2-9f58-e4f3c12ea687") },
		{ "b2a223d6-6905-4b72-a67b-96d8604fe688", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { Terminator.NumberTerminator, new ReduceInformation(1, new NonTerminator(NonTerminatorType.AddOperator))}, { Terminator.IdentifierTerminator, new ReduceInformation(1, new NonTerminator(NonTerminatorType.AddOperator))}, { new Terminator(DelimiterType.LeftParenthesis), new ReduceInformation(1, new NonTerminator(NonTerminatorType.AddOperator))}, { new Terminator(KeywordType.Not), new ReduceInformation(1, new NonTerminator(NonTerminatorType.AddOperator))}, { new Terminator(OperatorType.Minus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.AddOperator))}, }, "b2a223d6-6905-4b72-a67b-96d8604fe688") },
		{ "569b7d42-5c9a-4ba6-b124-8dacd7f387f4", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { Terminator.NumberTerminator, new ReduceInformation(1, new NonTerminator(NonTerminatorType.AddOperator))}, { Terminator.IdentifierTerminator, new ReduceInformation(1, new NonTerminator(NonTerminatorType.AddOperator))}, { new Terminator(DelimiterType.LeftParenthesis), new ReduceInformation(1, new NonTerminator(NonTerminatorType.AddOperator))}, { new Terminator(KeywordType.Not), new ReduceInformation(1, new NonTerminator(NonTerminatorType.AddOperator))}, { new Terminator(OperatorType.Minus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.AddOperator))}, }, "569b7d42-5c9a-4ba6-b124-8dacd7f387f4") },
		{ "37aaac08-c0a9-420d-bfd1-1ad8615a7513", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { Terminator.NumberTerminator, new ReduceInformation(1, new NonTerminator(NonTerminatorType.AddOperator))}, { Terminator.IdentifierTerminator, new ReduceInformation(1, new NonTerminator(NonTerminatorType.AddOperator))}, { new Terminator(DelimiterType.LeftParenthesis), new ReduceInformation(1, new NonTerminator(NonTerminatorType.AddOperator))}, { new Terminator(KeywordType.Not), new ReduceInformation(1, new NonTerminator(NonTerminatorType.AddOperator))}, { new Terminator(OperatorType.Minus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.AddOperator))}, }, "37aaac08-c0a9-420d-bfd1-1ad8615a7513") },
		{ "4221e2a4-55fa-4449-b6e3-fe531d22ac26", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Factor), "44b4a1fb-43f3-461f-ae64-3b8b736aab56"}, { Terminator.NumberTerminator, "f2e280ec-cc53-4dd5-9e78-4d96496b2e43"}, { new NonTerminator(NonTerminatorType.Variable), "61692e3e-df15-4f2e-bbda-0e43f17752df"}, { new Terminator(DelimiterType.LeftParenthesis), "77ef6b8f-5b0a-42d4-9397-9bee38e59991"}, { Terminator.IdentifierTerminator, "43a0e8af-43b0-436c-bf72-119f9adb1db1"}, { new Terminator(KeywordType.Not), "bf6cd66a-c610-4c98-8ec1-fce070c9ea7a"}, { new Terminator(OperatorType.Minus), "5a485bf7-252f-452b-a035-e99007748568"},}, new Dictionary<Terminator, ReduceInformation>{ }, "4221e2a4-55fa-4449-b6e3-fe531d22ac26") },
		{ "e576c109-5246-4139-8248-415b8e3e68de", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { Terminator.NumberTerminator, new ReduceInformation(1, new NonTerminator(NonTerminatorType.MultiplyOperator))}, { Terminator.IdentifierTerminator, new ReduceInformation(1, new NonTerminator(NonTerminatorType.MultiplyOperator))}, { new Terminator(DelimiterType.LeftParenthesis), new ReduceInformation(1, new NonTerminator(NonTerminatorType.MultiplyOperator))}, { new Terminator(KeywordType.Not), new ReduceInformation(1, new NonTerminator(NonTerminatorType.MultiplyOperator))}, { new Terminator(OperatorType.Minus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.MultiplyOperator))}, }, "e576c109-5246-4139-8248-415b8e3e68de") },
		{ "6a567f32-04a0-49b8-bf8e-42f55ee0600f", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { Terminator.NumberTerminator, new ReduceInformation(1, new NonTerminator(NonTerminatorType.MultiplyOperator))}, { Terminator.IdentifierTerminator, new ReduceInformation(1, new NonTerminator(NonTerminatorType.MultiplyOperator))}, { new Terminator(DelimiterType.LeftParenthesis), new ReduceInformation(1, new NonTerminator(NonTerminatorType.MultiplyOperator))}, { new Terminator(KeywordType.Not), new ReduceInformation(1, new NonTerminator(NonTerminatorType.MultiplyOperator))}, { new Terminator(OperatorType.Minus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.MultiplyOperator))}, }, "6a567f32-04a0-49b8-bf8e-42f55ee0600f") },
		{ "86be9c46-a085-4183-9264-b12d73fde46e", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { Terminator.NumberTerminator, new ReduceInformation(1, new NonTerminator(NonTerminatorType.MultiplyOperator))}, { Terminator.IdentifierTerminator, new ReduceInformation(1, new NonTerminator(NonTerminatorType.MultiplyOperator))}, { new Terminator(DelimiterType.LeftParenthesis), new ReduceInformation(1, new NonTerminator(NonTerminatorType.MultiplyOperator))}, { new Terminator(KeywordType.Not), new ReduceInformation(1, new NonTerminator(NonTerminatorType.MultiplyOperator))}, { new Terminator(OperatorType.Minus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.MultiplyOperator))}, }, "86be9c46-a085-4183-9264-b12d73fde46e") },
		{ "d7708b97-7da8-492a-a64f-6196ae9527f7", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { Terminator.NumberTerminator, new ReduceInformation(1, new NonTerminator(NonTerminatorType.MultiplyOperator))}, { Terminator.IdentifierTerminator, new ReduceInformation(1, new NonTerminator(NonTerminatorType.MultiplyOperator))}, { new Terminator(DelimiterType.LeftParenthesis), new ReduceInformation(1, new NonTerminator(NonTerminatorType.MultiplyOperator))}, { new Terminator(KeywordType.Not), new ReduceInformation(1, new NonTerminator(NonTerminatorType.MultiplyOperator))}, { new Terminator(OperatorType.Minus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.MultiplyOperator))}, }, "d7708b97-7da8-492a-a64f-6196ae9527f7") },
		{ "271d9f46-4be0-495c-a90b-ae8f19ee928b", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { Terminator.NumberTerminator, new ReduceInformation(1, new NonTerminator(NonTerminatorType.MultiplyOperator))}, { Terminator.IdentifierTerminator, new ReduceInformation(1, new NonTerminator(NonTerminatorType.MultiplyOperator))}, { new Terminator(DelimiterType.LeftParenthesis), new ReduceInformation(1, new NonTerminator(NonTerminatorType.MultiplyOperator))}, { new Terminator(KeywordType.Not), new ReduceInformation(1, new NonTerminator(NonTerminatorType.MultiplyOperator))}, { new Terminator(OperatorType.Minus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.MultiplyOperator))}, }, "271d9f46-4be0-495c-a90b-ae8f19ee928b") },
		{ "3f3b4ebb-4308-40b1-9c77-4c7dc69c4ae5", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.RightParenthesis), "a74d5a6c-e2b4-402f-9792-727949fcac7a"},}, new Dictionary<Terminator, ReduceInformation>{ }, "3f3b4ebb-4308-40b1-9c77-4c7dc69c4ae5") },
		{ "e566a9ad-f17c-46ff-981f-ee4fa81d2d3d", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.RelationOperator), "c2b3f5d3-e730-4f15-bb6d-b23725240589"}, { new Terminator(OperatorType.Equal), "1b49be29-a9d8-403a-b434-5c116300d15c"}, { new Terminator(OperatorType.NotEqual), "fb7de677-df97-4431-b5b7-086a8ded26ff"}, { new Terminator(OperatorType.Less), "8db4bd06-8fe0-4476-924a-3354772a5ac3"}, { new Terminator(OperatorType.LessEqual), "68ce8cad-2c90-4f71-822f-885e193cc394"}, { new Terminator(OperatorType.Greater), "f863fa1a-f90f-430f-8ee8-4cb33053ee45"}, { new Terminator(OperatorType.GreaterEqual), "d79c0f7f-9997-421d-bed4-1e0b2517774c"}, { new NonTerminator(NonTerminatorType.AddOperator), "f69a1f56-560a-4942-ab98-ecd516cb5a1a"}, { new Terminator(OperatorType.Plus), "b2a223d6-6905-4b72-a67b-96d8604fe688"}, { new Terminator(OperatorType.Minus), "569b7d42-5c9a-4ba6-b124-8dacd7f387f4"}, { new Terminator(KeywordType.Or), "37aaac08-c0a9-420d-bfd1-1ad8615a7513"},}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightParenthesis), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Expression))}, }, "e566a9ad-f17c-46ff-981f-ee4fa81d2d3d") },
		{ "1f1e4a05-7d30-44f4-aa5f-5caacfbce4b2", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.MultiplyOperator), "d53ff53c-5245-4c95-86c0-b368cdebee74"}, { new Terminator(OperatorType.Multiply), "e576c109-5246-4139-8248-415b8e3e68de"}, { new Terminator(OperatorType.Divide), "6a567f32-04a0-49b8-bf8e-42f55ee0600f"}, { new Terminator(KeywordType.Divide), "86be9c46-a085-4183-9264-b12d73fde46e"}, { new Terminator(KeywordType.Mod), "d7708b97-7da8-492a-a64f-6196ae9527f7"}, { new Terminator(KeywordType.And), "271d9f46-4be0-495c-a90b-ae8f19ee928b"},}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightParenthesis), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Equal), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Less), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Greater), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Plus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Minus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(KeywordType.Or), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, }, "1f1e4a05-7d30-44f4-aa5f-5caacfbce4b2") },
		{ "ff72d68f-ac20-46e8-94ed-090e841128c1", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightParenthesis), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Equal), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Less), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Greater), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Mod), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.And), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Plus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Minus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Or), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, }, "ff72d68f-ac20-46e8-94ed-090e841128c1") },
		{ "79939c29-990e-4743-8a8a-5fccd2ec415d", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightParenthesis), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Equal), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Less), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Greater), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, }, "79939c29-990e-4743-8a8a-5fccd2ec415d") },
		{ "28f501cb-0b97-40c3-8a64-d214832a6789", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightParenthesis), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Equal), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Less), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Greater), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, }, "28f501cb-0b97-40c3-8a64-d214832a6789") },
		{ "f1464d64-129c-4d68-a61e-43296e963ee1", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Expression), "6bdf7e41-6eeb-4d19-afc2-b45f24c683ab"}, { new NonTerminator(NonTerminatorType.SimpleExpression), "e566a9ad-f17c-46ff-981f-ee4fa81d2d3d"}, { new NonTerminator(NonTerminatorType.Term), "1f1e4a05-7d30-44f4-aa5f-5caacfbce4b2"}, { new NonTerminator(NonTerminatorType.Factor), "ff72d68f-ac20-46e8-94ed-090e841128c1"}, { Terminator.NumberTerminator, "79939c29-990e-4743-8a8a-5fccd2ec415d"}, { new NonTerminator(NonTerminatorType.Variable), "28f501cb-0b97-40c3-8a64-d214832a6789"}, { new Terminator(DelimiterType.LeftParenthesis), "f1464d64-129c-4d68-a61e-43296e963ee1"}, { Terminator.IdentifierTerminator, "9dd45908-639c-4e69-97e4-2471cdf5e189"}, { new Terminator(KeywordType.Not), "303776ee-584f-4977-b76a-9449d6a4e1ad"}, { new Terminator(OperatorType.Minus), "2401b03b-4e43-4409-989c-a4471708a795"},}, new Dictionary<Terminator, ReduceInformation>{ }, "f1464d64-129c-4d68-a61e-43296e963ee1") },
		{ "9dd45908-639c-4e69-97e4-2471cdf5e189", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.LeftParenthesis), "22565d79-b3bb-4b85-8ad9-f72e1c88e65f"}, { new NonTerminator(NonTerminatorType.IdVarPart), "64aa0e6e-093b-4546-abcc-aad4534b619b"}, { new Terminator(DelimiterType.LeftSquareBracket), "0f70ecea-4f81-433a-9d4e-9541b90b78e0"},}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightParenthesis), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Equal), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Less), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Greater), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Divide), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Divide), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Mod), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.And), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Plus), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Minus), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Or), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, }, "9dd45908-639c-4e69-97e4-2471cdf5e189") },
		{ "303776ee-584f-4977-b76a-9449d6a4e1ad", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Factor), "ffe1849f-88ee-4687-9f8e-860a984bd772"}, { Terminator.NumberTerminator, "79939c29-990e-4743-8a8a-5fccd2ec415d"}, { new NonTerminator(NonTerminatorType.Variable), "28f501cb-0b97-40c3-8a64-d214832a6789"}, { new Terminator(DelimiterType.LeftParenthesis), "f1464d64-129c-4d68-a61e-43296e963ee1"}, { Terminator.IdentifierTerminator, "9dd45908-639c-4e69-97e4-2471cdf5e189"}, { new Terminator(KeywordType.Not), "303776ee-584f-4977-b76a-9449d6a4e1ad"}, { new Terminator(OperatorType.Minus), "2401b03b-4e43-4409-989c-a4471708a795"},}, new Dictionary<Terminator, ReduceInformation>{ }, "303776ee-584f-4977-b76a-9449d6a4e1ad") },
		{ "2401b03b-4e43-4409-989c-a4471708a795", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Factor), "9e52d3c3-9893-4085-90f9-786048447625"}, { Terminator.NumberTerminator, "79939c29-990e-4743-8a8a-5fccd2ec415d"}, { new NonTerminator(NonTerminatorType.Variable), "28f501cb-0b97-40c3-8a64-d214832a6789"}, { new Terminator(DelimiterType.LeftParenthesis), "f1464d64-129c-4d68-a61e-43296e963ee1"}, { Terminator.IdentifierTerminator, "9dd45908-639c-4e69-97e4-2471cdf5e189"}, { new Terminator(KeywordType.Not), "303776ee-584f-4977-b76a-9449d6a4e1ad"}, { new Terminator(OperatorType.Minus), "2401b03b-4e43-4409-989c-a4471708a795"},}, new Dictionary<Terminator, ReduceInformation>{ }, "2401b03b-4e43-4409-989c-a4471708a795") },
		{ "e8e3778c-2637-497d-8220-1ad1da0361e9", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.ExpressionList), "e881dbac-a57f-488d-8e0a-13868896ba85"}, { new NonTerminator(NonTerminatorType.Expression), "4f59631a-5553-49da-9f0d-dda48b0d65b2"}, { new NonTerminator(NonTerminatorType.SimpleExpression), "e2a2e2c4-61e9-496f-8a60-ac108404247f"}, { new NonTerminator(NonTerminatorType.Term), "83d0456c-4817-49a5-bd20-c0d8a1c5b696"}, { new NonTerminator(NonTerminatorType.Factor), "f1db3e46-b446-4adc-93ae-5dfed0964c99"}, { Terminator.NumberTerminator, "0abfb7c0-083d-4242-a9a7-06183f7230f7"}, { new NonTerminator(NonTerminatorType.Variable), "a8c27dec-213a-4917-8b05-b0422260d1c1"}, { new Terminator(DelimiterType.LeftParenthesis), "45c4069f-f3cf-48f7-af29-8eb8303c47d6"}, { Terminator.IdentifierTerminator, "5aabfbf6-09f5-484f-9f95-ac0fa52c26c4"}, { new Terminator(KeywordType.Not), "253ff382-06bc-47a7-9645-cc04c48e1d0a"}, { new Terminator(OperatorType.Minus), "5708e399-73e5-4bcb-9aeb-f25d1e06c35c"},}, new Dictionary<Terminator, ReduceInformation>{ }, "e8e3778c-2637-497d-8220-1ad1da0361e9") },
		{ "54b1c17e-914a-427a-be92-e4713c545e91", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.Then), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.Equal), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.Less), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.Greater), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(KeywordType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(KeywordType.Mod), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(KeywordType.And), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.Plus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.Minus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(KeywordType.Or), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, }, "54b1c17e-914a-427a-be92-e4713c545e91") },
		{ "8b173d7f-1010-4129-8ec7-38968bd4309c", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.ExpressionList), "5494b16e-774b-4c23-8697-27e63fc5ba97"}, { new NonTerminator(NonTerminatorType.Expression), "a0ba5247-cbd6-46f6-b1fd-574979eb2b27"}, { new NonTerminator(NonTerminatorType.SimpleExpression), "e8b0426e-8437-45f8-a02f-3eebdfb48e84"}, { new NonTerminator(NonTerminatorType.Term), "cd4bfe7a-5a2e-41f1-a891-6359311b9403"}, { new NonTerminator(NonTerminatorType.Factor), "ceb0b15a-7ff9-4265-8766-716485412510"}, { Terminator.NumberTerminator, "06d206a2-8c81-4399-8790-c903a8acfe21"}, { new NonTerminator(NonTerminatorType.Variable), "422866b2-29d6-4004-b762-cb9ad9ddc5e7"}, { new Terminator(DelimiterType.LeftParenthesis), "4f42a0a3-3c4b-46a5-8dcc-b91007f93d5c"}, { Terminator.IdentifierTerminator, "7339bcec-b0ae-4611-83cd-08345e44324f"}, { new Terminator(KeywordType.Not), "bfdaacfe-9460-4db8-9469-712b6674efb5"}, { new Terminator(OperatorType.Minus), "eac356e4-6098-43be-9b50-3ff1a637a898"},}, new Dictionary<Terminator, ReduceInformation>{ }, "8b173d7f-1010-4129-8ec7-38968bd4309c") },
		{ "04b41bb6-4af4-4542-998c-65289d824970", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.Then), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Equal), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Less), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Greater), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, }, "04b41bb6-4af4-4542-998c-65289d824970") },
		{ "34ad805f-ae0b-4119-8ba0-5df56a2828d7", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.Then), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Equal), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Less), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Greater), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, }, "34ad805f-ae0b-4119-8ba0-5df56a2828d7") },
		{ "054f5564-12c5-415f-93da-fc890622f35f", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Expression), "688b27e3-4b7c-4ac0-a50d-0b9c8a2f7aca"}, { new NonTerminator(NonTerminatorType.SimpleExpression), "d04891ca-bc62-431f-b1aa-67a00abdd72a"}, { new NonTerminator(NonTerminatorType.Term), "2e23b417-e8d1-4bf4-b5fb-791730c5bed2"}, { new NonTerminator(NonTerminatorType.Factor), "e2eecffb-85c1-440e-9c45-1fdd778c942a"}, { Terminator.NumberTerminator, "656892f9-1aaa-4635-84ff-c12d4e1a0c2e"}, { new NonTerminator(NonTerminatorType.Variable), "fd6b2f33-2a46-4dc5-93fc-0b7816be2582"}, { new Terminator(DelimiterType.LeftParenthesis), "079e396f-7464-432e-8666-7fe4b0ca40a7"}, { Terminator.IdentifierTerminator, "902c5106-8d85-47b7-b482-58ed10129c15"}, { new Terminator(KeywordType.Not), "b715b0a1-976c-42f9-84d7-4e3968a3369f"}, { new Terminator(OperatorType.Minus), "b8cd8bf3-bd87-4006-b6a9-4f0305e07f51"},}, new Dictionary<Terminator, ReduceInformation>{ }, "054f5564-12c5-415f-93da-fc890622f35f") },
		{ "f758521d-4774-4528-bf36-57ba44f80c2c", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(3, new NonTerminator(NonTerminatorType.CompoundStatement))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(3, new NonTerminator(NonTerminatorType.CompoundStatement))}, }, "f758521d-4774-4528-bf36-57ba44f80c2c") },
		{ "f55f44f9-867a-45a5-85c7-e032d9310084", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.CompoundStatement), "f4be5160-a0ca-45d4-ac17-ff50d8deb6bd"}, { new Terminator(KeywordType.Begin), "104d4182-c681-4b56-9b5d-a412fc2511b7"},}, new Dictionary<Terminator, ReduceInformation>{ }, "f55f44f9-867a-45a5-85c7-e032d9310084") },
		{ "ad2c8046-f4a8-4148-907b-90b9ae3d5720", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.VarDeclaration), "3326988a-63b1-4a78-8ad5-7725e4deb786"}, { new NonTerminator(NonTerminatorType.IdentifierList), "8c027315-eabd-4555-bf71-ae4e3c7cda0c"}, { Terminator.IdentifierTerminator, "c50004c4-fac8-4812-a2e2-6467f2005495"},}, new Dictionary<Terminator, ReduceInformation>{ }, "ad2c8046-f4a8-4148-907b-90b9ae3d5720") },
		{ "945dbf84-e151-47ce-a3d3-f6d2b0942e46", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.Semicolon), "3257f57a-9ec9-42c6-9625-a64c41fe7f31"},}, new Dictionary<Terminator, ReduceInformation>{ }, "945dbf84-e151-47ce-a3d3-f6d2b0942e46") },
		{ "228ea2cd-dcd1-4cc4-9e60-a9e8a5973e75", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.RightParenthesis), "07d3907e-7aae-4264-9299-8b112adff427"}, { new Terminator(DelimiterType.Semicolon), "f90b62e1-21c0-4c6a-992a-65f702d49dae"},}, new Dictionary<Terminator, ReduceInformation>{ }, "228ea2cd-dcd1-4cc4-9e60-a9e8a5973e75") },
		{ "ac87171c-34b2-465b-91d8-68d97e9befec", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightParenthesis), new ReduceInformation(1, new NonTerminator(NonTerminatorType.ParameterList))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(1, new NonTerminator(NonTerminatorType.ParameterList))}, }, "ac87171c-34b2-465b-91d8-68d97e9befec") },
		{ "d610ed1d-74c4-4cdb-9cf2-48380c72c98d", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightParenthesis), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Parameter))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Parameter))}, }, "d610ed1d-74c4-4cdb-9cf2-48380c72c98d") },
		{ "aa35680f-188d-43a6-b147-3fc855c4a46e", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightParenthesis), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Parameter))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Parameter))}, }, "aa35680f-188d-43a6-b147-3fc855c4a46e") },
		{ "0375c645-e2b3-4040-bde4-504155d42326", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.ValueParameter), "69f6ae8e-41d2-4d77-842d-9a03d9449fa2"}, { new NonTerminator(NonTerminatorType.IdentifierList), "aa9a1909-68d6-41a2-a737-f3cd97f97b1e"}, { Terminator.IdentifierTerminator, "c50004c4-fac8-4812-a2e2-6467f2005495"},}, new Dictionary<Terminator, ReduceInformation>{ }, "0375c645-e2b3-4040-bde4-504155d42326") },
		{ "aa9a1909-68d6-41a2-a737-f3cd97f97b1e", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.Colon), "2593fb7e-3a7f-4fa4-a5b6-a73bb488ec0a"}, { new Terminator(DelimiterType.Comma), "15e52f16-d101-4097-a288-bb0e1585a9ff"},}, new Dictionary<Terminator, ReduceInformation>{ }, "aa9a1909-68d6-41a2-a737-f3cd97f97b1e") },
		{ "8d27c6b3-8c45-49f5-90f6-6d7ca86204ee", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.BasicType), "9bdc6c1e-2cd4-40d9-8072-34d49903a9a9"}, { new Terminator(KeywordType.Integer), "7bb70cf0-2067-4971-ada1-ccf973097e8b"}, { new Terminator(KeywordType.Real), "1023abfc-7be1-4c20-890e-3ff30b2c89fc"}, { new Terminator(KeywordType.Boolean), "7430a84c-7617-405a-9992-d202dcbc8adc"}, { new Terminator(KeywordType.Character), "c1bdaca1-017b-4856-84d6-4cd20b5911a3"},}, new Dictionary<Terminator, ReduceInformation>{ }, "8d27c6b3-8c45-49f5-90f6-6d7ca86204ee") },
		{ "5b8f927a-5497-434a-ba5d-afabbd00fd19", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.RightParenthesis), "3db740d1-4acd-4273-84ce-2e7ba6959de4"}, { new Terminator(DelimiterType.Semicolon), "f90b62e1-21c0-4c6a-992a-65f702d49dae"},}, new Dictionary<Terminator, ReduceInformation>{ }, "5b8f927a-5497-434a-ba5d-afabbd00fd19") },
		{ "ef2f6f7a-6bb2-4379-9e30-d82a840ae22d", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.Semicolon), new ReduceInformation(5, new NonTerminator(NonTerminatorType.VarDeclaration))}, }, "ef2f6f7a-6bb2-4379-9e30-d82a840ae22d") },
		{ "4599a572-dd97-46c6-9c12-feb2224ee0a7", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.RightSquareBracket), "02e14155-08e5-499f-8320-4593a992e35b"}, { new Terminator(DelimiterType.Comma), "f640fb19-c3d9-4e03-a9ea-d3b7c872e35f"},}, new Dictionary<Terminator, ReduceInformation>{ }, "4599a572-dd97-46c6-9c12-feb2224ee0a7") },
		{ "fdbf1b7d-4afc-4ef5-ba8b-2a174df24c40", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.DoubleDots), "4abf7ed3-be9e-48a7-b6ba-5423b100ae08"},}, new Dictionary<Terminator, ReduceInformation>{ }, "fdbf1b7d-4afc-4ef5-ba8b-2a174df24c40") },
		{ "57fa87e6-c9d4-47d0-b838-2294a07701be", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.SimpleExpression), "f5a89065-5a6d-439e-8e0e-3ec48375c13c"}, { new NonTerminator(NonTerminatorType.Term), "6c18e9f8-e7c2-4713-9b67-f3a82891e681"}, { new NonTerminator(NonTerminatorType.Factor), "62c5c2fb-98a9-4f73-96af-5b4f878d8429"}, { Terminator.NumberTerminator, "758ac2f2-d58c-4624-8756-aba601efb559"}, { new NonTerminator(NonTerminatorType.Variable), "cddf29be-3197-4140-9341-7394cab82f23"}, { new Terminator(DelimiterType.LeftParenthesis), "7a95618a-8bb9-488b-848f-86edd4edb091"}, { Terminator.IdentifierTerminator, "b294720f-aa1c-4977-9724-f1338cad4cc9"}, { new Terminator(KeywordType.Not), "28b7637d-584d-48e7-8493-444cae07acbc"}, { new Terminator(OperatorType.Minus), "f10f52a0-dbc4-4505-90ef-c50f0d3044cd"},}, new Dictionary<Terminator, ReduceInformation>{ }, "57fa87e6-c9d4-47d0-b838-2294a07701be") },
		{ "e85467ab-5ef0-408e-9322-b3f27a0bb07e", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Term), "97403812-0f03-4f36-b74e-ed6abeabdd34"}, { new NonTerminator(NonTerminatorType.Factor), "7a25b1d8-c432-47b5-b5d6-4b9d4ec99976"}, { Terminator.NumberTerminator, "51e160d6-1614-4bea-878b-b9200a78d134"}, { new NonTerminator(NonTerminatorType.Variable), "3ef7029f-efcc-4dfc-bbef-9de3a66266b7"}, { new Terminator(DelimiterType.LeftParenthesis), "271a6714-8230-4423-9ba9-b983289c29e4"}, { Terminator.IdentifierTerminator, "a21cd4d2-38d8-44d9-a8cc-f43a72534d65"}, { new Terminator(KeywordType.Not), "c4efdd9d-271b-4565-84a7-a0f9bae205fe"}, { new Terminator(OperatorType.Minus), "9ecf3833-38d5-4e56-84a4-74c622293d59"},}, new Dictionary<Terminator, ReduceInformation>{ }, "e85467ab-5ef0-408e-9322-b3f27a0bb07e") },
		{ "8c35e188-2fab-48b5-a1f4-538ee6d2a5bf", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Factor), "25b9fe64-a467-428d-a311-275b247d3a90"}, { Terminator.NumberTerminator, "51e160d6-1614-4bea-878b-b9200a78d134"}, { new NonTerminator(NonTerminatorType.Variable), "3ef7029f-efcc-4dfc-bbef-9de3a66266b7"}, { new Terminator(DelimiterType.LeftParenthesis), "271a6714-8230-4423-9ba9-b983289c29e4"}, { Terminator.IdentifierTerminator, "a21cd4d2-38d8-44d9-a8cc-f43a72534d65"}, { new Terminator(KeywordType.Not), "c4efdd9d-271b-4565-84a7-a0f9bae205fe"}, { new Terminator(OperatorType.Minus), "9ecf3833-38d5-4e56-84a4-74c622293d59"},}, new Dictionary<Terminator, ReduceInformation>{ }, "8c35e188-2fab-48b5-a1f4-538ee6d2a5bf") },
		{ "4d73edab-fe7c-48d4-a431-8f2315f61cfe", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.RightParenthesis), "39f16b9c-17be-46ec-b140-992e07c4d3bf"},}, new Dictionary<Terminator, ReduceInformation>{ }, "4d73edab-fe7c-48d4-a431-8f2315f61cfe") },
		{ "d9202fc2-c54d-4a37-8ece-deab98ca67f7", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.ExpressionList), "41a2d866-d9fa-4ca2-a6c9-f71e1ac4dcff"}, { new NonTerminator(NonTerminatorType.Expression), "4f59631a-5553-49da-9f0d-dda48b0d65b2"}, { new NonTerminator(NonTerminatorType.SimpleExpression), "e2a2e2c4-61e9-496f-8a60-ac108404247f"}, { new NonTerminator(NonTerminatorType.Term), "83d0456c-4817-49a5-bd20-c0d8a1c5b696"}, { new NonTerminator(NonTerminatorType.Factor), "f1db3e46-b446-4adc-93ae-5dfed0964c99"}, { Terminator.NumberTerminator, "0abfb7c0-083d-4242-a9a7-06183f7230f7"}, { new NonTerminator(NonTerminatorType.Variable), "a8c27dec-213a-4917-8b05-b0422260d1c1"}, { new Terminator(DelimiterType.LeftParenthesis), "45c4069f-f3cf-48f7-af29-8eb8303c47d6"}, { Terminator.IdentifierTerminator, "5aabfbf6-09f5-484f-9f95-ac0fa52c26c4"}, { new Terminator(KeywordType.Not), "253ff382-06bc-47a7-9645-cc04c48e1d0a"}, { new Terminator(OperatorType.Minus), "5708e399-73e5-4bcb-9aeb-f25d1e06c35c"},}, new Dictionary<Terminator, ReduceInformation>{ }, "d9202fc2-c54d-4a37-8ece-deab98ca67f7") },
		{ "2acc352f-a616-4da2-95de-b2e571c47ace", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.Equal), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.Less), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.Greater), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(KeywordType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(KeywordType.Mod), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(KeywordType.And), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.Plus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.Minus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(KeywordType.Or), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, }, "2acc352f-a616-4da2-95de-b2e571c47ace") },
		{ "71f2a7de-322f-4c2c-99f6-1ab754fe6cfa", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.ExpressionList), "a77ae8ad-848c-438e-9253-fbe27a190792"}, { new NonTerminator(NonTerminatorType.Expression), "a0ba5247-cbd6-46f6-b1fd-574979eb2b27"}, { new NonTerminator(NonTerminatorType.SimpleExpression), "e8b0426e-8437-45f8-a02f-3eebdfb48e84"}, { new NonTerminator(NonTerminatorType.Term), "cd4bfe7a-5a2e-41f1-a891-6359311b9403"}, { new NonTerminator(NonTerminatorType.Factor), "ceb0b15a-7ff9-4265-8766-716485412510"}, { Terminator.NumberTerminator, "06d206a2-8c81-4399-8790-c903a8acfe21"}, { new NonTerminator(NonTerminatorType.Variable), "422866b2-29d6-4004-b762-cb9ad9ddc5e7"}, { new Terminator(DelimiterType.LeftParenthesis), "4f42a0a3-3c4b-46a5-8dcc-b91007f93d5c"}, { Terminator.IdentifierTerminator, "7339bcec-b0ae-4611-83cd-08345e44324f"}, { new Terminator(KeywordType.Not), "bfdaacfe-9460-4db8-9469-712b6674efb5"}, { new Terminator(OperatorType.Minus), "eac356e4-6098-43be-9b50-3ff1a637a898"},}, new Dictionary<Terminator, ReduceInformation>{ }, "71f2a7de-322f-4c2c-99f6-1ab754fe6cfa") },
		{ "75201f13-168d-46bc-bf6e-c2bae2776382", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Equal), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Less), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Greater), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, }, "75201f13-168d-46bc-bf6e-c2bae2776382") },
		{ "c1ae23a9-136a-4b7e-905d-16545bbb0514", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Equal), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Less), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Greater), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, }, "c1ae23a9-136a-4b7e-905d-16545bbb0514") },
		{ "0aca3c7f-d427-4a07-9482-6be4a300ff1c", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(OperatorType.Assign), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, }, "0aca3c7f-d427-4a07-9482-6be4a300ff1c") },
		{ "082dc96a-aa2d-444e-b2b0-8a6bca7f6ba2", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Expression), "dc202ebe-7108-4d52-adee-9bb586ea7716"}, { new NonTerminator(NonTerminatorType.SimpleExpression), "e8b0426e-8437-45f8-a02f-3eebdfb48e84"}, { new NonTerminator(NonTerminatorType.Term), "cd4bfe7a-5a2e-41f1-a891-6359311b9403"}, { new NonTerminator(NonTerminatorType.Factor), "ceb0b15a-7ff9-4265-8766-716485412510"}, { Terminator.NumberTerminator, "06d206a2-8c81-4399-8790-c903a8acfe21"}, { new NonTerminator(NonTerminatorType.Variable), "422866b2-29d6-4004-b762-cb9ad9ddc5e7"}, { new Terminator(DelimiterType.LeftParenthesis), "4f42a0a3-3c4b-46a5-8dcc-b91007f93d5c"}, { Terminator.IdentifierTerminator, "7339bcec-b0ae-4611-83cd-08345e44324f"}, { new Terminator(KeywordType.Not), "bfdaacfe-9460-4db8-9469-712b6674efb5"}, { new Terminator(OperatorType.Minus), "eac356e4-6098-43be-9b50-3ff1a637a898"},}, new Dictionary<Terminator, ReduceInformation>{ }, "082dc96a-aa2d-444e-b2b0-8a6bca7f6ba2") },
		{ "bb12941d-5825-4f3f-a62a-8c6682ff512e", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.SimpleExpression), "b717c153-e677-41f5-8942-c986ab0f8d5d"}, { new NonTerminator(NonTerminatorType.Term), "80e6a780-248a-4bf8-8bf1-4edd152badf3"}, { new NonTerminator(NonTerminatorType.Factor), "76c787f9-ae62-41d2-a3fd-4902c82e377b"}, { Terminator.NumberTerminator, "633178ec-3161-44c4-a90f-74a17a450286"}, { new NonTerminator(NonTerminatorType.Variable), "1490c0e9-6dd0-4710-8d43-856abc874cb6"}, { new Terminator(DelimiterType.LeftParenthesis), "f3f1b924-071c-4005-91d3-f9ac72d17c33"}, { Terminator.IdentifierTerminator, "dbede38f-60d1-4090-bfcf-5aab43d40fdd"}, { new Terminator(KeywordType.Not), "e3da8607-d17a-4bde-a57a-3c247abe7e39"}, { new Terminator(OperatorType.Minus), "01a95a22-0a4a-4fc4-b510-2d88603ba6c8"},}, new Dictionary<Terminator, ReduceInformation>{ }, "bb12941d-5825-4f3f-a62a-8c6682ff512e") },
		{ "4c209434-ee62-4c84-9b2f-4f4007bc4c3a", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Term), "2bd4c7c4-a1f7-499c-96cf-dcfc4cdedd94"}, { new NonTerminator(NonTerminatorType.Factor), "ceb0b15a-7ff9-4265-8766-716485412510"}, { Terminator.NumberTerminator, "06d206a2-8c81-4399-8790-c903a8acfe21"}, { new NonTerminator(NonTerminatorType.Variable), "422866b2-29d6-4004-b762-cb9ad9ddc5e7"}, { new Terminator(DelimiterType.LeftParenthesis), "4f42a0a3-3c4b-46a5-8dcc-b91007f93d5c"}, { Terminator.IdentifierTerminator, "7339bcec-b0ae-4611-83cd-08345e44324f"}, { new Terminator(KeywordType.Not), "bfdaacfe-9460-4db8-9469-712b6674efb5"}, { new Terminator(OperatorType.Minus), "eac356e4-6098-43be-9b50-3ff1a637a898"},}, new Dictionary<Terminator, ReduceInformation>{ }, "4c209434-ee62-4c84-9b2f-4f4007bc4c3a") },
		{ "2941ea9b-6c72-4779-8d5b-54e620856301", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Factor), "51faaaa3-0b9d-4395-ab4e-691a004a9668"}, { Terminator.NumberTerminator, "06d206a2-8c81-4399-8790-c903a8acfe21"}, { new NonTerminator(NonTerminatorType.Variable), "422866b2-29d6-4004-b762-cb9ad9ddc5e7"}, { new Terminator(DelimiterType.LeftParenthesis), "4f42a0a3-3c4b-46a5-8dcc-b91007f93d5c"}, { Terminator.IdentifierTerminator, "7339bcec-b0ae-4611-83cd-08345e44324f"}, { new Terminator(KeywordType.Not), "bfdaacfe-9460-4db8-9469-712b6674efb5"}, { new Terminator(OperatorType.Minus), "eac356e4-6098-43be-9b50-3ff1a637a898"},}, new Dictionary<Terminator, ReduceInformation>{ }, "2941ea9b-6c72-4779-8d5b-54e620856301") },
		{ "35b2a70a-6d3e-4b34-aa2d-477ff691c677", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.RightParenthesis), "5009f8c0-5221-4aca-a27f-47c9b94215ad"},}, new Dictionary<Terminator, ReduceInformation>{ }, "35b2a70a-6d3e-4b34-aa2d-477ff691c677") },
		{ "13ad14c5-3184-4246-892c-e5c8b1aa8ab0", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.ExpressionList), "e8e829bd-f41a-4d58-b2f0-730827f84c0b"}, { new NonTerminator(NonTerminatorType.Expression), "4f59631a-5553-49da-9f0d-dda48b0d65b2"}, { new NonTerminator(NonTerminatorType.SimpleExpression), "e2a2e2c4-61e9-496f-8a60-ac108404247f"}, { new NonTerminator(NonTerminatorType.Term), "83d0456c-4817-49a5-bd20-c0d8a1c5b696"}, { new NonTerminator(NonTerminatorType.Factor), "f1db3e46-b446-4adc-93ae-5dfed0964c99"}, { Terminator.NumberTerminator, "0abfb7c0-083d-4242-a9a7-06183f7230f7"}, { new NonTerminator(NonTerminatorType.Variable), "a8c27dec-213a-4917-8b05-b0422260d1c1"}, { new Terminator(DelimiterType.LeftParenthesis), "45c4069f-f3cf-48f7-af29-8eb8303c47d6"}, { Terminator.IdentifierTerminator, "5aabfbf6-09f5-484f-9f95-ac0fa52c26c4"}, { new Terminator(KeywordType.Not), "253ff382-06bc-47a7-9645-cc04c48e1d0a"}, { new Terminator(OperatorType.Minus), "5708e399-73e5-4bcb-9aeb-f25d1e06c35c"},}, new Dictionary<Terminator, ReduceInformation>{ }, "13ad14c5-3184-4246-892c-e5c8b1aa8ab0") },
		{ "3d2d3751-03bf-4a08-88e2-2ac7aa664671", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightSquareBracket), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.Equal), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.Less), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.Greater), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(KeywordType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(KeywordType.Mod), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(KeywordType.And), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.Plus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.Minus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(KeywordType.Or), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(DelimiterType.Comma), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, }, "3d2d3751-03bf-4a08-88e2-2ac7aa664671") },
		{ "c06874e3-04d1-4be1-ac5d-f77ab188ede7", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.ExpressionList), "194cb9d7-2361-4e28-9681-ba31187c2e47"}, { new NonTerminator(NonTerminatorType.Expression), "a0ba5247-cbd6-46f6-b1fd-574979eb2b27"}, { new NonTerminator(NonTerminatorType.SimpleExpression), "e8b0426e-8437-45f8-a02f-3eebdfb48e84"}, { new NonTerminator(NonTerminatorType.Term), "cd4bfe7a-5a2e-41f1-a891-6359311b9403"}, { new NonTerminator(NonTerminatorType.Factor), "ceb0b15a-7ff9-4265-8766-716485412510"}, { Terminator.NumberTerminator, "06d206a2-8c81-4399-8790-c903a8acfe21"}, { new NonTerminator(NonTerminatorType.Variable), "422866b2-29d6-4004-b762-cb9ad9ddc5e7"}, { new Terminator(DelimiterType.LeftParenthesis), "4f42a0a3-3c4b-46a5-8dcc-b91007f93d5c"}, { Terminator.IdentifierTerminator, "7339bcec-b0ae-4611-83cd-08345e44324f"}, { new Terminator(KeywordType.Not), "bfdaacfe-9460-4db8-9469-712b6674efb5"}, { new Terminator(OperatorType.Minus), "eac356e4-6098-43be-9b50-3ff1a637a898"},}, new Dictionary<Terminator, ReduceInformation>{ }, "c06874e3-04d1-4be1-ac5d-f77ab188ede7") },
		{ "871776e4-b741-4664-a3d1-c82844ec1a17", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightSquareBracket), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Equal), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Less), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Greater), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(DelimiterType.Comma), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, }, "871776e4-b741-4664-a3d1-c82844ec1a17") },
		{ "af4bc17c-8325-48dc-b52b-92f8d164c0d7", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightSquareBracket), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Equal), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Less), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Greater), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(DelimiterType.Comma), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, }, "af4bc17c-8325-48dc-b52b-92f8d164c0d7") },
		{ "bb8ace84-3c8a-497f-83e3-979ed4529545", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(4, new NonTerminator(NonTerminatorType.ProcedureCall))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(4, new NonTerminator(NonTerminatorType.ProcedureCall))}, }, "bb8ace84-3c8a-497f-83e3-979ed4529545") },
		{ "077bd83e-2dfc-4686-bf99-ec6992c05b60", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Expression), "6334aae5-3c87-4e92-95b1-fd0fecc41c35"}, { new NonTerminator(NonTerminatorType.SimpleExpression), "e2a2e2c4-61e9-496f-8a60-ac108404247f"}, { new NonTerminator(NonTerminatorType.Term), "83d0456c-4817-49a5-bd20-c0d8a1c5b696"}, { new NonTerminator(NonTerminatorType.Factor), "f1db3e46-b446-4adc-93ae-5dfed0964c99"}, { Terminator.NumberTerminator, "0abfb7c0-083d-4242-a9a7-06183f7230f7"}, { new NonTerminator(NonTerminatorType.Variable), "a8c27dec-213a-4917-8b05-b0422260d1c1"}, { new Terminator(DelimiterType.LeftParenthesis), "45c4069f-f3cf-48f7-af29-8eb8303c47d6"}, { Terminator.IdentifierTerminator, "5aabfbf6-09f5-484f-9f95-ac0fa52c26c4"}, { new Terminator(KeywordType.Not), "253ff382-06bc-47a7-9645-cc04c48e1d0a"}, { new Terminator(OperatorType.Minus), "5708e399-73e5-4bcb-9aeb-f25d1e06c35c"},}, new Dictionary<Terminator, ReduceInformation>{ }, "077bd83e-2dfc-4686-bf99-ec6992c05b60") },
		{ "5b323743-f503-403e-be13-7cd5cdb76bd8", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.SimpleExpression), "3cc0e5b3-de8a-442f-94c2-6edecc76ee3f"}, { new NonTerminator(NonTerminatorType.Term), "428b393c-7b47-4695-8334-e45639c30393"}, { new NonTerminator(NonTerminatorType.Factor), "943c00cb-ed4b-4d82-a518-e4dc6503a69b"}, { Terminator.NumberTerminator, "29cbbe86-9ccc-4bc4-8d0e-87b4ee384c1c"}, { new NonTerminator(NonTerminatorType.Variable), "6df173e5-a9e3-4900-b741-a97da24e5875"}, { new Terminator(DelimiterType.LeftParenthesis), "90be55bd-50ad-4d9b-881d-a98f6114b390"}, { Terminator.IdentifierTerminator, "331ff71b-3088-4da0-acd4-81f2ef4930a1"}, { new Terminator(KeywordType.Not), "5c70b501-eb1c-4473-98cb-c37355c66519"}, { new Terminator(OperatorType.Minus), "705a9f2b-51ea-4a38-9368-d3f8e27b2ed4"},}, new Dictionary<Terminator, ReduceInformation>{ }, "5b323743-f503-403e-be13-7cd5cdb76bd8") },
		{ "38426e6b-4d67-4d75-b310-309e6ea86212", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Term), "c5dc4110-1dcd-4262-abb8-c91b956d5578"}, { new NonTerminator(NonTerminatorType.Factor), "f1db3e46-b446-4adc-93ae-5dfed0964c99"}, { Terminator.NumberTerminator, "0abfb7c0-083d-4242-a9a7-06183f7230f7"}, { new NonTerminator(NonTerminatorType.Variable), "a8c27dec-213a-4917-8b05-b0422260d1c1"}, { new Terminator(DelimiterType.LeftParenthesis), "45c4069f-f3cf-48f7-af29-8eb8303c47d6"}, { Terminator.IdentifierTerminator, "5aabfbf6-09f5-484f-9f95-ac0fa52c26c4"}, { new Terminator(KeywordType.Not), "253ff382-06bc-47a7-9645-cc04c48e1d0a"}, { new Terminator(OperatorType.Minus), "5708e399-73e5-4bcb-9aeb-f25d1e06c35c"},}, new Dictionary<Terminator, ReduceInformation>{ }, "38426e6b-4d67-4d75-b310-309e6ea86212") },
		{ "37d17fe6-f232-4c2e-8d64-2ffb041dd55d", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Factor), "2f75ea4d-8b01-49a7-83f4-f8e16251603c"}, { Terminator.NumberTerminator, "0abfb7c0-083d-4242-a9a7-06183f7230f7"}, { new NonTerminator(NonTerminatorType.Variable), "a8c27dec-213a-4917-8b05-b0422260d1c1"}, { new Terminator(DelimiterType.LeftParenthesis), "45c4069f-f3cf-48f7-af29-8eb8303c47d6"}, { Terminator.IdentifierTerminator, "5aabfbf6-09f5-484f-9f95-ac0fa52c26c4"}, { new Terminator(KeywordType.Not), "253ff382-06bc-47a7-9645-cc04c48e1d0a"}, { new Terminator(OperatorType.Minus), "5708e399-73e5-4bcb-9aeb-f25d1e06c35c"},}, new Dictionary<Terminator, ReduceInformation>{ }, "37d17fe6-f232-4c2e-8d64-2ffb041dd55d") },
		{ "7eebe5e3-6af0-439d-977f-db6e51804ba1", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.RightParenthesis), "58bd90c3-19fd-4868-b23c-53f8243463d8"},}, new Dictionary<Terminator, ReduceInformation>{ }, "7eebe5e3-6af0-439d-977f-db6e51804ba1") },
		{ "48d6264a-4895-469d-8a3d-303a6be14a00", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.ExpressionList), "1ba508ca-0a53-49bd-a95c-41a6061127ad"}, { new NonTerminator(NonTerminatorType.Expression), "4f59631a-5553-49da-9f0d-dda48b0d65b2"}, { new NonTerminator(NonTerminatorType.SimpleExpression), "e2a2e2c4-61e9-496f-8a60-ac108404247f"}, { new NonTerminator(NonTerminatorType.Term), "83d0456c-4817-49a5-bd20-c0d8a1c5b696"}, { new NonTerminator(NonTerminatorType.Factor), "f1db3e46-b446-4adc-93ae-5dfed0964c99"}, { Terminator.NumberTerminator, "0abfb7c0-083d-4242-a9a7-06183f7230f7"}, { new NonTerminator(NonTerminatorType.Variable), "a8c27dec-213a-4917-8b05-b0422260d1c1"}, { new Terminator(DelimiterType.LeftParenthesis), "45c4069f-f3cf-48f7-af29-8eb8303c47d6"}, { Terminator.IdentifierTerminator, "5aabfbf6-09f5-484f-9f95-ac0fa52c26c4"}, { new Terminator(KeywordType.Not), "253ff382-06bc-47a7-9645-cc04c48e1d0a"}, { new Terminator(OperatorType.Minus), "5708e399-73e5-4bcb-9aeb-f25d1e06c35c"},}, new Dictionary<Terminator, ReduceInformation>{ }, "48d6264a-4895-469d-8a3d-303a6be14a00") },
		{ "0ba414d6-6370-42f8-a088-5f7b8e9fd42b", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightParenthesis), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.Equal), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.Less), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.Greater), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(KeywordType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(KeywordType.Mod), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(KeywordType.And), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.Plus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.Minus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(KeywordType.Or), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(DelimiterType.Comma), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, }, "0ba414d6-6370-42f8-a088-5f7b8e9fd42b") },
		{ "315084fc-3051-4edf-b8f8-4955459fecff", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.ExpressionList), "8e52bd07-93a3-4210-ab95-da094a0620a3"}, { new NonTerminator(NonTerminatorType.Expression), "a0ba5247-cbd6-46f6-b1fd-574979eb2b27"}, { new NonTerminator(NonTerminatorType.SimpleExpression), "e8b0426e-8437-45f8-a02f-3eebdfb48e84"}, { new NonTerminator(NonTerminatorType.Term), "cd4bfe7a-5a2e-41f1-a891-6359311b9403"}, { new NonTerminator(NonTerminatorType.Factor), "ceb0b15a-7ff9-4265-8766-716485412510"}, { Terminator.NumberTerminator, "06d206a2-8c81-4399-8790-c903a8acfe21"}, { new NonTerminator(NonTerminatorType.Variable), "422866b2-29d6-4004-b762-cb9ad9ddc5e7"}, { new Terminator(DelimiterType.LeftParenthesis), "4f42a0a3-3c4b-46a5-8dcc-b91007f93d5c"}, { Terminator.IdentifierTerminator, "7339bcec-b0ae-4611-83cd-08345e44324f"}, { new Terminator(KeywordType.Not), "bfdaacfe-9460-4db8-9469-712b6674efb5"}, { new Terminator(OperatorType.Minus), "eac356e4-6098-43be-9b50-3ff1a637a898"},}, new Dictionary<Terminator, ReduceInformation>{ }, "315084fc-3051-4edf-b8f8-4955459fecff") },
		{ "70ee78ef-1325-49e3-bba6-b20c44383937", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightParenthesis), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Equal), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Less), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Greater), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(DelimiterType.Comma), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, }, "70ee78ef-1325-49e3-bba6-b20c44383937") },
		{ "649e70ee-496e-40c8-b504-178b9a7d99ef", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightParenthesis), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Equal), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Less), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Greater), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(DelimiterType.Comma), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, }, "649e70ee-496e-40c8-b504-178b9a7d99ef") },
		{ "ea81cccb-cba2-4edd-ad23-836b2c152424", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.ElsePart), "f5fa97ce-7d99-4f9e-b8c8-7dfc085cd29a"}, { new Terminator(KeywordType.Else), "c9524055-57cc-46ca-8c58-c8ada812f3e2"},}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(0, new NonTerminator(NonTerminatorType.ElsePart))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(0, new NonTerminator(NonTerminatorType.ElsePart))}, }, "ea81cccb-cba2-4edd-ad23-836b2c152424") },
		{ "f7849288-2cd0-4c68-aeee-abb4f910fa32", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(OperatorType.Assign), "28b672a6-8d5a-4b12-937c-388394aae243"},}, new Dictionary<Terminator, ReduceInformation>{ }, "f7849288-2cd0-4c68-aeee-abb4f910fa32") },
		{ "403840ce-3751-4324-a4da-a2b0a7653957", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(OperatorType.Assign), "0eb426b5-dddc-47d3-870d-c74504073eb6"}, { new NonTerminator(NonTerminatorType.IdVarPart), "a4f48ffb-2f04-4d1c-a0cd-03d46da54477"}, { new Terminator(DelimiterType.LeftSquareBracket), "d00f55eb-4b09-4fe5-9eab-efff3126957e"}, { new Terminator(DelimiterType.LeftParenthesis), "5520a954-85a0-40f5-ac6e-e9b2b57f9c49"},}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(OperatorType.Assign), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.End), new ReduceInformation(1, new NonTerminator(NonTerminatorType.ProcedureCall))}, { new Terminator(KeywordType.Else), new ReduceInformation(1, new NonTerminator(NonTerminatorType.ProcedureCall))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(1, new NonTerminator(NonTerminatorType.ProcedureCall))}, }, "403840ce-3751-4324-a4da-a2b0a7653957") },
		{ "504c43d8-8335-43de-8838-5c3205046f5d", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Statement))}, { new Terminator(KeywordType.Else), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Statement))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Statement))}, }, "504c43d8-8335-43de-8838-5c3205046f5d") },
		{ "15b27fe6-dc50-4e57-8246-bc2e8a5eb05a", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Statement))}, { new Terminator(KeywordType.Else), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Statement))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Statement))}, }, "15b27fe6-dc50-4e57-8246-bc2e8a5eb05a") },
		{ "1e290bf0-8ecb-4a2d-b5dc-bd2aa7ff6505", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Expression), "bcf13123-831e-47a3-ad25-514752e9242c"}, { new NonTerminator(NonTerminatorType.SimpleExpression), "c709baca-481b-49c4-90c0-916900b02b3e"}, { new NonTerminator(NonTerminatorType.Term), "f5df4190-3621-4ffd-abac-15a525352e5c"}, { new NonTerminator(NonTerminatorType.Factor), "aa2a69ba-536a-4716-9726-fd4bca6a2ecb"}, { Terminator.NumberTerminator, "f2e280ec-cc53-4dd5-9e78-4d96496b2e43"}, { new NonTerminator(NonTerminatorType.Variable), "61692e3e-df15-4f2e-bbda-0e43f17752df"}, { new Terminator(DelimiterType.LeftParenthesis), "77ef6b8f-5b0a-42d4-9397-9bee38e59991"}, { Terminator.IdentifierTerminator, "43a0e8af-43b0-436c-bf72-119f9adb1db1"}, { new Terminator(KeywordType.Not), "bf6cd66a-c610-4c98-8ec1-fce070c9ea7a"}, { new Terminator(OperatorType.Minus), "5a485bf7-252f-452b-a035-e99007748568"},}, new Dictionary<Terminator, ReduceInformation>{ }, "1e290bf0-8ecb-4a2d-b5dc-bd2aa7ff6505") },
		{ "83bfcb23-a73e-4ba5-ac65-68e5a019cc6a", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { Terminator.IdentifierTerminator, "dd979ebe-7eb8-4e84-90eb-f1ce6c95bdcc"},}, new Dictionary<Terminator, ReduceInformation>{ }, "83bfcb23-a73e-4ba5-ac65-68e5a019cc6a") },
		{ "1fd17dc7-14a7-4500-8bc1-e768a6969eae", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.StatementList), "5823bb9b-c5c9-455a-9756-458d6ae46ae8"}, { new NonTerminator(NonTerminatorType.Statement), "5d302683-f6d0-4c7f-8a86-9311c20ee9f5"}, { new NonTerminator(NonTerminatorType.Variable), "168b14a3-c5c2-4d3b-85d6-8401a5d72d3f"}, { Terminator.IdentifierTerminator, "a9bcbf2a-3d77-41a8-a4cb-35695590c1a7"}, { new NonTerminator(NonTerminatorType.ProcedureCall), "ee7bd04c-ba1b-4c05-9f5a-92dcbd37e67b"}, { new NonTerminator(NonTerminatorType.CompoundStatement), "372dfbe4-160d-4966-bc33-aafc60a96c43"}, { new Terminator(KeywordType.If), "67ff610b-d0c2-4e7d-9505-4dceffaa4ee1"}, { new Terminator(KeywordType.For), "3bee9f71-bf83-4ee3-a95e-aeefaa9e9f45"}, { new Terminator(KeywordType.Begin), "de7a627a-1e9b-41f4-835e-c30359b633fd"},}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(0, new NonTerminator(NonTerminatorType.Statement))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(0, new NonTerminator(NonTerminatorType.Statement))}, }, "1fd17dc7-14a7-4500-8bc1-e768a6969eae") },
		{ "9f283a60-47f9-425f-9e6e-1d4df9415f95", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.AddOperator), "59a4aee5-b84f-4593-87a4-75a882f37d05"}, { new Terminator(OperatorType.Plus), "b2a223d6-6905-4b72-a67b-96d8604fe688"}, { new Terminator(OperatorType.Minus), "569b7d42-5c9a-4ba6-b124-8dacd7f387f4"}, { new Terminator(KeywordType.Or), "37aaac08-c0a9-420d-bfd1-1ad8615a7513"},}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.Then), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Expression))}, }, "9f283a60-47f9-425f-9e6e-1d4df9415f95") },
		{ "70c7c6ca-23db-4dc4-b18f-b7fec5c12330", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.MultiplyOperator), "2a0c4c84-0d34-48b7-ac1e-6c4263554c2c"}, { new Terminator(OperatorType.Multiply), "e576c109-5246-4139-8248-415b8e3e68de"}, { new Terminator(OperatorType.Divide), "6a567f32-04a0-49b8-bf8e-42f55ee0600f"}, { new Terminator(KeywordType.Divide), "86be9c46-a085-4183-9264-b12d73fde46e"}, { new Terminator(KeywordType.Mod), "d7708b97-7da8-492a-a64f-6196ae9527f7"}, { new Terminator(KeywordType.And), "271d9f46-4be0-495c-a90b-ae8f19ee928b"},}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.Then), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Plus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Minus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(KeywordType.Or), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, }, "70c7c6ca-23db-4dc4-b18f-b7fec5c12330") },
		{ "4c6ca0be-90cb-4569-8441-0ffc0c0a22a4", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.Then), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Mod), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.And), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Plus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Minus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Or), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, }, "4c6ca0be-90cb-4569-8441-0ffc0c0a22a4") },
		{ "f9789d84-d480-461d-a4a2-0404654a02e1", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.Then), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, }, "f9789d84-d480-461d-a4a2-0404654a02e1") },
		{ "4056346c-1389-4501-9fa7-8d71820cf943", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.Then), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, }, "4056346c-1389-4501-9fa7-8d71820cf943") },
		{ "141303a8-ff5b-48d6-9e48-10252b617d09", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Expression), "67454e29-e959-4a3f-87ce-8095c42f3fa1"}, { new NonTerminator(NonTerminatorType.SimpleExpression), "e566a9ad-f17c-46ff-981f-ee4fa81d2d3d"}, { new NonTerminator(NonTerminatorType.Term), "1f1e4a05-7d30-44f4-aa5f-5caacfbce4b2"}, { new NonTerminator(NonTerminatorType.Factor), "ff72d68f-ac20-46e8-94ed-090e841128c1"}, { Terminator.NumberTerminator, "79939c29-990e-4743-8a8a-5fccd2ec415d"}, { new NonTerminator(NonTerminatorType.Variable), "28f501cb-0b97-40c3-8a64-d214832a6789"}, { new Terminator(DelimiterType.LeftParenthesis), "f1464d64-129c-4d68-a61e-43296e963ee1"}, { Terminator.IdentifierTerminator, "9dd45908-639c-4e69-97e4-2471cdf5e189"}, { new Terminator(KeywordType.Not), "303776ee-584f-4977-b76a-9449d6a4e1ad"}, { new Terminator(OperatorType.Minus), "2401b03b-4e43-4409-989c-a4471708a795"},}, new Dictionary<Terminator, ReduceInformation>{ }, "141303a8-ff5b-48d6-9e48-10252b617d09") },
		{ "5b888b7a-23af-4ebe-a292-bafb94fabb7b", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.LeftParenthesis), "c067be76-c0c1-49fb-9ffd-3d978b7131d9"}, { new NonTerminator(NonTerminatorType.IdVarPart), "a45a8ba3-16ac-4520-8fc7-86ebc8d7082a"}, { new Terminator(DelimiterType.LeftSquareBracket), "ff8d8bb2-7c51-4e08-9145-b0bc208e3f5a"},}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.Then), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Divide), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Divide), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Mod), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.And), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Plus), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Minus), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Or), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, }, "5b888b7a-23af-4ebe-a292-bafb94fabb7b") },
		{ "df7787b9-6d11-4bb2-b89d-3527a0e1b4fb", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Factor), "cf1104ab-9c32-43e7-80cd-17ddb5f313ff"}, { Terminator.NumberTerminator, "f9789d84-d480-461d-a4a2-0404654a02e1"}, { new NonTerminator(NonTerminatorType.Variable), "4056346c-1389-4501-9fa7-8d71820cf943"}, { new Terminator(DelimiterType.LeftParenthesis), "141303a8-ff5b-48d6-9e48-10252b617d09"}, { Terminator.IdentifierTerminator, "5b888b7a-23af-4ebe-a292-bafb94fabb7b"}, { new Terminator(KeywordType.Not), "df7787b9-6d11-4bb2-b89d-3527a0e1b4fb"}, { new Terminator(OperatorType.Minus), "3404318e-e4c9-45b1-9569-32aeb85c2397"},}, new Dictionary<Terminator, ReduceInformation>{ }, "df7787b9-6d11-4bb2-b89d-3527a0e1b4fb") },
		{ "3404318e-e4c9-45b1-9569-32aeb85c2397", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Factor), "182d73f3-b8fa-428f-8894-ca7c3b587eaa"}, { Terminator.NumberTerminator, "f9789d84-d480-461d-a4a2-0404654a02e1"}, { new NonTerminator(NonTerminatorType.Variable), "4056346c-1389-4501-9fa7-8d71820cf943"}, { new Terminator(DelimiterType.LeftParenthesis), "141303a8-ff5b-48d6-9e48-10252b617d09"}, { Terminator.IdentifierTerminator, "5b888b7a-23af-4ebe-a292-bafb94fabb7b"}, { new Terminator(KeywordType.Not), "df7787b9-6d11-4bb2-b89d-3527a0e1b4fb"}, { new Terminator(OperatorType.Minus), "3404318e-e4c9-45b1-9569-32aeb85c2397"},}, new Dictionary<Terminator, ReduceInformation>{ }, "3404318e-e4c9-45b1-9569-32aeb85c2397") },
		{ "8b8539ff-6425-420b-bc8a-71286ffe917d", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.MultiplyOperator), "4221e2a4-55fa-4449-b6e3-fe531d22ac26"}, { new Terminator(OperatorType.Multiply), "e576c109-5246-4139-8248-415b8e3e68de"}, { new Terminator(OperatorType.Divide), "6a567f32-04a0-49b8-bf8e-42f55ee0600f"}, { new Terminator(KeywordType.Divide), "86be9c46-a085-4183-9264-b12d73fde46e"}, { new Terminator(KeywordType.Mod), "d7708b97-7da8-492a-a64f-6196ae9527f7"}, { new Terminator(KeywordType.And), "271d9f46-4be0-495c-a90b-ae8f19ee928b"},}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.Then), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Equal), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Less), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Greater), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Plus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Minus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(KeywordType.Or), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, }, "8b8539ff-6425-420b-bc8a-71286ffe917d") },
		{ "44b4a1fb-43f3-461f-ae64-3b8b736aab56", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.Then), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Equal), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Less), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Greater), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Mod), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.And), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Plus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Minus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Or), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, }, "44b4a1fb-43f3-461f-ae64-3b8b736aab56") },
		{ "a74d5a6c-e2b4-402f-9792-727949fcac7a", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.Then), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Equal), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Less), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Greater), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, }, "a74d5a6c-e2b4-402f-9792-727949fcac7a") },
		{ "c2b3f5d3-e730-4f15-bb6d-b23725240589", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.SimpleExpression), "14c29054-6d68-488c-89c3-b347db913a3c"}, { new NonTerminator(NonTerminatorType.Term), "d185d164-373f-4e5b-bc4a-b15fba215e7f"}, { new NonTerminator(NonTerminatorType.Factor), "e469f27a-a5b7-4a0a-aa29-0092ba56b087"}, { Terminator.NumberTerminator, "b0508395-27cc-4179-8a0f-1a5ac38d89b7"}, { new NonTerminator(NonTerminatorType.Variable), "6d5d615b-e7d3-422e-a5ba-123e617a33e0"}, { new Terminator(DelimiterType.LeftParenthesis), "617493e7-522c-4e09-b525-6b29f2c96528"}, { Terminator.IdentifierTerminator, "a8b3e8cf-a167-4e52-a876-56fc36b99e81"}, { new Terminator(KeywordType.Not), "65e2d29a-99a0-4377-95af-9b1954c5e202"}, { new Terminator(OperatorType.Minus), "dba55308-efb5-4ad7-8bee-f379f817f752"},}, new Dictionary<Terminator, ReduceInformation>{ }, "c2b3f5d3-e730-4f15-bb6d-b23725240589") },
		{ "f69a1f56-560a-4942-ab98-ecd516cb5a1a", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Term), "2a3db56a-f45c-49ef-a3d3-3276e864d148"}, { new NonTerminator(NonTerminatorType.Factor), "ff72d68f-ac20-46e8-94ed-090e841128c1"}, { Terminator.NumberTerminator, "79939c29-990e-4743-8a8a-5fccd2ec415d"}, { new NonTerminator(NonTerminatorType.Variable), "28f501cb-0b97-40c3-8a64-d214832a6789"}, { new Terminator(DelimiterType.LeftParenthesis), "f1464d64-129c-4d68-a61e-43296e963ee1"}, { Terminator.IdentifierTerminator, "9dd45908-639c-4e69-97e4-2471cdf5e189"}, { new Terminator(KeywordType.Not), "303776ee-584f-4977-b76a-9449d6a4e1ad"}, { new Terminator(OperatorType.Minus), "2401b03b-4e43-4409-989c-a4471708a795"},}, new Dictionary<Terminator, ReduceInformation>{ }, "f69a1f56-560a-4942-ab98-ecd516cb5a1a") },
		{ "d53ff53c-5245-4c95-86c0-b368cdebee74", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Factor), "255c54b3-cc93-438f-805c-6c4751f74a07"}, { Terminator.NumberTerminator, "79939c29-990e-4743-8a8a-5fccd2ec415d"}, { new NonTerminator(NonTerminatorType.Variable), "28f501cb-0b97-40c3-8a64-d214832a6789"}, { new Terminator(DelimiterType.LeftParenthesis), "f1464d64-129c-4d68-a61e-43296e963ee1"}, { Terminator.IdentifierTerminator, "9dd45908-639c-4e69-97e4-2471cdf5e189"}, { new Terminator(KeywordType.Not), "303776ee-584f-4977-b76a-9449d6a4e1ad"}, { new Terminator(OperatorType.Minus), "2401b03b-4e43-4409-989c-a4471708a795"},}, new Dictionary<Terminator, ReduceInformation>{ }, "d53ff53c-5245-4c95-86c0-b368cdebee74") },
		{ "6bdf7e41-6eeb-4d19-afc2-b45f24c683ab", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.RightParenthesis), "b4d256a5-9ef4-4102-83f5-8f89facde412"},}, new Dictionary<Terminator, ReduceInformation>{ }, "6bdf7e41-6eeb-4d19-afc2-b45f24c683ab") },
		{ "22565d79-b3bb-4b85-8ad9-f72e1c88e65f", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.ExpressionList), "98477bb8-54b2-403f-a917-742fcb30f71f"}, { new NonTerminator(NonTerminatorType.Expression), "4f59631a-5553-49da-9f0d-dda48b0d65b2"}, { new NonTerminator(NonTerminatorType.SimpleExpression), "e2a2e2c4-61e9-496f-8a60-ac108404247f"}, { new NonTerminator(NonTerminatorType.Term), "83d0456c-4817-49a5-bd20-c0d8a1c5b696"}, { new NonTerminator(NonTerminatorType.Factor), "f1db3e46-b446-4adc-93ae-5dfed0964c99"}, { Terminator.NumberTerminator, "0abfb7c0-083d-4242-a9a7-06183f7230f7"}, { new NonTerminator(NonTerminatorType.Variable), "a8c27dec-213a-4917-8b05-b0422260d1c1"}, { new Terminator(DelimiterType.LeftParenthesis), "45c4069f-f3cf-48f7-af29-8eb8303c47d6"}, { Terminator.IdentifierTerminator, "5aabfbf6-09f5-484f-9f95-ac0fa52c26c4"}, { new Terminator(KeywordType.Not), "253ff382-06bc-47a7-9645-cc04c48e1d0a"}, { new Terminator(OperatorType.Minus), "5708e399-73e5-4bcb-9aeb-f25d1e06c35c"},}, new Dictionary<Terminator, ReduceInformation>{ }, "22565d79-b3bb-4b85-8ad9-f72e1c88e65f") },
		{ "64aa0e6e-093b-4546-abcc-aad4534b619b", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightParenthesis), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.Equal), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.Less), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.Greater), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(KeywordType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(KeywordType.Mod), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(KeywordType.And), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.Plus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.Minus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(KeywordType.Or), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, }, "64aa0e6e-093b-4546-abcc-aad4534b619b") },
		{ "0f70ecea-4f81-433a-9d4e-9541b90b78e0", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.ExpressionList), "da77dc61-b08b-4ce3-80a1-baef83e7ea3e"}, { new NonTerminator(NonTerminatorType.Expression), "a0ba5247-cbd6-46f6-b1fd-574979eb2b27"}, { new NonTerminator(NonTerminatorType.SimpleExpression), "e8b0426e-8437-45f8-a02f-3eebdfb48e84"}, { new NonTerminator(NonTerminatorType.Term), "cd4bfe7a-5a2e-41f1-a891-6359311b9403"}, { new NonTerminator(NonTerminatorType.Factor), "ceb0b15a-7ff9-4265-8766-716485412510"}, { Terminator.NumberTerminator, "06d206a2-8c81-4399-8790-c903a8acfe21"}, { new NonTerminator(NonTerminatorType.Variable), "422866b2-29d6-4004-b762-cb9ad9ddc5e7"}, { new Terminator(DelimiterType.LeftParenthesis), "4f42a0a3-3c4b-46a5-8dcc-b91007f93d5c"}, { Terminator.IdentifierTerminator, "7339bcec-b0ae-4611-83cd-08345e44324f"}, { new Terminator(KeywordType.Not), "bfdaacfe-9460-4db8-9469-712b6674efb5"}, { new Terminator(OperatorType.Minus), "eac356e4-6098-43be-9b50-3ff1a637a898"},}, new Dictionary<Terminator, ReduceInformation>{ }, "0f70ecea-4f81-433a-9d4e-9541b90b78e0") },
		{ "ffe1849f-88ee-4687-9f8e-860a984bd772", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightParenthesis), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Equal), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Less), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Greater), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, }, "ffe1849f-88ee-4687-9f8e-860a984bd772") },
		{ "9e52d3c3-9893-4085-90f9-786048447625", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightParenthesis), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Equal), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Less), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Greater), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, }, "9e52d3c3-9893-4085-90f9-786048447625") },
		{ "e881dbac-a57f-488d-8e0a-13868896ba85", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.RightParenthesis), "8fa4269c-234c-4ef1-9b8d-c0efe7b89c92"}, { new Terminator(DelimiterType.Comma), "077bd83e-2dfc-4686-bf99-ec6992c05b60"},}, new Dictionary<Terminator, ReduceInformation>{ }, "e881dbac-a57f-488d-8e0a-13868896ba85") },
		{ "5494b16e-774b-4c23-8697-27e63fc5ba97", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.RightSquareBracket), "9b9df458-8d8f-4d49-844f-6b254033b604"}, { new Terminator(DelimiterType.Comma), "082dc96a-aa2d-444e-b2b0-8a6bca7f6ba2"},}, new Dictionary<Terminator, ReduceInformation>{ }, "5494b16e-774b-4c23-8697-27e63fc5ba97") },
		{ "688b27e3-4b7c-4ac0-a50d-0b9c8a2f7aca", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(KeywordType.To), "71db17f2-1dde-4f6a-ac54-28213b704612"},}, new Dictionary<Terminator, ReduceInformation>{ }, "688b27e3-4b7c-4ac0-a50d-0b9c8a2f7aca") },
		{ "d04891ca-bc62-431f-b1aa-67a00abdd72a", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.RelationOperator), "806b2d65-cc39-4355-addb-97d15dfea312"}, { new Terminator(OperatorType.Equal), "1b49be29-a9d8-403a-b434-5c116300d15c"}, { new Terminator(OperatorType.NotEqual), "fb7de677-df97-4431-b5b7-086a8ded26ff"}, { new Terminator(OperatorType.Less), "8db4bd06-8fe0-4476-924a-3354772a5ac3"}, { new Terminator(OperatorType.LessEqual), "68ce8cad-2c90-4f71-822f-885e193cc394"}, { new Terminator(OperatorType.Greater), "f863fa1a-f90f-430f-8ee8-4cb33053ee45"}, { new Terminator(OperatorType.GreaterEqual), "d79c0f7f-9997-421d-bed4-1e0b2517774c"}, { new NonTerminator(NonTerminatorType.AddOperator), "a22c3e34-c025-417e-b171-36b1c26a94dd"}, { new Terminator(OperatorType.Plus), "b2a223d6-6905-4b72-a67b-96d8604fe688"}, { new Terminator(OperatorType.Minus), "569b7d42-5c9a-4ba6-b124-8dacd7f387f4"}, { new Terminator(KeywordType.Or), "37aaac08-c0a9-420d-bfd1-1ad8615a7513"},}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.To), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Expression))}, }, "d04891ca-bc62-431f-b1aa-67a00abdd72a") },
		{ "2e23b417-e8d1-4bf4-b5fb-791730c5bed2", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.MultiplyOperator), "a2d36e1c-1210-4175-a84b-9df1fe589223"}, { new Terminator(OperatorType.Multiply), "e576c109-5246-4139-8248-415b8e3e68de"}, { new Terminator(OperatorType.Divide), "6a567f32-04a0-49b8-bf8e-42f55ee0600f"}, { new Terminator(KeywordType.Divide), "86be9c46-a085-4183-9264-b12d73fde46e"}, { new Terminator(KeywordType.Mod), "d7708b97-7da8-492a-a64f-6196ae9527f7"}, { new Terminator(KeywordType.And), "271d9f46-4be0-495c-a90b-ae8f19ee928b"},}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.To), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Equal), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Less), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Greater), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Plus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Minus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(KeywordType.Or), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, }, "2e23b417-e8d1-4bf4-b5fb-791730c5bed2") },
		{ "e2eecffb-85c1-440e-9c45-1fdd778c942a", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.To), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Equal), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Less), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Greater), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Mod), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.And), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Plus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Minus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Or), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, }, "e2eecffb-85c1-440e-9c45-1fdd778c942a") },
		{ "656892f9-1aaa-4635-84ff-c12d4e1a0c2e", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.To), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Equal), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Less), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Greater), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, }, "656892f9-1aaa-4635-84ff-c12d4e1a0c2e") },
		{ "fd6b2f33-2a46-4dc5-93fc-0b7816be2582", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.To), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Equal), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Less), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Greater), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, }, "fd6b2f33-2a46-4dc5-93fc-0b7816be2582") },
		{ "079e396f-7464-432e-8666-7fe4b0ca40a7", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Expression), "bc5aaf35-5671-4613-9b9c-1590c9882f5c"}, { new NonTerminator(NonTerminatorType.SimpleExpression), "e566a9ad-f17c-46ff-981f-ee4fa81d2d3d"}, { new NonTerminator(NonTerminatorType.Term), "1f1e4a05-7d30-44f4-aa5f-5caacfbce4b2"}, { new NonTerminator(NonTerminatorType.Factor), "ff72d68f-ac20-46e8-94ed-090e841128c1"}, { Terminator.NumberTerminator, "79939c29-990e-4743-8a8a-5fccd2ec415d"}, { new NonTerminator(NonTerminatorType.Variable), "28f501cb-0b97-40c3-8a64-d214832a6789"}, { new Terminator(DelimiterType.LeftParenthesis), "f1464d64-129c-4d68-a61e-43296e963ee1"}, { Terminator.IdentifierTerminator, "9dd45908-639c-4e69-97e4-2471cdf5e189"}, { new Terminator(KeywordType.Not), "303776ee-584f-4977-b76a-9449d6a4e1ad"}, { new Terminator(OperatorType.Minus), "2401b03b-4e43-4409-989c-a4471708a795"},}, new Dictionary<Terminator, ReduceInformation>{ }, "079e396f-7464-432e-8666-7fe4b0ca40a7") },
		{ "902c5106-8d85-47b7-b482-58ed10129c15", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.LeftParenthesis), "6fac19b0-1bb1-4052-b3c5-882b032d0029"}, { new NonTerminator(NonTerminatorType.IdVarPart), "e2eeef26-5763-4d1c-b737-96be01a478bf"}, { new Terminator(DelimiterType.LeftSquareBracket), "5444abce-0a3e-40bd-ab62-c420bab088e1"},}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.To), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Equal), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Less), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Greater), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Divide), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Divide), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Mod), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.And), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Plus), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Minus), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Or), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, }, "902c5106-8d85-47b7-b482-58ed10129c15") },
		{ "b715b0a1-976c-42f9-84d7-4e3968a3369f", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Factor), "f77a3c90-e980-406e-b0e2-e373308241fa"}, { Terminator.NumberTerminator, "656892f9-1aaa-4635-84ff-c12d4e1a0c2e"}, { new NonTerminator(NonTerminatorType.Variable), "fd6b2f33-2a46-4dc5-93fc-0b7816be2582"}, { new Terminator(DelimiterType.LeftParenthesis), "079e396f-7464-432e-8666-7fe4b0ca40a7"}, { Terminator.IdentifierTerminator, "902c5106-8d85-47b7-b482-58ed10129c15"}, { new Terminator(KeywordType.Not), "b715b0a1-976c-42f9-84d7-4e3968a3369f"}, { new Terminator(OperatorType.Minus), "b8cd8bf3-bd87-4006-b6a9-4f0305e07f51"},}, new Dictionary<Terminator, ReduceInformation>{ }, "b715b0a1-976c-42f9-84d7-4e3968a3369f") },
		{ "b8cd8bf3-bd87-4006-b6a9-4f0305e07f51", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Factor), "255e6483-e7ed-49d3-affc-d58bbf391c0d"}, { Terminator.NumberTerminator, "656892f9-1aaa-4635-84ff-c12d4e1a0c2e"}, { new NonTerminator(NonTerminatorType.Variable), "fd6b2f33-2a46-4dc5-93fc-0b7816be2582"}, { new Terminator(DelimiterType.LeftParenthesis), "079e396f-7464-432e-8666-7fe4b0ca40a7"}, { Terminator.IdentifierTerminator, "902c5106-8d85-47b7-b482-58ed10129c15"}, { new Terminator(KeywordType.Not), "b715b0a1-976c-42f9-84d7-4e3968a3369f"}, { new Terminator(OperatorType.Minus), "b8cd8bf3-bd87-4006-b6a9-4f0305e07f51"},}, new Dictionary<Terminator, ReduceInformation>{ }, "b8cd8bf3-bd87-4006-b6a9-4f0305e07f51") },
		{ "f4be5160-a0ca-45d4-ac17-ff50d8deb6bd", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.Semicolon), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SubprogramBody))}, }, "f4be5160-a0ca-45d4-ac17-ff50d8deb6bd") },
		{ "104d4182-c681-4b56-9b5d-a412fc2511b7", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.StatementList), "3a25beb9-a581-4483-ab34-73483caedf0c"}, { new NonTerminator(NonTerminatorType.Statement), "5d302683-f6d0-4c7f-8a86-9311c20ee9f5"}, { new NonTerminator(NonTerminatorType.Variable), "168b14a3-c5c2-4d3b-85d6-8401a5d72d3f"}, { Terminator.IdentifierTerminator, "a9bcbf2a-3d77-41a8-a4cb-35695590c1a7"}, { new NonTerminator(NonTerminatorType.ProcedureCall), "ee7bd04c-ba1b-4c05-9f5a-92dcbd37e67b"}, { new NonTerminator(NonTerminatorType.CompoundStatement), "372dfbe4-160d-4966-bc33-aafc60a96c43"}, { new Terminator(KeywordType.If), "67ff610b-d0c2-4e7d-9505-4dceffaa4ee1"}, { new Terminator(KeywordType.For), "3bee9f71-bf83-4ee3-a95e-aeefaa9e9f45"}, { new Terminator(KeywordType.Begin), "de7a627a-1e9b-41f4-835e-c30359b633fd"},}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(0, new NonTerminator(NonTerminatorType.Statement))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(0, new NonTerminator(NonTerminatorType.Statement))}, }, "104d4182-c681-4b56-9b5d-a412fc2511b7") },
		{ "3326988a-63b1-4a78-8ad5-7725e4deb786", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.Semicolon), "ca200296-ea1c-456f-a213-2310dad1fac7"},}, new Dictionary<Terminator, ReduceInformation>{ }, "3326988a-63b1-4a78-8ad5-7725e4deb786") },
		{ "3257f57a-9ec9-42c6-9625-a64c41fe7f31", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { Terminator.IdentifierTerminator, "5e84e22e-943e-4d5b-bda9-89e059814265"},}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.Begin), new ReduceInformation(3, new NonTerminator(NonTerminatorType.ConstDeclarations))}, { new Terminator(KeywordType.Var), new ReduceInformation(3, new NonTerminator(NonTerminatorType.ConstDeclarations))}, }, "3257f57a-9ec9-42c6-9625-a64c41fe7f31") },
		{ "07d3907e-7aae-4264-9299-8b112adff427", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.Semicolon), new ReduceInformation(3, new NonTerminator(NonTerminatorType.FormalParameter))}, }, "07d3907e-7aae-4264-9299-8b112adff427") },
		{ "f90b62e1-21c0-4c6a-992a-65f702d49dae", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Parameter), "a40fe53b-e5f3-4539-8924-d66d122f99a7"}, { new NonTerminator(NonTerminatorType.VarParameter), "d610ed1d-74c4-4cdb-9cf2-48380c72c98d"}, { new NonTerminator(NonTerminatorType.ValueParameter), "aa35680f-188d-43a6-b147-3fc855c4a46e"}, { new Terminator(KeywordType.Var), "0375c645-e2b3-4040-bde4-504155d42326"}, { new NonTerminator(NonTerminatorType.IdentifierList), "aa9a1909-68d6-41a2-a737-f3cd97f97b1e"}, { Terminator.IdentifierTerminator, "c50004c4-fac8-4812-a2e2-6467f2005495"},}, new Dictionary<Terminator, ReduceInformation>{ }, "f90b62e1-21c0-4c6a-992a-65f702d49dae") },
		{ "69f6ae8e-41d2-4d77-842d-9a03d9449fa2", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightParenthesis), new ReduceInformation(2, new NonTerminator(NonTerminatorType.VarParameter))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(2, new NonTerminator(NonTerminatorType.VarParameter))}, }, "69f6ae8e-41d2-4d77-842d-9a03d9449fa2") },
		{ "2593fb7e-3a7f-4fa4-a5b6-a73bb488ec0a", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.BasicType), "380ebab4-0036-4a21-9aa6-f5371bed4cc8"}, { new Terminator(KeywordType.Integer), "e4b6a6bc-d2bc-476b-a5a0-311483e9d80d"}, { new Terminator(KeywordType.Real), "c6cc9782-a41f-4889-bcbb-946126edec40"}, { new Terminator(KeywordType.Boolean), "8cfac156-0b52-4e72-9664-8957b522fc7c"}, { new Terminator(KeywordType.Character), "1e1234cc-b049-4146-836a-ef291747ebb6"},}, new Dictionary<Terminator, ReduceInformation>{ }, "2593fb7e-3a7f-4fa4-a5b6-a73bb488ec0a") },
		{ "9bdc6c1e-2cd4-40d9-8072-34d49903a9a9", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.Semicolon), new ReduceInformation(5, new NonTerminator(NonTerminatorType.SubprogramHead))}, }, "9bdc6c1e-2cd4-40d9-8072-34d49903a9a9") },
		{ "3db740d1-4acd-4273-84ce-2e7ba6959de4", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.Colon), new ReduceInformation(3, new NonTerminator(NonTerminatorType.FormalParameter))}, }, "3db740d1-4acd-4273-84ce-2e7ba6959de4") },
		{ "02e14155-08e5-499f-8320-4593a992e35b", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(KeywordType.Of), "75999946-7bf8-4c24-9606-df0f9038f602"},}, new Dictionary<Terminator, ReduceInformation>{ }, "02e14155-08e5-499f-8320-4593a992e35b") },
		{ "f640fb19-c3d9-4e03-a9ea-d3b7c872e35f", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { Terminator.NumberTerminator, "096a7769-5bc2-4e3a-92d8-7fa0d5e764bf"},}, new Dictionary<Terminator, ReduceInformation>{ }, "f640fb19-c3d9-4e03-a9ea-d3b7c872e35f") },
		{ "4abf7ed3-be9e-48a7-b6ba-5423b100ae08", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { Terminator.NumberTerminator, "92b1055b-bbdb-4853-beb5-9f0b3b2c5cd0"},}, new Dictionary<Terminator, ReduceInformation>{ }, "4abf7ed3-be9e-48a7-b6ba-5423b100ae08") },
		{ "f5a89065-5a6d-439e-8e0e-3ec48375c13c", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.AddOperator), "0efc69e2-dd61-45f4-9e36-bd84f61b8217"}, { new Terminator(OperatorType.Plus), "b2a223d6-6905-4b72-a67b-96d8604fe688"}, { new Terminator(OperatorType.Minus), "569b7d42-5c9a-4ba6-b124-8dacd7f387f4"}, { new Terminator(KeywordType.Or), "37aaac08-c0a9-420d-bfd1-1ad8615a7513"},}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Expression))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Expression))}, }, "f5a89065-5a6d-439e-8e0e-3ec48375c13c") },
		{ "6c18e9f8-e7c2-4713-9b67-f3a82891e681", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.MultiplyOperator), "2f27ae56-216f-4e19-b722-ed4dd82cebbb"}, { new Terminator(OperatorType.Multiply), "e576c109-5246-4139-8248-415b8e3e68de"}, { new Terminator(OperatorType.Divide), "6a567f32-04a0-49b8-bf8e-42f55ee0600f"}, { new Terminator(KeywordType.Divide), "86be9c46-a085-4183-9264-b12d73fde46e"}, { new Terminator(KeywordType.Mod), "d7708b97-7da8-492a-a64f-6196ae9527f7"}, { new Terminator(KeywordType.And), "271d9f46-4be0-495c-a90b-ae8f19ee928b"},}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Plus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Minus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(KeywordType.Or), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, }, "6c18e9f8-e7c2-4713-9b67-f3a82891e681") },
		{ "62c5c2fb-98a9-4f73-96af-5b4f878d8429", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Mod), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.And), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Plus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Minus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Or), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, }, "62c5c2fb-98a9-4f73-96af-5b4f878d8429") },
		{ "758ac2f2-d58c-4624-8756-aba601efb559", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, }, "758ac2f2-d58c-4624-8756-aba601efb559") },
		{ "cddf29be-3197-4140-9341-7394cab82f23", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, }, "cddf29be-3197-4140-9341-7394cab82f23") },
		{ "7a95618a-8bb9-488b-848f-86edd4edb091", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Expression), "f584e8fa-31a6-4c8b-a675-9718057f4aa4"}, { new NonTerminator(NonTerminatorType.SimpleExpression), "e566a9ad-f17c-46ff-981f-ee4fa81d2d3d"}, { new NonTerminator(NonTerminatorType.Term), "1f1e4a05-7d30-44f4-aa5f-5caacfbce4b2"}, { new NonTerminator(NonTerminatorType.Factor), "ff72d68f-ac20-46e8-94ed-090e841128c1"}, { Terminator.NumberTerminator, "79939c29-990e-4743-8a8a-5fccd2ec415d"}, { new NonTerminator(NonTerminatorType.Variable), "28f501cb-0b97-40c3-8a64-d214832a6789"}, { new Terminator(DelimiterType.LeftParenthesis), "f1464d64-129c-4d68-a61e-43296e963ee1"}, { Terminator.IdentifierTerminator, "9dd45908-639c-4e69-97e4-2471cdf5e189"}, { new Terminator(KeywordType.Not), "303776ee-584f-4977-b76a-9449d6a4e1ad"}, { new Terminator(OperatorType.Minus), "2401b03b-4e43-4409-989c-a4471708a795"},}, new Dictionary<Terminator, ReduceInformation>{ }, "7a95618a-8bb9-488b-848f-86edd4edb091") },
		{ "b294720f-aa1c-4977-9724-f1338cad4cc9", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.LeftParenthesis), "829f5068-893b-4ca4-9c7e-f42656c66be5"}, { new NonTerminator(NonTerminatorType.IdVarPart), "d891e285-c4e8-4851-a322-ef787d98617a"}, { new Terminator(DelimiterType.LeftSquareBracket), "aa6cb10e-8b1e-4838-9859-5f24d9f7dc9c"},}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Divide), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Divide), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Mod), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.And), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Plus), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Minus), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Or), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, }, "b294720f-aa1c-4977-9724-f1338cad4cc9") },
		{ "28b7637d-584d-48e7-8493-444cae07acbc", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Factor), "4bfa1678-d410-43d5-b40a-fd6324530c55"}, { Terminator.NumberTerminator, "758ac2f2-d58c-4624-8756-aba601efb559"}, { new NonTerminator(NonTerminatorType.Variable), "cddf29be-3197-4140-9341-7394cab82f23"}, { new Terminator(DelimiterType.LeftParenthesis), "7a95618a-8bb9-488b-848f-86edd4edb091"}, { Terminator.IdentifierTerminator, "b294720f-aa1c-4977-9724-f1338cad4cc9"}, { new Terminator(KeywordType.Not), "28b7637d-584d-48e7-8493-444cae07acbc"}, { new Terminator(OperatorType.Minus), "f10f52a0-dbc4-4505-90ef-c50f0d3044cd"},}, new Dictionary<Terminator, ReduceInformation>{ }, "28b7637d-584d-48e7-8493-444cae07acbc") },
		{ "f10f52a0-dbc4-4505-90ef-c50f0d3044cd", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Factor), "641501ea-443a-43d3-b0d4-1b158a4ecd0d"}, { Terminator.NumberTerminator, "758ac2f2-d58c-4624-8756-aba601efb559"}, { new NonTerminator(NonTerminatorType.Variable), "cddf29be-3197-4140-9341-7394cab82f23"}, { new Terminator(DelimiterType.LeftParenthesis), "7a95618a-8bb9-488b-848f-86edd4edb091"}, { Terminator.IdentifierTerminator, "b294720f-aa1c-4977-9724-f1338cad4cc9"}, { new Terminator(KeywordType.Not), "28b7637d-584d-48e7-8493-444cae07acbc"}, { new Terminator(OperatorType.Minus), "f10f52a0-dbc4-4505-90ef-c50f0d3044cd"},}, new Dictionary<Terminator, ReduceInformation>{ }, "f10f52a0-dbc4-4505-90ef-c50f0d3044cd") },
		{ "97403812-0f03-4f36-b74e-ed6abeabdd34", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.MultiplyOperator), "8c35e188-2fab-48b5-a1f4-538ee6d2a5bf"}, { new Terminator(OperatorType.Multiply), "e576c109-5246-4139-8248-415b8e3e68de"}, { new Terminator(OperatorType.Divide), "6a567f32-04a0-49b8-bf8e-42f55ee0600f"}, { new Terminator(KeywordType.Divide), "86be9c46-a085-4183-9264-b12d73fde46e"}, { new Terminator(KeywordType.Mod), "d7708b97-7da8-492a-a64f-6196ae9527f7"}, { new Terminator(KeywordType.And), "271d9f46-4be0-495c-a90b-ae8f19ee928b"},}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Equal), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Less), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Greater), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Plus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Minus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(KeywordType.Or), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, }, "97403812-0f03-4f36-b74e-ed6abeabdd34") },
		{ "25b9fe64-a467-428d-a311-275b247d3a90", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Equal), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Less), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Greater), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Mod), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.And), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Plus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Minus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Or), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, }, "25b9fe64-a467-428d-a311-275b247d3a90") },
		{ "39f16b9c-17be-46ec-b140-992e07c4d3bf", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Equal), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Less), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Greater), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, }, "39f16b9c-17be-46ec-b140-992e07c4d3bf") },
		{ "41a2d866-d9fa-4ca2-a6c9-f71e1ac4dcff", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.RightParenthesis), "714c11fa-d1cb-49e9-b85e-de43a21c70eb"}, { new Terminator(DelimiterType.Comma), "077bd83e-2dfc-4686-bf99-ec6992c05b60"},}, new Dictionary<Terminator, ReduceInformation>{ }, "41a2d866-d9fa-4ca2-a6c9-f71e1ac4dcff") },
		{ "a77ae8ad-848c-438e-9253-fbe27a190792", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.RightSquareBracket), "52f8dc5c-e7ff-4788-bb96-c1ec6582e0a4"}, { new Terminator(DelimiterType.Comma), "082dc96a-aa2d-444e-b2b0-8a6bca7f6ba2"},}, new Dictionary<Terminator, ReduceInformation>{ }, "a77ae8ad-848c-438e-9253-fbe27a190792") },
		{ "dc202ebe-7108-4d52-adee-9bb586ea7716", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightSquareBracket), new ReduceInformation(3, new NonTerminator(NonTerminatorType.ExpressionList))}, { new Terminator(DelimiterType.Comma), new ReduceInformation(3, new NonTerminator(NonTerminatorType.ExpressionList))}, }, "dc202ebe-7108-4d52-adee-9bb586ea7716") },
		{ "b717c153-e677-41f5-8942-c986ab0f8d5d", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.AddOperator), "cb32f811-1f28-49cc-be2b-da3c842b47a8"}, { new Terminator(OperatorType.Plus), "b2a223d6-6905-4b72-a67b-96d8604fe688"}, { new Terminator(OperatorType.Minus), "569b7d42-5c9a-4ba6-b124-8dacd7f387f4"}, { new Terminator(KeywordType.Or), "37aaac08-c0a9-420d-bfd1-1ad8615a7513"},}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightSquareBracket), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Expression))}, { new Terminator(DelimiterType.Comma), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Expression))}, }, "b717c153-e677-41f5-8942-c986ab0f8d5d") },
		{ "80e6a780-248a-4bf8-8bf1-4edd152badf3", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.MultiplyOperator), "fda8ff9f-0d9c-4a54-9207-d10bf57748e1"}, { new Terminator(OperatorType.Multiply), "e576c109-5246-4139-8248-415b8e3e68de"}, { new Terminator(OperatorType.Divide), "6a567f32-04a0-49b8-bf8e-42f55ee0600f"}, { new Terminator(KeywordType.Divide), "86be9c46-a085-4183-9264-b12d73fde46e"}, { new Terminator(KeywordType.Mod), "d7708b97-7da8-492a-a64f-6196ae9527f7"}, { new Terminator(KeywordType.And), "271d9f46-4be0-495c-a90b-ae8f19ee928b"},}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightSquareBracket), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Plus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Minus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(KeywordType.Or), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(DelimiterType.Comma), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, }, "80e6a780-248a-4bf8-8bf1-4edd152badf3") },
		{ "76c787f9-ae62-41d2-a3fd-4902c82e377b", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightSquareBracket), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Mod), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.And), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Plus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Minus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Or), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(DelimiterType.Comma), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, }, "76c787f9-ae62-41d2-a3fd-4902c82e377b") },
		{ "633178ec-3161-44c4-a90f-74a17a450286", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightSquareBracket), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(DelimiterType.Comma), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, }, "633178ec-3161-44c4-a90f-74a17a450286") },
		{ "1490c0e9-6dd0-4710-8d43-856abc874cb6", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightSquareBracket), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(DelimiterType.Comma), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, }, "1490c0e9-6dd0-4710-8d43-856abc874cb6") },
		{ "f3f1b924-071c-4005-91d3-f9ac72d17c33", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Expression), "f3bcbd79-4d59-4161-b974-473ba5208d1b"}, { new NonTerminator(NonTerminatorType.SimpleExpression), "e566a9ad-f17c-46ff-981f-ee4fa81d2d3d"}, { new NonTerminator(NonTerminatorType.Term), "1f1e4a05-7d30-44f4-aa5f-5caacfbce4b2"}, { new NonTerminator(NonTerminatorType.Factor), "ff72d68f-ac20-46e8-94ed-090e841128c1"}, { Terminator.NumberTerminator, "79939c29-990e-4743-8a8a-5fccd2ec415d"}, { new NonTerminator(NonTerminatorType.Variable), "28f501cb-0b97-40c3-8a64-d214832a6789"}, { new Terminator(DelimiterType.LeftParenthesis), "f1464d64-129c-4d68-a61e-43296e963ee1"}, { Terminator.IdentifierTerminator, "9dd45908-639c-4e69-97e4-2471cdf5e189"}, { new Terminator(KeywordType.Not), "303776ee-584f-4977-b76a-9449d6a4e1ad"}, { new Terminator(OperatorType.Minus), "2401b03b-4e43-4409-989c-a4471708a795"},}, new Dictionary<Terminator, ReduceInformation>{ }, "f3f1b924-071c-4005-91d3-f9ac72d17c33") },
		{ "dbede38f-60d1-4090-bfcf-5aab43d40fdd", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.LeftParenthesis), "e5a843a5-4497-40f8-bed7-9ffb986ea6ca"}, { new NonTerminator(NonTerminatorType.IdVarPart), "9821331b-3752-4fcc-a594-ee2086beed82"}, { new Terminator(DelimiterType.LeftSquareBracket), "1019d72f-ddf3-4512-860c-c92bd9bb70ef"},}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightSquareBracket), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Divide), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Divide), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Mod), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.And), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Plus), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Minus), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Or), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(DelimiterType.Comma), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, }, "dbede38f-60d1-4090-bfcf-5aab43d40fdd") },
		{ "e3da8607-d17a-4bde-a57a-3c247abe7e39", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Factor), "e97b41de-7b5d-455c-995b-d3c3b5d34e08"}, { Terminator.NumberTerminator, "633178ec-3161-44c4-a90f-74a17a450286"}, { new NonTerminator(NonTerminatorType.Variable), "1490c0e9-6dd0-4710-8d43-856abc874cb6"}, { new Terminator(DelimiterType.LeftParenthesis), "f3f1b924-071c-4005-91d3-f9ac72d17c33"}, { Terminator.IdentifierTerminator, "dbede38f-60d1-4090-bfcf-5aab43d40fdd"}, { new Terminator(KeywordType.Not), "e3da8607-d17a-4bde-a57a-3c247abe7e39"}, { new Terminator(OperatorType.Minus), "01a95a22-0a4a-4fc4-b510-2d88603ba6c8"},}, new Dictionary<Terminator, ReduceInformation>{ }, "e3da8607-d17a-4bde-a57a-3c247abe7e39") },
		{ "01a95a22-0a4a-4fc4-b510-2d88603ba6c8", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Factor), "8a4b12cc-a316-4835-8228-ef65e5f01612"}, { Terminator.NumberTerminator, "633178ec-3161-44c4-a90f-74a17a450286"}, { new NonTerminator(NonTerminatorType.Variable), "1490c0e9-6dd0-4710-8d43-856abc874cb6"}, { new Terminator(DelimiterType.LeftParenthesis), "f3f1b924-071c-4005-91d3-f9ac72d17c33"}, { Terminator.IdentifierTerminator, "dbede38f-60d1-4090-bfcf-5aab43d40fdd"}, { new Terminator(KeywordType.Not), "e3da8607-d17a-4bde-a57a-3c247abe7e39"}, { new Terminator(OperatorType.Minus), "01a95a22-0a4a-4fc4-b510-2d88603ba6c8"},}, new Dictionary<Terminator, ReduceInformation>{ }, "01a95a22-0a4a-4fc4-b510-2d88603ba6c8") },
		{ "2bd4c7c4-a1f7-499c-96cf-dcfc4cdedd94", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.MultiplyOperator), "2941ea9b-6c72-4779-8d5b-54e620856301"}, { new Terminator(OperatorType.Multiply), "e576c109-5246-4139-8248-415b8e3e68de"}, { new Terminator(OperatorType.Divide), "6a567f32-04a0-49b8-bf8e-42f55ee0600f"}, { new Terminator(KeywordType.Divide), "86be9c46-a085-4183-9264-b12d73fde46e"}, { new Terminator(KeywordType.Mod), "d7708b97-7da8-492a-a64f-6196ae9527f7"}, { new Terminator(KeywordType.And), "271d9f46-4be0-495c-a90b-ae8f19ee928b"},}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightSquareBracket), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Equal), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Less), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Greater), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Plus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Minus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(KeywordType.Or), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(DelimiterType.Comma), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, }, "2bd4c7c4-a1f7-499c-96cf-dcfc4cdedd94") },
		{ "51faaaa3-0b9d-4395-ab4e-691a004a9668", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightSquareBracket), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Equal), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Less), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Greater), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Mod), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.And), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Plus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Minus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Or), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(DelimiterType.Comma), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, }, "51faaaa3-0b9d-4395-ab4e-691a004a9668") },
		{ "5009f8c0-5221-4aca-a27f-47c9b94215ad", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightSquareBracket), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Equal), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Less), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Greater), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(DelimiterType.Comma), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, }, "5009f8c0-5221-4aca-a27f-47c9b94215ad") },
		{ "e8e829bd-f41a-4d58-b2f0-730827f84c0b", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.RightParenthesis), "513889a7-e087-48ca-af5e-b92f1bd4cf06"}, { new Terminator(DelimiterType.Comma), "077bd83e-2dfc-4686-bf99-ec6992c05b60"},}, new Dictionary<Terminator, ReduceInformation>{ }, "e8e829bd-f41a-4d58-b2f0-730827f84c0b") },
		{ "194cb9d7-2361-4e28-9681-ba31187c2e47", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.RightSquareBracket), "379270a2-7f40-40a1-b030-e8416dd5dd70"}, { new Terminator(DelimiterType.Comma), "082dc96a-aa2d-444e-b2b0-8a6bca7f6ba2"},}, new Dictionary<Terminator, ReduceInformation>{ }, "194cb9d7-2361-4e28-9681-ba31187c2e47") },
		{ "6334aae5-3c87-4e92-95b1-fd0fecc41c35", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightParenthesis), new ReduceInformation(3, new NonTerminator(NonTerminatorType.ExpressionList))}, { new Terminator(DelimiterType.Comma), new ReduceInformation(3, new NonTerminator(NonTerminatorType.ExpressionList))}, }, "6334aae5-3c87-4e92-95b1-fd0fecc41c35") },
		{ "3cc0e5b3-de8a-442f-94c2-6edecc76ee3f", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.AddOperator), "08b76352-9cbd-4207-addf-e1df764428cf"}, { new Terminator(OperatorType.Plus), "b2a223d6-6905-4b72-a67b-96d8604fe688"}, { new Terminator(OperatorType.Minus), "569b7d42-5c9a-4ba6-b124-8dacd7f387f4"}, { new Terminator(KeywordType.Or), "37aaac08-c0a9-420d-bfd1-1ad8615a7513"},}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightParenthesis), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Expression))}, { new Terminator(DelimiterType.Comma), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Expression))}, }, "3cc0e5b3-de8a-442f-94c2-6edecc76ee3f") },
		{ "428b393c-7b47-4695-8334-e45639c30393", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.MultiplyOperator), "a37c0513-7bbf-49ad-8a65-73ce9ee54a69"}, { new Terminator(OperatorType.Multiply), "e576c109-5246-4139-8248-415b8e3e68de"}, { new Terminator(OperatorType.Divide), "6a567f32-04a0-49b8-bf8e-42f55ee0600f"}, { new Terminator(KeywordType.Divide), "86be9c46-a085-4183-9264-b12d73fde46e"}, { new Terminator(KeywordType.Mod), "d7708b97-7da8-492a-a64f-6196ae9527f7"}, { new Terminator(KeywordType.And), "271d9f46-4be0-495c-a90b-ae8f19ee928b"},}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightParenthesis), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Plus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Minus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(KeywordType.Or), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(DelimiterType.Comma), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, }, "428b393c-7b47-4695-8334-e45639c30393") },
		{ "943c00cb-ed4b-4d82-a518-e4dc6503a69b", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightParenthesis), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Mod), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.And), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Plus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Minus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Or), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(DelimiterType.Comma), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, }, "943c00cb-ed4b-4d82-a518-e4dc6503a69b") },
		{ "29cbbe86-9ccc-4bc4-8d0e-87b4ee384c1c", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightParenthesis), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(DelimiterType.Comma), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, }, "29cbbe86-9ccc-4bc4-8d0e-87b4ee384c1c") },
		{ "6df173e5-a9e3-4900-b741-a97da24e5875", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightParenthesis), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(DelimiterType.Comma), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, }, "6df173e5-a9e3-4900-b741-a97da24e5875") },
		{ "90be55bd-50ad-4d9b-881d-a98f6114b390", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Expression), "10a0b5b2-20c1-45cc-a771-9db9e18dc2ad"}, { new NonTerminator(NonTerminatorType.SimpleExpression), "e566a9ad-f17c-46ff-981f-ee4fa81d2d3d"}, { new NonTerminator(NonTerminatorType.Term), "1f1e4a05-7d30-44f4-aa5f-5caacfbce4b2"}, { new NonTerminator(NonTerminatorType.Factor), "ff72d68f-ac20-46e8-94ed-090e841128c1"}, { Terminator.NumberTerminator, "79939c29-990e-4743-8a8a-5fccd2ec415d"}, { new NonTerminator(NonTerminatorType.Variable), "28f501cb-0b97-40c3-8a64-d214832a6789"}, { new Terminator(DelimiterType.LeftParenthesis), "f1464d64-129c-4d68-a61e-43296e963ee1"}, { Terminator.IdentifierTerminator, "9dd45908-639c-4e69-97e4-2471cdf5e189"}, { new Terminator(KeywordType.Not), "303776ee-584f-4977-b76a-9449d6a4e1ad"}, { new Terminator(OperatorType.Minus), "2401b03b-4e43-4409-989c-a4471708a795"},}, new Dictionary<Terminator, ReduceInformation>{ }, "90be55bd-50ad-4d9b-881d-a98f6114b390") },
		{ "331ff71b-3088-4da0-acd4-81f2ef4930a1", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.LeftParenthesis), "4b2e3c36-ab02-4972-81f2-7d5af3fb02a1"}, { new NonTerminator(NonTerminatorType.IdVarPart), "6101cf2b-3c08-46a4-b42f-cdbf32a372e9"}, { new Terminator(DelimiterType.LeftSquareBracket), "ff3e717b-816c-4fc4-b8fe-21ad89303b12"},}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightParenthesis), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Divide), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Divide), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Mod), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.And), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Plus), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Minus), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Or), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(DelimiterType.Comma), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, }, "331ff71b-3088-4da0-acd4-81f2ef4930a1") },
		{ "5c70b501-eb1c-4473-98cb-c37355c66519", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Factor), "0cb854f1-c280-4dd4-99d4-d5a3b2c379a4"}, { Terminator.NumberTerminator, "29cbbe86-9ccc-4bc4-8d0e-87b4ee384c1c"}, { new NonTerminator(NonTerminatorType.Variable), "6df173e5-a9e3-4900-b741-a97da24e5875"}, { new Terminator(DelimiterType.LeftParenthesis), "90be55bd-50ad-4d9b-881d-a98f6114b390"}, { Terminator.IdentifierTerminator, "331ff71b-3088-4da0-acd4-81f2ef4930a1"}, { new Terminator(KeywordType.Not), "5c70b501-eb1c-4473-98cb-c37355c66519"}, { new Terminator(OperatorType.Minus), "705a9f2b-51ea-4a38-9368-d3f8e27b2ed4"},}, new Dictionary<Terminator, ReduceInformation>{ }, "5c70b501-eb1c-4473-98cb-c37355c66519") },
		{ "705a9f2b-51ea-4a38-9368-d3f8e27b2ed4", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Factor), "22d7d273-e8a4-44d8-8b90-50434f32c8fd"}, { Terminator.NumberTerminator, "29cbbe86-9ccc-4bc4-8d0e-87b4ee384c1c"}, { new NonTerminator(NonTerminatorType.Variable), "6df173e5-a9e3-4900-b741-a97da24e5875"}, { new Terminator(DelimiterType.LeftParenthesis), "90be55bd-50ad-4d9b-881d-a98f6114b390"}, { Terminator.IdentifierTerminator, "331ff71b-3088-4da0-acd4-81f2ef4930a1"}, { new Terminator(KeywordType.Not), "5c70b501-eb1c-4473-98cb-c37355c66519"}, { new Terminator(OperatorType.Minus), "705a9f2b-51ea-4a38-9368-d3f8e27b2ed4"},}, new Dictionary<Terminator, ReduceInformation>{ }, "705a9f2b-51ea-4a38-9368-d3f8e27b2ed4") },
		{ "c5dc4110-1dcd-4262-abb8-c91b956d5578", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.MultiplyOperator), "37d17fe6-f232-4c2e-8d64-2ffb041dd55d"}, { new Terminator(OperatorType.Multiply), "e576c109-5246-4139-8248-415b8e3e68de"}, { new Terminator(OperatorType.Divide), "6a567f32-04a0-49b8-bf8e-42f55ee0600f"}, { new Terminator(KeywordType.Divide), "86be9c46-a085-4183-9264-b12d73fde46e"}, { new Terminator(KeywordType.Mod), "d7708b97-7da8-492a-a64f-6196ae9527f7"}, { new Terminator(KeywordType.And), "271d9f46-4be0-495c-a90b-ae8f19ee928b"},}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightParenthesis), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Equal), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Less), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Greater), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Plus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Minus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(KeywordType.Or), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(DelimiterType.Comma), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, }, "c5dc4110-1dcd-4262-abb8-c91b956d5578") },
		{ "2f75ea4d-8b01-49a7-83f4-f8e16251603c", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightParenthesis), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Equal), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Less), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Greater), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Mod), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.And), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Plus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Minus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Or), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(DelimiterType.Comma), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, }, "2f75ea4d-8b01-49a7-83f4-f8e16251603c") },
		{ "58bd90c3-19fd-4868-b23c-53f8243463d8", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightParenthesis), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Equal), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Less), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Greater), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(DelimiterType.Comma), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, }, "58bd90c3-19fd-4868-b23c-53f8243463d8") },
		{ "1ba508ca-0a53-49bd-a95c-41a6061127ad", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.RightParenthesis), "70e2a79d-494e-480c-ad7b-8ef65c844484"}, { new Terminator(DelimiterType.Comma), "077bd83e-2dfc-4686-bf99-ec6992c05b60"},}, new Dictionary<Terminator, ReduceInformation>{ }, "1ba508ca-0a53-49bd-a95c-41a6061127ad") },
		{ "8e52bd07-93a3-4210-ab95-da094a0620a3", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.RightSquareBracket), "33dbbefe-a95a-4d00-9d08-ae26273859dc"}, { new Terminator(DelimiterType.Comma), "082dc96a-aa2d-444e-b2b0-8a6bca7f6ba2"},}, new Dictionary<Terminator, ReduceInformation>{ }, "8e52bd07-93a3-4210-ab95-da094a0620a3") },
		{ "f5fa97ce-7d99-4f9e-b8c8-7dfc085cd29a", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(5, new NonTerminator(NonTerminatorType.Statement))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(5, new NonTerminator(NonTerminatorType.Statement))}, }, "f5fa97ce-7d99-4f9e-b8c8-7dfc085cd29a") },
		{ "c9524055-57cc-46ca-8c58-c8ada812f3e2", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Statement), "8e3f9ecd-6c3f-411a-a840-ca18b76e185a"}, { new NonTerminator(NonTerminatorType.Variable), "168b14a3-c5c2-4d3b-85d6-8401a5d72d3f"}, { Terminator.IdentifierTerminator, "a9bcbf2a-3d77-41a8-a4cb-35695590c1a7"}, { new NonTerminator(NonTerminatorType.ProcedureCall), "ee7bd04c-ba1b-4c05-9f5a-92dcbd37e67b"}, { new NonTerminator(NonTerminatorType.CompoundStatement), "372dfbe4-160d-4966-bc33-aafc60a96c43"}, { new Terminator(KeywordType.If), "67ff610b-d0c2-4e7d-9505-4dceffaa4ee1"}, { new Terminator(KeywordType.For), "3bee9f71-bf83-4ee3-a95e-aeefaa9e9f45"}, { new Terminator(KeywordType.Begin), "de7a627a-1e9b-41f4-835e-c30359b633fd"},}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(0, new NonTerminator(NonTerminatorType.Statement))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(0, new NonTerminator(NonTerminatorType.Statement))}, }, "c9524055-57cc-46ca-8c58-c8ada812f3e2") },
		{ "28b672a6-8d5a-4b12-937c-388394aae243", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Expression), "8f395f71-8009-423b-a6eb-9c3e7c865f26"}, { new NonTerminator(NonTerminatorType.SimpleExpression), "ff5e3d10-5b5d-425f-ab1f-9fdd2a3cc3fd"}, { new NonTerminator(NonTerminatorType.Term), "b73935ec-c2f7-405d-b963-c028060e4aef"}, { new NonTerminator(NonTerminatorType.Factor), "4aa8e9d3-a1a6-4705-80c6-5c186dc595e8"}, { Terminator.NumberTerminator, "012315f4-1fde-4502-b517-f4971dda6e61"}, { new NonTerminator(NonTerminatorType.Variable), "54633641-356f-4730-9414-2206c55b142e"}, { new Terminator(DelimiterType.LeftParenthesis), "27dee30b-ae72-4e32-a279-cca6925591e0"}, { Terminator.IdentifierTerminator, "162410d2-e221-4dcc-b587-ecb26e782be7"}, { new Terminator(KeywordType.Not), "75cbf022-d2eb-4962-8b19-cbc270acb474"}, { new Terminator(OperatorType.Minus), "d8b62d2f-6f12-438e-9866-38ee91689fff"},}, new Dictionary<Terminator, ReduceInformation>{ }, "28b672a6-8d5a-4b12-937c-388394aae243") },
		{ "0eb426b5-dddc-47d3-870d-c74504073eb6", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Expression), "96fed0e4-0e78-4e98-87bc-e5353e32b6a9"}, { new NonTerminator(NonTerminatorType.SimpleExpression), "ff5e3d10-5b5d-425f-ab1f-9fdd2a3cc3fd"}, { new NonTerminator(NonTerminatorType.Term), "b73935ec-c2f7-405d-b963-c028060e4aef"}, { new NonTerminator(NonTerminatorType.Factor), "4aa8e9d3-a1a6-4705-80c6-5c186dc595e8"}, { Terminator.NumberTerminator, "012315f4-1fde-4502-b517-f4971dda6e61"}, { new NonTerminator(NonTerminatorType.Variable), "54633641-356f-4730-9414-2206c55b142e"}, { new Terminator(DelimiterType.LeftParenthesis), "27dee30b-ae72-4e32-a279-cca6925591e0"}, { Terminator.IdentifierTerminator, "162410d2-e221-4dcc-b587-ecb26e782be7"}, { new Terminator(KeywordType.Not), "75cbf022-d2eb-4962-8b19-cbc270acb474"}, { new Terminator(OperatorType.Minus), "d8b62d2f-6f12-438e-9866-38ee91689fff"},}, new Dictionary<Terminator, ReduceInformation>{ }, "0eb426b5-dddc-47d3-870d-c74504073eb6") },
		{ "5520a954-85a0-40f5-ac6e-e9b2b57f9c49", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.ExpressionList), "ec56d7f1-93f2-4594-85e3-ecac0c328912"}, { new NonTerminator(NonTerminatorType.Expression), "4f59631a-5553-49da-9f0d-dda48b0d65b2"}, { new NonTerminator(NonTerminatorType.SimpleExpression), "e2a2e2c4-61e9-496f-8a60-ac108404247f"}, { new NonTerminator(NonTerminatorType.Term), "83d0456c-4817-49a5-bd20-c0d8a1c5b696"}, { new NonTerminator(NonTerminatorType.Factor), "f1db3e46-b446-4adc-93ae-5dfed0964c99"}, { Terminator.NumberTerminator, "0abfb7c0-083d-4242-a9a7-06183f7230f7"}, { new NonTerminator(NonTerminatorType.Variable), "a8c27dec-213a-4917-8b05-b0422260d1c1"}, { new Terminator(DelimiterType.LeftParenthesis), "45c4069f-f3cf-48f7-af29-8eb8303c47d6"}, { Terminator.IdentifierTerminator, "5aabfbf6-09f5-484f-9f95-ac0fa52c26c4"}, { new Terminator(KeywordType.Not), "253ff382-06bc-47a7-9645-cc04c48e1d0a"}, { new Terminator(OperatorType.Minus), "5708e399-73e5-4bcb-9aeb-f25d1e06c35c"},}, new Dictionary<Terminator, ReduceInformation>{ }, "5520a954-85a0-40f5-ac6e-e9b2b57f9c49") },
		{ "bcf13123-831e-47a3-ad25-514752e9242c", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(KeywordType.Then), "3dfb3788-fd4a-4da1-bc2c-54ed1108602e"},}, new Dictionary<Terminator, ReduceInformation>{ }, "bcf13123-831e-47a3-ad25-514752e9242c") },
		{ "dd979ebe-7eb8-4e84-90eb-f1ce6c95bdcc", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(OperatorType.Assign), "852b35d2-4d23-4fdc-852b-10e8cbbc4a52"},}, new Dictionary<Terminator, ReduceInformation>{ }, "dd979ebe-7eb8-4e84-90eb-f1ce6c95bdcc") },
		{ "5823bb9b-c5c9-455a-9756-458d6ae46ae8", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(KeywordType.End), "db50ea31-01ea-498f-bf72-f0744d2f3cd2"}, { new Terminator(DelimiterType.Semicolon), "41bdccbb-3f79-4d74-9b06-5a48b781a2c2"},}, new Dictionary<Terminator, ReduceInformation>{ }, "5823bb9b-c5c9-455a-9756-458d6ae46ae8") },
		{ "59a4aee5-b84f-4593-87a4-75a882f37d05", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Term), "faddedbb-3247-4458-98ea-694e7b6ed843"}, { new NonTerminator(NonTerminatorType.Factor), "4c6ca0be-90cb-4569-8441-0ffc0c0a22a4"}, { Terminator.NumberTerminator, "f9789d84-d480-461d-a4a2-0404654a02e1"}, { new NonTerminator(NonTerminatorType.Variable), "4056346c-1389-4501-9fa7-8d71820cf943"}, { new Terminator(DelimiterType.LeftParenthesis), "141303a8-ff5b-48d6-9e48-10252b617d09"}, { Terminator.IdentifierTerminator, "5b888b7a-23af-4ebe-a292-bafb94fabb7b"}, { new Terminator(KeywordType.Not), "df7787b9-6d11-4bb2-b89d-3527a0e1b4fb"}, { new Terminator(OperatorType.Minus), "3404318e-e4c9-45b1-9569-32aeb85c2397"},}, new Dictionary<Terminator, ReduceInformation>{ }, "59a4aee5-b84f-4593-87a4-75a882f37d05") },
		{ "2a0c4c84-0d34-48b7-ac1e-6c4263554c2c", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Factor), "42a84635-03d1-42a5-9c25-cf101cde9d2b"}, { Terminator.NumberTerminator, "f9789d84-d480-461d-a4a2-0404654a02e1"}, { new NonTerminator(NonTerminatorType.Variable), "4056346c-1389-4501-9fa7-8d71820cf943"}, { new Terminator(DelimiterType.LeftParenthesis), "141303a8-ff5b-48d6-9e48-10252b617d09"}, { Terminator.IdentifierTerminator, "5b888b7a-23af-4ebe-a292-bafb94fabb7b"}, { new Terminator(KeywordType.Not), "df7787b9-6d11-4bb2-b89d-3527a0e1b4fb"}, { new Terminator(OperatorType.Minus), "3404318e-e4c9-45b1-9569-32aeb85c2397"},}, new Dictionary<Terminator, ReduceInformation>{ }, "2a0c4c84-0d34-48b7-ac1e-6c4263554c2c") },
		{ "67454e29-e959-4a3f-87ce-8095c42f3fa1", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.RightParenthesis), "c2e31402-c226-4545-9be0-759dfcd81339"},}, new Dictionary<Terminator, ReduceInformation>{ }, "67454e29-e959-4a3f-87ce-8095c42f3fa1") },
		{ "c067be76-c0c1-49fb-9ffd-3d978b7131d9", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.ExpressionList), "09352cee-2c04-4475-b2f9-4f57fd76d991"}, { new NonTerminator(NonTerminatorType.Expression), "4f59631a-5553-49da-9f0d-dda48b0d65b2"}, { new NonTerminator(NonTerminatorType.SimpleExpression), "e2a2e2c4-61e9-496f-8a60-ac108404247f"}, { new NonTerminator(NonTerminatorType.Term), "83d0456c-4817-49a5-bd20-c0d8a1c5b696"}, { new NonTerminator(NonTerminatorType.Factor), "f1db3e46-b446-4adc-93ae-5dfed0964c99"}, { Terminator.NumberTerminator, "0abfb7c0-083d-4242-a9a7-06183f7230f7"}, { new NonTerminator(NonTerminatorType.Variable), "a8c27dec-213a-4917-8b05-b0422260d1c1"}, { new Terminator(DelimiterType.LeftParenthesis), "45c4069f-f3cf-48f7-af29-8eb8303c47d6"}, { Terminator.IdentifierTerminator, "5aabfbf6-09f5-484f-9f95-ac0fa52c26c4"}, { new Terminator(KeywordType.Not), "253ff382-06bc-47a7-9645-cc04c48e1d0a"}, { new Terminator(OperatorType.Minus), "5708e399-73e5-4bcb-9aeb-f25d1e06c35c"},}, new Dictionary<Terminator, ReduceInformation>{ }, "c067be76-c0c1-49fb-9ffd-3d978b7131d9") },
		{ "a45a8ba3-16ac-4520-8fc7-86ebc8d7082a", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.Then), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(KeywordType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(KeywordType.Mod), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(KeywordType.And), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.Plus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.Minus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(KeywordType.Or), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, }, "a45a8ba3-16ac-4520-8fc7-86ebc8d7082a") },
		{ "ff8d8bb2-7c51-4e08-9145-b0bc208e3f5a", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.ExpressionList), "f101e13b-d089-42c8-9bef-cd57e53a54a0"}, { new NonTerminator(NonTerminatorType.Expression), "a0ba5247-cbd6-46f6-b1fd-574979eb2b27"}, { new NonTerminator(NonTerminatorType.SimpleExpression), "e8b0426e-8437-45f8-a02f-3eebdfb48e84"}, { new NonTerminator(NonTerminatorType.Term), "cd4bfe7a-5a2e-41f1-a891-6359311b9403"}, { new NonTerminator(NonTerminatorType.Factor), "ceb0b15a-7ff9-4265-8766-716485412510"}, { Terminator.NumberTerminator, "06d206a2-8c81-4399-8790-c903a8acfe21"}, { new NonTerminator(NonTerminatorType.Variable), "422866b2-29d6-4004-b762-cb9ad9ddc5e7"}, { new Terminator(DelimiterType.LeftParenthesis), "4f42a0a3-3c4b-46a5-8dcc-b91007f93d5c"}, { Terminator.IdentifierTerminator, "7339bcec-b0ae-4611-83cd-08345e44324f"}, { new Terminator(KeywordType.Not), "bfdaacfe-9460-4db8-9469-712b6674efb5"}, { new Terminator(OperatorType.Minus), "eac356e4-6098-43be-9b50-3ff1a637a898"},}, new Dictionary<Terminator, ReduceInformation>{ }, "ff8d8bb2-7c51-4e08-9145-b0bc208e3f5a") },
		{ "cf1104ab-9c32-43e7-80cd-17ddb5f313ff", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.Then), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, }, "cf1104ab-9c32-43e7-80cd-17ddb5f313ff") },
		{ "182d73f3-b8fa-428f-8894-ca7c3b587eaa", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.Then), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, }, "182d73f3-b8fa-428f-8894-ca7c3b587eaa") },
		{ "14c29054-6d68-488c-89c3-b347db913a3c", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.AddOperator), "0fe43d0d-66a7-4555-a250-6b3300e008ab"}, { new Terminator(OperatorType.Plus), "b2a223d6-6905-4b72-a67b-96d8604fe688"}, { new Terminator(OperatorType.Minus), "569b7d42-5c9a-4ba6-b124-8dacd7f387f4"}, { new Terminator(KeywordType.Or), "37aaac08-c0a9-420d-bfd1-1ad8615a7513"},}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightParenthesis), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Expression))}, }, "14c29054-6d68-488c-89c3-b347db913a3c") },
		{ "d185d164-373f-4e5b-bc4a-b15fba215e7f", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.MultiplyOperator), "a48ba1d6-7955-4c69-ac2a-e5cf8d58e33a"}, { new Terminator(OperatorType.Multiply), "e576c109-5246-4139-8248-415b8e3e68de"}, { new Terminator(OperatorType.Divide), "6a567f32-04a0-49b8-bf8e-42f55ee0600f"}, { new Terminator(KeywordType.Divide), "86be9c46-a085-4183-9264-b12d73fde46e"}, { new Terminator(KeywordType.Mod), "d7708b97-7da8-492a-a64f-6196ae9527f7"}, { new Terminator(KeywordType.And), "271d9f46-4be0-495c-a90b-ae8f19ee928b"},}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightParenthesis), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Plus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Minus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(KeywordType.Or), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, }, "d185d164-373f-4e5b-bc4a-b15fba215e7f") },
		{ "e469f27a-a5b7-4a0a-aa29-0092ba56b087", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightParenthesis), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Mod), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.And), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Plus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Minus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Or), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, }, "e469f27a-a5b7-4a0a-aa29-0092ba56b087") },
		{ "b0508395-27cc-4179-8a0f-1a5ac38d89b7", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightParenthesis), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, }, "b0508395-27cc-4179-8a0f-1a5ac38d89b7") },
		{ "6d5d615b-e7d3-422e-a5ba-123e617a33e0", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightParenthesis), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, }, "6d5d615b-e7d3-422e-a5ba-123e617a33e0") },
		{ "617493e7-522c-4e09-b525-6b29f2c96528", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Expression), "a7246b45-ae4e-485e-87c0-43c1e81d53c1"}, { new NonTerminator(NonTerminatorType.SimpleExpression), "e566a9ad-f17c-46ff-981f-ee4fa81d2d3d"}, { new NonTerminator(NonTerminatorType.Term), "1f1e4a05-7d30-44f4-aa5f-5caacfbce4b2"}, { new NonTerminator(NonTerminatorType.Factor), "ff72d68f-ac20-46e8-94ed-090e841128c1"}, { Terminator.NumberTerminator, "79939c29-990e-4743-8a8a-5fccd2ec415d"}, { new NonTerminator(NonTerminatorType.Variable), "28f501cb-0b97-40c3-8a64-d214832a6789"}, { new Terminator(DelimiterType.LeftParenthesis), "f1464d64-129c-4d68-a61e-43296e963ee1"}, { Terminator.IdentifierTerminator, "9dd45908-639c-4e69-97e4-2471cdf5e189"}, { new Terminator(KeywordType.Not), "303776ee-584f-4977-b76a-9449d6a4e1ad"}, { new Terminator(OperatorType.Minus), "2401b03b-4e43-4409-989c-a4471708a795"},}, new Dictionary<Terminator, ReduceInformation>{ }, "617493e7-522c-4e09-b525-6b29f2c96528") },
		{ "a8b3e8cf-a167-4e52-a876-56fc36b99e81", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.LeftParenthesis), "dd2f25ec-2fc5-49f5-9ad9-42cdbfadcdcf"}, { new NonTerminator(NonTerminatorType.IdVarPart), "6f6e0fef-9b3f-42d1-a8eb-dbd918b57142"}, { new Terminator(DelimiterType.LeftSquareBracket), "86d71483-4b09-4a09-b5e8-e97a0ae2fbe9"},}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightParenthesis), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Divide), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Divide), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Mod), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.And), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Plus), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Minus), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Or), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, }, "a8b3e8cf-a167-4e52-a876-56fc36b99e81") },
		{ "65e2d29a-99a0-4377-95af-9b1954c5e202", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Factor), "39060e84-cdeb-42ee-a24e-8a018b2b0944"}, { Terminator.NumberTerminator, "b0508395-27cc-4179-8a0f-1a5ac38d89b7"}, { new NonTerminator(NonTerminatorType.Variable), "6d5d615b-e7d3-422e-a5ba-123e617a33e0"}, { new Terminator(DelimiterType.LeftParenthesis), "617493e7-522c-4e09-b525-6b29f2c96528"}, { Terminator.IdentifierTerminator, "a8b3e8cf-a167-4e52-a876-56fc36b99e81"}, { new Terminator(KeywordType.Not), "65e2d29a-99a0-4377-95af-9b1954c5e202"}, { new Terminator(OperatorType.Minus), "dba55308-efb5-4ad7-8bee-f379f817f752"},}, new Dictionary<Terminator, ReduceInformation>{ }, "65e2d29a-99a0-4377-95af-9b1954c5e202") },
		{ "dba55308-efb5-4ad7-8bee-f379f817f752", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Factor), "95beb628-7860-4f11-9712-15c738b9c1cd"}, { Terminator.NumberTerminator, "b0508395-27cc-4179-8a0f-1a5ac38d89b7"}, { new NonTerminator(NonTerminatorType.Variable), "6d5d615b-e7d3-422e-a5ba-123e617a33e0"}, { new Terminator(DelimiterType.LeftParenthesis), "617493e7-522c-4e09-b525-6b29f2c96528"}, { Terminator.IdentifierTerminator, "a8b3e8cf-a167-4e52-a876-56fc36b99e81"}, { new Terminator(KeywordType.Not), "65e2d29a-99a0-4377-95af-9b1954c5e202"}, { new Terminator(OperatorType.Minus), "dba55308-efb5-4ad7-8bee-f379f817f752"},}, new Dictionary<Terminator, ReduceInformation>{ }, "dba55308-efb5-4ad7-8bee-f379f817f752") },
		{ "2a3db56a-f45c-49ef-a3d3-3276e864d148", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.MultiplyOperator), "d53ff53c-5245-4c95-86c0-b368cdebee74"}, { new Terminator(OperatorType.Multiply), "e576c109-5246-4139-8248-415b8e3e68de"}, { new Terminator(OperatorType.Divide), "6a567f32-04a0-49b8-bf8e-42f55ee0600f"}, { new Terminator(KeywordType.Divide), "86be9c46-a085-4183-9264-b12d73fde46e"}, { new Terminator(KeywordType.Mod), "d7708b97-7da8-492a-a64f-6196ae9527f7"}, { new Terminator(KeywordType.And), "271d9f46-4be0-495c-a90b-ae8f19ee928b"},}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightParenthesis), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Equal), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Less), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Greater), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Plus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Minus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(KeywordType.Or), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, }, "2a3db56a-f45c-49ef-a3d3-3276e864d148") },
		{ "255c54b3-cc93-438f-805c-6c4751f74a07", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightParenthesis), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Equal), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Less), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Greater), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Mod), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.And), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Plus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Minus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Or), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, }, "255c54b3-cc93-438f-805c-6c4751f74a07") },
		{ "b4d256a5-9ef4-4102-83f5-8f89facde412", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightParenthesis), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Equal), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Less), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Greater), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, }, "b4d256a5-9ef4-4102-83f5-8f89facde412") },
		{ "98477bb8-54b2-403f-a917-742fcb30f71f", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.RightParenthesis), "08a8ab6c-dd7e-46cf-9145-20af1cb47384"}, { new Terminator(DelimiterType.Comma), "077bd83e-2dfc-4686-bf99-ec6992c05b60"},}, new Dictionary<Terminator, ReduceInformation>{ }, "98477bb8-54b2-403f-a917-742fcb30f71f") },
		{ "da77dc61-b08b-4ce3-80a1-baef83e7ea3e", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.RightSquareBracket), "4e4b9686-7296-431d-b4f9-d9feb8b73e03"}, { new Terminator(DelimiterType.Comma), "082dc96a-aa2d-444e-b2b0-8a6bca7f6ba2"},}, new Dictionary<Terminator, ReduceInformation>{ }, "da77dc61-b08b-4ce3-80a1-baef83e7ea3e") },
		{ "8fa4269c-234c-4ef1-9b8d-c0efe7b89c92", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.Then), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Equal), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Less), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Greater), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, }, "8fa4269c-234c-4ef1-9b8d-c0efe7b89c92") },
		{ "9b9df458-8d8f-4d49-844f-6b254033b604", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.Then), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Equal), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Less), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Greater), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Mod), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.And), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Plus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Minus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Or), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, }, "9b9df458-8d8f-4d49-844f-6b254033b604") },
		{ "71db17f2-1dde-4f6a-ac54-28213b704612", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Expression), "c937afc1-3473-48bd-b95f-77b995ca08c8"}, { new NonTerminator(NonTerminatorType.SimpleExpression), "edabf765-08f8-4d58-8722-863c2b38bf71"}, { new NonTerminator(NonTerminatorType.Term), "96b22d74-a3ce-42d1-980f-ab9ca7edff4d"}, { new NonTerminator(NonTerminatorType.Factor), "a7870627-59bc-434d-bf03-0c1d5b649d45"}, { Terminator.NumberTerminator, "906b0111-f667-467d-a74a-c2be40fb7905"}, { new NonTerminator(NonTerminatorType.Variable), "ecc69635-01d5-4cdf-aaa8-750d1fa736f1"}, { new Terminator(DelimiterType.LeftParenthesis), "e41a1788-a93a-4fb3-846e-a2d33bd77958"}, { Terminator.IdentifierTerminator, "71f58c48-8093-40c7-b29d-0e1099642b47"}, { new Terminator(KeywordType.Not), "c5c60bb4-0f4e-4e7b-a222-1e9f7c12472d"}, { new Terminator(OperatorType.Minus), "cb5f054d-2d9d-417c-8843-2156b2d466a7"},}, new Dictionary<Terminator, ReduceInformation>{ }, "71db17f2-1dde-4f6a-ac54-28213b704612") },
		{ "806b2d65-cc39-4355-addb-97d15dfea312", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.SimpleExpression), "5f59a76e-a0df-4ac7-a095-ae456f4a1564"}, { new NonTerminator(NonTerminatorType.Term), "475339af-ff8f-4fac-b64c-01fe36f26db0"}, { new NonTerminator(NonTerminatorType.Factor), "aaa1c6ff-81ea-415b-a7f2-0ede345af1dd"}, { Terminator.NumberTerminator, "f62db516-a4fe-45b8-8c43-7013e181d8d1"}, { new NonTerminator(NonTerminatorType.Variable), "e1c60364-c243-481b-b5ad-fac016bd7b32"}, { new Terminator(DelimiterType.LeftParenthesis), "d2ba8656-b05e-4fea-9aeb-3e7e5aef0cd3"}, { Terminator.IdentifierTerminator, "e81de486-3409-4f08-98da-f145bd2fb370"}, { new Terminator(KeywordType.Not), "0b574dc0-93f9-4835-b426-8a0d43f17059"}, { new Terminator(OperatorType.Minus), "b1e61a3b-133a-4ff2-bc0c-e4580ad4a90a"},}, new Dictionary<Terminator, ReduceInformation>{ }, "806b2d65-cc39-4355-addb-97d15dfea312") },
		{ "a22c3e34-c025-417e-b171-36b1c26a94dd", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Term), "b69dc680-816c-4e2d-9c51-145d5c2a980b"}, { new NonTerminator(NonTerminatorType.Factor), "e2eecffb-85c1-440e-9c45-1fdd778c942a"}, { Terminator.NumberTerminator, "656892f9-1aaa-4635-84ff-c12d4e1a0c2e"}, { new NonTerminator(NonTerminatorType.Variable), "fd6b2f33-2a46-4dc5-93fc-0b7816be2582"}, { new Terminator(DelimiterType.LeftParenthesis), "079e396f-7464-432e-8666-7fe4b0ca40a7"}, { Terminator.IdentifierTerminator, "902c5106-8d85-47b7-b482-58ed10129c15"}, { new Terminator(KeywordType.Not), "b715b0a1-976c-42f9-84d7-4e3968a3369f"}, { new Terminator(OperatorType.Minus), "b8cd8bf3-bd87-4006-b6a9-4f0305e07f51"},}, new Dictionary<Terminator, ReduceInformation>{ }, "a22c3e34-c025-417e-b171-36b1c26a94dd") },
		{ "a2d36e1c-1210-4175-a84b-9df1fe589223", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Factor), "0aedeb64-c294-4d32-8738-24b107274f8d"}, { Terminator.NumberTerminator, "656892f9-1aaa-4635-84ff-c12d4e1a0c2e"}, { new NonTerminator(NonTerminatorType.Variable), "fd6b2f33-2a46-4dc5-93fc-0b7816be2582"}, { new Terminator(DelimiterType.LeftParenthesis), "079e396f-7464-432e-8666-7fe4b0ca40a7"}, { Terminator.IdentifierTerminator, "902c5106-8d85-47b7-b482-58ed10129c15"}, { new Terminator(KeywordType.Not), "b715b0a1-976c-42f9-84d7-4e3968a3369f"}, { new Terminator(OperatorType.Minus), "b8cd8bf3-bd87-4006-b6a9-4f0305e07f51"},}, new Dictionary<Terminator, ReduceInformation>{ }, "a2d36e1c-1210-4175-a84b-9df1fe589223") },
		{ "bc5aaf35-5671-4613-9b9c-1590c9882f5c", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.RightParenthesis), "eee81e2f-ca56-4fc9-a31b-7e8616370f67"},}, new Dictionary<Terminator, ReduceInformation>{ }, "bc5aaf35-5671-4613-9b9c-1590c9882f5c") },
		{ "6fac19b0-1bb1-4052-b3c5-882b032d0029", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.ExpressionList), "051cd12a-adfd-4234-89d2-238b590e7687"}, { new NonTerminator(NonTerminatorType.Expression), "4f59631a-5553-49da-9f0d-dda48b0d65b2"}, { new NonTerminator(NonTerminatorType.SimpleExpression), "e2a2e2c4-61e9-496f-8a60-ac108404247f"}, { new NonTerminator(NonTerminatorType.Term), "83d0456c-4817-49a5-bd20-c0d8a1c5b696"}, { new NonTerminator(NonTerminatorType.Factor), "f1db3e46-b446-4adc-93ae-5dfed0964c99"}, { Terminator.NumberTerminator, "0abfb7c0-083d-4242-a9a7-06183f7230f7"}, { new NonTerminator(NonTerminatorType.Variable), "a8c27dec-213a-4917-8b05-b0422260d1c1"}, { new Terminator(DelimiterType.LeftParenthesis), "45c4069f-f3cf-48f7-af29-8eb8303c47d6"}, { Terminator.IdentifierTerminator, "5aabfbf6-09f5-484f-9f95-ac0fa52c26c4"}, { new Terminator(KeywordType.Not), "253ff382-06bc-47a7-9645-cc04c48e1d0a"}, { new Terminator(OperatorType.Minus), "5708e399-73e5-4bcb-9aeb-f25d1e06c35c"},}, new Dictionary<Terminator, ReduceInformation>{ }, "6fac19b0-1bb1-4052-b3c5-882b032d0029") },
		{ "e2eeef26-5763-4d1c-b737-96be01a478bf", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.To), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.Equal), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.Less), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.Greater), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(KeywordType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(KeywordType.Mod), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(KeywordType.And), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.Plus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.Minus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(KeywordType.Or), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, }, "e2eeef26-5763-4d1c-b737-96be01a478bf") },
		{ "5444abce-0a3e-40bd-ab62-c420bab088e1", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.ExpressionList), "6baa2e2e-995a-4434-b8c2-e80b1bef7a42"}, { new NonTerminator(NonTerminatorType.Expression), "a0ba5247-cbd6-46f6-b1fd-574979eb2b27"}, { new NonTerminator(NonTerminatorType.SimpleExpression), "e8b0426e-8437-45f8-a02f-3eebdfb48e84"}, { new NonTerminator(NonTerminatorType.Term), "cd4bfe7a-5a2e-41f1-a891-6359311b9403"}, { new NonTerminator(NonTerminatorType.Factor), "ceb0b15a-7ff9-4265-8766-716485412510"}, { Terminator.NumberTerminator, "06d206a2-8c81-4399-8790-c903a8acfe21"}, { new NonTerminator(NonTerminatorType.Variable), "422866b2-29d6-4004-b762-cb9ad9ddc5e7"}, { new Terminator(DelimiterType.LeftParenthesis), "4f42a0a3-3c4b-46a5-8dcc-b91007f93d5c"}, { Terminator.IdentifierTerminator, "7339bcec-b0ae-4611-83cd-08345e44324f"}, { new Terminator(KeywordType.Not), "bfdaacfe-9460-4db8-9469-712b6674efb5"}, { new Terminator(OperatorType.Minus), "eac356e4-6098-43be-9b50-3ff1a637a898"},}, new Dictionary<Terminator, ReduceInformation>{ }, "5444abce-0a3e-40bd-ab62-c420bab088e1") },
		{ "f77a3c90-e980-406e-b0e2-e373308241fa", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.To), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Equal), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Less), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Greater), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, }, "f77a3c90-e980-406e-b0e2-e373308241fa") },
		{ "255e6483-e7ed-49d3-affc-d58bbf391c0d", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.To), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Equal), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Less), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Greater), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, }, "255e6483-e7ed-49d3-affc-d58bbf391c0d") },
		{ "3a25beb9-a581-4483-ab34-73483caedf0c", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(KeywordType.End), "1f4f1827-a7f7-41ff-968f-07d84486ed66"}, { new Terminator(DelimiterType.Semicolon), "41bdccbb-3f79-4d74-9b06-5a48b781a2c2"},}, new Dictionary<Terminator, ReduceInformation>{ }, "3a25beb9-a581-4483-ab34-73483caedf0c") },
		{ "ca200296-ea1c-456f-a213-2310dad1fac7", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.IdentifierList), "7080b182-243c-4083-8e43-08fb4cd8954a"}, { Terminator.IdentifierTerminator, "c50004c4-fac8-4812-a2e2-6467f2005495"},}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.Begin), new ReduceInformation(3, new NonTerminator(NonTerminatorType.VarDeclarations))}, }, "ca200296-ea1c-456f-a213-2310dad1fac7") },
		{ "a40fe53b-e5f3-4539-8924-d66d122f99a7", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightParenthesis), new ReduceInformation(3, new NonTerminator(NonTerminatorType.ParameterList))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(3, new NonTerminator(NonTerminatorType.ParameterList))}, }, "a40fe53b-e5f3-4539-8924-d66d122f99a7") },
		{ "380ebab4-0036-4a21-9aa6-f5371bed4cc8", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightParenthesis), new ReduceInformation(3, new NonTerminator(NonTerminatorType.ValueParameter))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(3, new NonTerminator(NonTerminatorType.ValueParameter))}, }, "380ebab4-0036-4a21-9aa6-f5371bed4cc8") },
		{ "e4b6a6bc-d2bc-476b-a5a0-311483e9d80d", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightParenthesis), new ReduceInformation(1, new NonTerminator(NonTerminatorType.BasicType))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(1, new NonTerminator(NonTerminatorType.BasicType))}, }, "e4b6a6bc-d2bc-476b-a5a0-311483e9d80d") },
		{ "c6cc9782-a41f-4889-bcbb-946126edec40", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightParenthesis), new ReduceInformation(1, new NonTerminator(NonTerminatorType.BasicType))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(1, new NonTerminator(NonTerminatorType.BasicType))}, }, "c6cc9782-a41f-4889-bcbb-946126edec40") },
		{ "8cfac156-0b52-4e72-9664-8957b522fc7c", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightParenthesis), new ReduceInformation(1, new NonTerminator(NonTerminatorType.BasicType))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(1, new NonTerminator(NonTerminatorType.BasicType))}, }, "8cfac156-0b52-4e72-9664-8957b522fc7c") },
		{ "1e1234cc-b049-4146-836a-ef291747ebb6", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightParenthesis), new ReduceInformation(1, new NonTerminator(NonTerminatorType.BasicType))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(1, new NonTerminator(NonTerminatorType.BasicType))}, }, "1e1234cc-b049-4146-836a-ef291747ebb6") },
		{ "75999946-7bf8-4c24-9606-df0f9038f602", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.BasicType), "ad6e6545-4780-4630-9eb3-e455f9525849"}, { new Terminator(KeywordType.Integer), "7bb70cf0-2067-4971-ada1-ccf973097e8b"}, { new Terminator(KeywordType.Real), "1023abfc-7be1-4c20-890e-3ff30b2c89fc"}, { new Terminator(KeywordType.Boolean), "7430a84c-7617-405a-9992-d202dcbc8adc"}, { new Terminator(KeywordType.Character), "c1bdaca1-017b-4856-84d6-4cd20b5911a3"},}, new Dictionary<Terminator, ReduceInformation>{ }, "75999946-7bf8-4c24-9606-df0f9038f602") },
		{ "096a7769-5bc2-4e3a-92d8-7fa0d5e764bf", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.DoubleDots), "e9c44581-4124-4d63-9fb0-7b4cafb8d601"},}, new Dictionary<Terminator, ReduceInformation>{ }, "096a7769-5bc2-4e3a-92d8-7fa0d5e764bf") },
		{ "92b1055b-bbdb-4853-beb5-9f0b3b2c5cd0", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightSquareBracket), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Period))}, { new Terminator(DelimiterType.Comma), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Period))}, }, "92b1055b-bbdb-4853-beb5-9f0b3b2c5cd0") },
		{ "0efc69e2-dd61-45f4-9e36-bd84f61b8217", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Term), "c0fddf91-3585-48d3-9b2f-fa5838ce5bbe"}, { new NonTerminator(NonTerminatorType.Factor), "62c5c2fb-98a9-4f73-96af-5b4f878d8429"}, { Terminator.NumberTerminator, "758ac2f2-d58c-4624-8756-aba601efb559"}, { new NonTerminator(NonTerminatorType.Variable), "cddf29be-3197-4140-9341-7394cab82f23"}, { new Terminator(DelimiterType.LeftParenthesis), "7a95618a-8bb9-488b-848f-86edd4edb091"}, { Terminator.IdentifierTerminator, "b294720f-aa1c-4977-9724-f1338cad4cc9"}, { new Terminator(KeywordType.Not), "28b7637d-584d-48e7-8493-444cae07acbc"}, { new Terminator(OperatorType.Minus), "f10f52a0-dbc4-4505-90ef-c50f0d3044cd"},}, new Dictionary<Terminator, ReduceInformation>{ }, "0efc69e2-dd61-45f4-9e36-bd84f61b8217") },
		{ "2f27ae56-216f-4e19-b722-ed4dd82cebbb", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Factor), "64b11017-23c6-4007-ae3d-43b98aa96cbf"}, { Terminator.NumberTerminator, "758ac2f2-d58c-4624-8756-aba601efb559"}, { new NonTerminator(NonTerminatorType.Variable), "cddf29be-3197-4140-9341-7394cab82f23"}, { new Terminator(DelimiterType.LeftParenthesis), "7a95618a-8bb9-488b-848f-86edd4edb091"}, { Terminator.IdentifierTerminator, "b294720f-aa1c-4977-9724-f1338cad4cc9"}, { new Terminator(KeywordType.Not), "28b7637d-584d-48e7-8493-444cae07acbc"}, { new Terminator(OperatorType.Minus), "f10f52a0-dbc4-4505-90ef-c50f0d3044cd"},}, new Dictionary<Terminator, ReduceInformation>{ }, "2f27ae56-216f-4e19-b722-ed4dd82cebbb") },
		{ "f584e8fa-31a6-4c8b-a675-9718057f4aa4", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.RightParenthesis), "b6dc85b0-25f6-4140-b85f-8fcc92436e6e"},}, new Dictionary<Terminator, ReduceInformation>{ }, "f584e8fa-31a6-4c8b-a675-9718057f4aa4") },
		{ "829f5068-893b-4ca4-9c7e-f42656c66be5", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.ExpressionList), "083d8249-ed1c-42c0-98c8-92237661ef9a"}, { new NonTerminator(NonTerminatorType.Expression), "4f59631a-5553-49da-9f0d-dda48b0d65b2"}, { new NonTerminator(NonTerminatorType.SimpleExpression), "e2a2e2c4-61e9-496f-8a60-ac108404247f"}, { new NonTerminator(NonTerminatorType.Term), "83d0456c-4817-49a5-bd20-c0d8a1c5b696"}, { new NonTerminator(NonTerminatorType.Factor), "f1db3e46-b446-4adc-93ae-5dfed0964c99"}, { Terminator.NumberTerminator, "0abfb7c0-083d-4242-a9a7-06183f7230f7"}, { new NonTerminator(NonTerminatorType.Variable), "a8c27dec-213a-4917-8b05-b0422260d1c1"}, { new Terminator(DelimiterType.LeftParenthesis), "45c4069f-f3cf-48f7-af29-8eb8303c47d6"}, { Terminator.IdentifierTerminator, "5aabfbf6-09f5-484f-9f95-ac0fa52c26c4"}, { new Terminator(KeywordType.Not), "253ff382-06bc-47a7-9645-cc04c48e1d0a"}, { new Terminator(OperatorType.Minus), "5708e399-73e5-4bcb-9aeb-f25d1e06c35c"},}, new Dictionary<Terminator, ReduceInformation>{ }, "829f5068-893b-4ca4-9c7e-f42656c66be5") },
		{ "d891e285-c4e8-4851-a322-ef787d98617a", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(KeywordType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(KeywordType.Mod), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(KeywordType.And), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.Plus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.Minus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(KeywordType.Or), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, }, "d891e285-c4e8-4851-a322-ef787d98617a") },
		{ "aa6cb10e-8b1e-4838-9859-5f24d9f7dc9c", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.ExpressionList), "1ec358e9-0a98-4c60-a070-c1529d2559bf"}, { new NonTerminator(NonTerminatorType.Expression), "a0ba5247-cbd6-46f6-b1fd-574979eb2b27"}, { new NonTerminator(NonTerminatorType.SimpleExpression), "e8b0426e-8437-45f8-a02f-3eebdfb48e84"}, { new NonTerminator(NonTerminatorType.Term), "cd4bfe7a-5a2e-41f1-a891-6359311b9403"}, { new NonTerminator(NonTerminatorType.Factor), "ceb0b15a-7ff9-4265-8766-716485412510"}, { Terminator.NumberTerminator, "06d206a2-8c81-4399-8790-c903a8acfe21"}, { new NonTerminator(NonTerminatorType.Variable), "422866b2-29d6-4004-b762-cb9ad9ddc5e7"}, { new Terminator(DelimiterType.LeftParenthesis), "4f42a0a3-3c4b-46a5-8dcc-b91007f93d5c"}, { Terminator.IdentifierTerminator, "7339bcec-b0ae-4611-83cd-08345e44324f"}, { new Terminator(KeywordType.Not), "bfdaacfe-9460-4db8-9469-712b6674efb5"}, { new Terminator(OperatorType.Minus), "eac356e4-6098-43be-9b50-3ff1a637a898"},}, new Dictionary<Terminator, ReduceInformation>{ }, "aa6cb10e-8b1e-4838-9859-5f24d9f7dc9c") },
		{ "4bfa1678-d410-43d5-b40a-fd6324530c55", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, }, "4bfa1678-d410-43d5-b40a-fd6324530c55") },
		{ "641501ea-443a-43d3-b0d4-1b158a4ecd0d", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, }, "641501ea-443a-43d3-b0d4-1b158a4ecd0d") },
		{ "714c11fa-d1cb-49e9-b85e-de43a21c70eb", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Equal), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Less), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Greater), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, }, "714c11fa-d1cb-49e9-b85e-de43a21c70eb") },
		{ "52f8dc5c-e7ff-4788-bb96-c1ec6582e0a4", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Equal), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Less), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Greater), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Mod), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.And), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Plus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Minus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Or), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, }, "52f8dc5c-e7ff-4788-bb96-c1ec6582e0a4") },
		{ "cb32f811-1f28-49cc-be2b-da3c842b47a8", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Term), "489a9a58-71fe-42a8-a046-0e59b767b261"}, { new NonTerminator(NonTerminatorType.Factor), "76c787f9-ae62-41d2-a3fd-4902c82e377b"}, { Terminator.NumberTerminator, "633178ec-3161-44c4-a90f-74a17a450286"}, { new NonTerminator(NonTerminatorType.Variable), "1490c0e9-6dd0-4710-8d43-856abc874cb6"}, { new Terminator(DelimiterType.LeftParenthesis), "f3f1b924-071c-4005-91d3-f9ac72d17c33"}, { Terminator.IdentifierTerminator, "dbede38f-60d1-4090-bfcf-5aab43d40fdd"}, { new Terminator(KeywordType.Not), "e3da8607-d17a-4bde-a57a-3c247abe7e39"}, { new Terminator(OperatorType.Minus), "01a95a22-0a4a-4fc4-b510-2d88603ba6c8"},}, new Dictionary<Terminator, ReduceInformation>{ }, "cb32f811-1f28-49cc-be2b-da3c842b47a8") },
		{ "fda8ff9f-0d9c-4a54-9207-d10bf57748e1", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Factor), "49e98f98-fe8e-414f-b78e-507b72727731"}, { Terminator.NumberTerminator, "633178ec-3161-44c4-a90f-74a17a450286"}, { new NonTerminator(NonTerminatorType.Variable), "1490c0e9-6dd0-4710-8d43-856abc874cb6"}, { new Terminator(DelimiterType.LeftParenthesis), "f3f1b924-071c-4005-91d3-f9ac72d17c33"}, { Terminator.IdentifierTerminator, "dbede38f-60d1-4090-bfcf-5aab43d40fdd"}, { new Terminator(KeywordType.Not), "e3da8607-d17a-4bde-a57a-3c247abe7e39"}, { new Terminator(OperatorType.Minus), "01a95a22-0a4a-4fc4-b510-2d88603ba6c8"},}, new Dictionary<Terminator, ReduceInformation>{ }, "fda8ff9f-0d9c-4a54-9207-d10bf57748e1") },
		{ "f3bcbd79-4d59-4161-b974-473ba5208d1b", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.RightParenthesis), "ffd07de0-d88a-4227-bd15-c48495e37164"},}, new Dictionary<Terminator, ReduceInformation>{ }, "f3bcbd79-4d59-4161-b974-473ba5208d1b") },
		{ "e5a843a5-4497-40f8-bed7-9ffb986ea6ca", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.ExpressionList), "dc755815-5742-4ca8-a2c2-bc83ee7bef33"}, { new NonTerminator(NonTerminatorType.Expression), "4f59631a-5553-49da-9f0d-dda48b0d65b2"}, { new NonTerminator(NonTerminatorType.SimpleExpression), "e2a2e2c4-61e9-496f-8a60-ac108404247f"}, { new NonTerminator(NonTerminatorType.Term), "83d0456c-4817-49a5-bd20-c0d8a1c5b696"}, { new NonTerminator(NonTerminatorType.Factor), "f1db3e46-b446-4adc-93ae-5dfed0964c99"}, { Terminator.NumberTerminator, "0abfb7c0-083d-4242-a9a7-06183f7230f7"}, { new NonTerminator(NonTerminatorType.Variable), "a8c27dec-213a-4917-8b05-b0422260d1c1"}, { new Terminator(DelimiterType.LeftParenthesis), "45c4069f-f3cf-48f7-af29-8eb8303c47d6"}, { Terminator.IdentifierTerminator, "5aabfbf6-09f5-484f-9f95-ac0fa52c26c4"}, { new Terminator(KeywordType.Not), "253ff382-06bc-47a7-9645-cc04c48e1d0a"}, { new Terminator(OperatorType.Minus), "5708e399-73e5-4bcb-9aeb-f25d1e06c35c"},}, new Dictionary<Terminator, ReduceInformation>{ }, "e5a843a5-4497-40f8-bed7-9ffb986ea6ca") },
		{ "9821331b-3752-4fcc-a594-ee2086beed82", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightSquareBracket), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(KeywordType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(KeywordType.Mod), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(KeywordType.And), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.Plus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.Minus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(KeywordType.Or), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(DelimiterType.Comma), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, }, "9821331b-3752-4fcc-a594-ee2086beed82") },
		{ "1019d72f-ddf3-4512-860c-c92bd9bb70ef", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.ExpressionList), "2af7807b-4b78-4fe8-889c-1623f0065ab0"}, { new NonTerminator(NonTerminatorType.Expression), "a0ba5247-cbd6-46f6-b1fd-574979eb2b27"}, { new NonTerminator(NonTerminatorType.SimpleExpression), "e8b0426e-8437-45f8-a02f-3eebdfb48e84"}, { new NonTerminator(NonTerminatorType.Term), "cd4bfe7a-5a2e-41f1-a891-6359311b9403"}, { new NonTerminator(NonTerminatorType.Factor), "ceb0b15a-7ff9-4265-8766-716485412510"}, { Terminator.NumberTerminator, "06d206a2-8c81-4399-8790-c903a8acfe21"}, { new NonTerminator(NonTerminatorType.Variable), "422866b2-29d6-4004-b762-cb9ad9ddc5e7"}, { new Terminator(DelimiterType.LeftParenthesis), "4f42a0a3-3c4b-46a5-8dcc-b91007f93d5c"}, { Terminator.IdentifierTerminator, "7339bcec-b0ae-4611-83cd-08345e44324f"}, { new Terminator(KeywordType.Not), "bfdaacfe-9460-4db8-9469-712b6674efb5"}, { new Terminator(OperatorType.Minus), "eac356e4-6098-43be-9b50-3ff1a637a898"},}, new Dictionary<Terminator, ReduceInformation>{ }, "1019d72f-ddf3-4512-860c-c92bd9bb70ef") },
		{ "e97b41de-7b5d-455c-995b-d3c3b5d34e08", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightSquareBracket), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(DelimiterType.Comma), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, }, "e97b41de-7b5d-455c-995b-d3c3b5d34e08") },
		{ "8a4b12cc-a316-4835-8228-ef65e5f01612", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightSquareBracket), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(DelimiterType.Comma), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, }, "8a4b12cc-a316-4835-8228-ef65e5f01612") },
		{ "513889a7-e087-48ca-af5e-b92f1bd4cf06", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightSquareBracket), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Equal), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Less), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Greater), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(DelimiterType.Comma), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, }, "513889a7-e087-48ca-af5e-b92f1bd4cf06") },
		{ "379270a2-7f40-40a1-b030-e8416dd5dd70", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightSquareBracket), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Equal), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Less), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Greater), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Mod), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.And), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Plus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Minus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Or), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(DelimiterType.Comma), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, }, "379270a2-7f40-40a1-b030-e8416dd5dd70") },
		{ "08b76352-9cbd-4207-addf-e1df764428cf", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Term), "8da10347-87c7-4248-85fc-5c2724d54627"}, { new NonTerminator(NonTerminatorType.Factor), "943c00cb-ed4b-4d82-a518-e4dc6503a69b"}, { Terminator.NumberTerminator, "29cbbe86-9ccc-4bc4-8d0e-87b4ee384c1c"}, { new NonTerminator(NonTerminatorType.Variable), "6df173e5-a9e3-4900-b741-a97da24e5875"}, { new Terminator(DelimiterType.LeftParenthesis), "90be55bd-50ad-4d9b-881d-a98f6114b390"}, { Terminator.IdentifierTerminator, "331ff71b-3088-4da0-acd4-81f2ef4930a1"}, { new Terminator(KeywordType.Not), "5c70b501-eb1c-4473-98cb-c37355c66519"}, { new Terminator(OperatorType.Minus), "705a9f2b-51ea-4a38-9368-d3f8e27b2ed4"},}, new Dictionary<Terminator, ReduceInformation>{ }, "08b76352-9cbd-4207-addf-e1df764428cf") },
		{ "a37c0513-7bbf-49ad-8a65-73ce9ee54a69", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Factor), "92e5c3df-1503-47ec-9f00-e45f84ef5001"}, { Terminator.NumberTerminator, "29cbbe86-9ccc-4bc4-8d0e-87b4ee384c1c"}, { new NonTerminator(NonTerminatorType.Variable), "6df173e5-a9e3-4900-b741-a97da24e5875"}, { new Terminator(DelimiterType.LeftParenthesis), "90be55bd-50ad-4d9b-881d-a98f6114b390"}, { Terminator.IdentifierTerminator, "331ff71b-3088-4da0-acd4-81f2ef4930a1"}, { new Terminator(KeywordType.Not), "5c70b501-eb1c-4473-98cb-c37355c66519"}, { new Terminator(OperatorType.Minus), "705a9f2b-51ea-4a38-9368-d3f8e27b2ed4"},}, new Dictionary<Terminator, ReduceInformation>{ }, "a37c0513-7bbf-49ad-8a65-73ce9ee54a69") },
		{ "10a0b5b2-20c1-45cc-a771-9db9e18dc2ad", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.RightParenthesis), "519d03d5-16d4-404d-bf9b-96b9e663743e"},}, new Dictionary<Terminator, ReduceInformation>{ }, "10a0b5b2-20c1-45cc-a771-9db9e18dc2ad") },
		{ "4b2e3c36-ab02-4972-81f2-7d5af3fb02a1", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.ExpressionList), "aa1370ba-efea-43e6-ae33-0a00afafc098"}, { new NonTerminator(NonTerminatorType.Expression), "4f59631a-5553-49da-9f0d-dda48b0d65b2"}, { new NonTerminator(NonTerminatorType.SimpleExpression), "e2a2e2c4-61e9-496f-8a60-ac108404247f"}, { new NonTerminator(NonTerminatorType.Term), "83d0456c-4817-49a5-bd20-c0d8a1c5b696"}, { new NonTerminator(NonTerminatorType.Factor), "f1db3e46-b446-4adc-93ae-5dfed0964c99"}, { Terminator.NumberTerminator, "0abfb7c0-083d-4242-a9a7-06183f7230f7"}, { new NonTerminator(NonTerminatorType.Variable), "a8c27dec-213a-4917-8b05-b0422260d1c1"}, { new Terminator(DelimiterType.LeftParenthesis), "45c4069f-f3cf-48f7-af29-8eb8303c47d6"}, { Terminator.IdentifierTerminator, "5aabfbf6-09f5-484f-9f95-ac0fa52c26c4"}, { new Terminator(KeywordType.Not), "253ff382-06bc-47a7-9645-cc04c48e1d0a"}, { new Terminator(OperatorType.Minus), "5708e399-73e5-4bcb-9aeb-f25d1e06c35c"},}, new Dictionary<Terminator, ReduceInformation>{ }, "4b2e3c36-ab02-4972-81f2-7d5af3fb02a1") },
		{ "6101cf2b-3c08-46a4-b42f-cdbf32a372e9", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightParenthesis), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(KeywordType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(KeywordType.Mod), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(KeywordType.And), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.Plus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.Minus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(KeywordType.Or), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(DelimiterType.Comma), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, }, "6101cf2b-3c08-46a4-b42f-cdbf32a372e9") },
		{ "ff3e717b-816c-4fc4-b8fe-21ad89303b12", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.ExpressionList), "81dd75f5-6044-4c57-9b8f-79de660a2a26"}, { new NonTerminator(NonTerminatorType.Expression), "a0ba5247-cbd6-46f6-b1fd-574979eb2b27"}, { new NonTerminator(NonTerminatorType.SimpleExpression), "e8b0426e-8437-45f8-a02f-3eebdfb48e84"}, { new NonTerminator(NonTerminatorType.Term), "cd4bfe7a-5a2e-41f1-a891-6359311b9403"}, { new NonTerminator(NonTerminatorType.Factor), "ceb0b15a-7ff9-4265-8766-716485412510"}, { Terminator.NumberTerminator, "06d206a2-8c81-4399-8790-c903a8acfe21"}, { new NonTerminator(NonTerminatorType.Variable), "422866b2-29d6-4004-b762-cb9ad9ddc5e7"}, { new Terminator(DelimiterType.LeftParenthesis), "4f42a0a3-3c4b-46a5-8dcc-b91007f93d5c"}, { Terminator.IdentifierTerminator, "7339bcec-b0ae-4611-83cd-08345e44324f"}, { new Terminator(KeywordType.Not), "bfdaacfe-9460-4db8-9469-712b6674efb5"}, { new Terminator(OperatorType.Minus), "eac356e4-6098-43be-9b50-3ff1a637a898"},}, new Dictionary<Terminator, ReduceInformation>{ }, "ff3e717b-816c-4fc4-b8fe-21ad89303b12") },
		{ "0cb854f1-c280-4dd4-99d4-d5a3b2c379a4", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightParenthesis), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(DelimiterType.Comma), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, }, "0cb854f1-c280-4dd4-99d4-d5a3b2c379a4") },
		{ "22d7d273-e8a4-44d8-8b90-50434f32c8fd", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightParenthesis), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(DelimiterType.Comma), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, }, "22d7d273-e8a4-44d8-8b90-50434f32c8fd") },
		{ "70e2a79d-494e-480c-ad7b-8ef65c844484", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightParenthesis), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Equal), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Less), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Greater), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(DelimiterType.Comma), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, }, "70e2a79d-494e-480c-ad7b-8ef65c844484") },
		{ "33dbbefe-a95a-4d00-9d08-ae26273859dc", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightParenthesis), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Equal), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Less), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Greater), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Mod), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.And), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Plus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Minus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Or), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(DelimiterType.Comma), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, }, "33dbbefe-a95a-4d00-9d08-ae26273859dc") },
		{ "8e3f9ecd-6c3f-411a-a840-ca18b76e185a", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(2, new NonTerminator(NonTerminatorType.ElsePart))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(2, new NonTerminator(NonTerminatorType.ElsePart))}, }, "8e3f9ecd-6c3f-411a-a840-ca18b76e185a") },
		{ "8f395f71-8009-423b-a6eb-9c3e7c865f26", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Statement))}, { new Terminator(KeywordType.Else), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Statement))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Statement))}, }, "8f395f71-8009-423b-a6eb-9c3e7c865f26") },
		{ "ff5e3d10-5b5d-425f-ab1f-9fdd2a3cc3fd", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.RelationOperator), "bdc6514f-a58e-4634-8a6f-bcc320b035bf"}, { new Terminator(OperatorType.Equal), "1b49be29-a9d8-403a-b434-5c116300d15c"}, { new Terminator(OperatorType.NotEqual), "fb7de677-df97-4431-b5b7-086a8ded26ff"}, { new Terminator(OperatorType.Less), "8db4bd06-8fe0-4476-924a-3354772a5ac3"}, { new Terminator(OperatorType.LessEqual), "68ce8cad-2c90-4f71-822f-885e193cc394"}, { new Terminator(OperatorType.Greater), "f863fa1a-f90f-430f-8ee8-4cb33053ee45"}, { new Terminator(OperatorType.GreaterEqual), "d79c0f7f-9997-421d-bed4-1e0b2517774c"}, { new NonTerminator(NonTerminatorType.AddOperator), "38b1cd92-ad64-41f6-9986-f00b145da852"}, { new Terminator(OperatorType.Plus), "b2a223d6-6905-4b72-a67b-96d8604fe688"}, { new Terminator(OperatorType.Minus), "569b7d42-5c9a-4ba6-b124-8dacd7f387f4"}, { new Terminator(KeywordType.Or), "37aaac08-c0a9-420d-bfd1-1ad8615a7513"},}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Expression))}, { new Terminator(KeywordType.Else), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Expression))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Expression))}, }, "ff5e3d10-5b5d-425f-ab1f-9fdd2a3cc3fd") },
		{ "b73935ec-c2f7-405d-b963-c028060e4aef", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.MultiplyOperator), "aa2b1cf1-871a-4c42-9c5c-99e15f272913"}, { new Terminator(OperatorType.Multiply), "e576c109-5246-4139-8248-415b8e3e68de"}, { new Terminator(OperatorType.Divide), "6a567f32-04a0-49b8-bf8e-42f55ee0600f"}, { new Terminator(KeywordType.Divide), "86be9c46-a085-4183-9264-b12d73fde46e"}, { new Terminator(KeywordType.Mod), "d7708b97-7da8-492a-a64f-6196ae9527f7"}, { new Terminator(KeywordType.And), "271d9f46-4be0-495c-a90b-ae8f19ee928b"},}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Equal), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Less), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Greater), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Plus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Minus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(KeywordType.Or), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(KeywordType.Else), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, }, "b73935ec-c2f7-405d-b963-c028060e4aef") },
		{ "4aa8e9d3-a1a6-4705-80c6-5c186dc595e8", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Equal), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Less), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Greater), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Mod), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.And), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Plus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Minus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Or), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Else), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, }, "4aa8e9d3-a1a6-4705-80c6-5c186dc595e8") },
		{ "012315f4-1fde-4502-b517-f4971dda6e61", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Equal), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Less), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Greater), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Else), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, }, "012315f4-1fde-4502-b517-f4971dda6e61") },
		{ "54633641-356f-4730-9414-2206c55b142e", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Equal), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Less), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Greater), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Else), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, }, "54633641-356f-4730-9414-2206c55b142e") },
		{ "27dee30b-ae72-4e32-a279-cca6925591e0", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Expression), "32e027bd-473f-4b6a-ac01-0a052520f552"}, { new NonTerminator(NonTerminatorType.SimpleExpression), "e566a9ad-f17c-46ff-981f-ee4fa81d2d3d"}, { new NonTerminator(NonTerminatorType.Term), "1f1e4a05-7d30-44f4-aa5f-5caacfbce4b2"}, { new NonTerminator(NonTerminatorType.Factor), "ff72d68f-ac20-46e8-94ed-090e841128c1"}, { Terminator.NumberTerminator, "79939c29-990e-4743-8a8a-5fccd2ec415d"}, { new NonTerminator(NonTerminatorType.Variable), "28f501cb-0b97-40c3-8a64-d214832a6789"}, { new Terminator(DelimiterType.LeftParenthesis), "f1464d64-129c-4d68-a61e-43296e963ee1"}, { Terminator.IdentifierTerminator, "9dd45908-639c-4e69-97e4-2471cdf5e189"}, { new Terminator(KeywordType.Not), "303776ee-584f-4977-b76a-9449d6a4e1ad"}, { new Terminator(OperatorType.Minus), "2401b03b-4e43-4409-989c-a4471708a795"},}, new Dictionary<Terminator, ReduceInformation>{ }, "27dee30b-ae72-4e32-a279-cca6925591e0") },
		{ "162410d2-e221-4dcc-b587-ecb26e782be7", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.LeftParenthesis), "79605cae-447f-4d0b-bfb8-d6859a50ddb5"}, { new NonTerminator(NonTerminatorType.IdVarPart), "d2de827c-7bb6-460d-ab0b-e8e6fb62790b"}, { new Terminator(DelimiterType.LeftSquareBracket), "1c78eb36-1121-41ad-b141-80d582c7874d"},}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Equal), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Less), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Greater), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Divide), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Divide), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Mod), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.And), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Plus), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Minus), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Or), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Else), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, }, "162410d2-e221-4dcc-b587-ecb26e782be7") },
		{ "75cbf022-d2eb-4962-8b19-cbc270acb474", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Factor), "6fdc7e31-69dc-4672-9330-620107141f23"}, { Terminator.NumberTerminator, "012315f4-1fde-4502-b517-f4971dda6e61"}, { new NonTerminator(NonTerminatorType.Variable), "54633641-356f-4730-9414-2206c55b142e"}, { new Terminator(DelimiterType.LeftParenthesis), "27dee30b-ae72-4e32-a279-cca6925591e0"}, { Terminator.IdentifierTerminator, "162410d2-e221-4dcc-b587-ecb26e782be7"}, { new Terminator(KeywordType.Not), "75cbf022-d2eb-4962-8b19-cbc270acb474"}, { new Terminator(OperatorType.Minus), "d8b62d2f-6f12-438e-9866-38ee91689fff"},}, new Dictionary<Terminator, ReduceInformation>{ }, "75cbf022-d2eb-4962-8b19-cbc270acb474") },
		{ "d8b62d2f-6f12-438e-9866-38ee91689fff", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Factor), "f2302ba9-1f2e-4b86-9151-57eadcbc32b8"}, { Terminator.NumberTerminator, "012315f4-1fde-4502-b517-f4971dda6e61"}, { new NonTerminator(NonTerminatorType.Variable), "54633641-356f-4730-9414-2206c55b142e"}, { new Terminator(DelimiterType.LeftParenthesis), "27dee30b-ae72-4e32-a279-cca6925591e0"}, { Terminator.IdentifierTerminator, "162410d2-e221-4dcc-b587-ecb26e782be7"}, { new Terminator(KeywordType.Not), "75cbf022-d2eb-4962-8b19-cbc270acb474"}, { new Terminator(OperatorType.Minus), "d8b62d2f-6f12-438e-9866-38ee91689fff"},}, new Dictionary<Terminator, ReduceInformation>{ }, "d8b62d2f-6f12-438e-9866-38ee91689fff") },
		{ "96fed0e4-0e78-4e98-87bc-e5353e32b6a9", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Statement))}, { new Terminator(KeywordType.Else), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Statement))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Statement))}, }, "96fed0e4-0e78-4e98-87bc-e5353e32b6a9") },
		{ "ec56d7f1-93f2-4594-85e3-ecac0c328912", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.RightParenthesis), "07c1847f-9fe9-44d0-9631-2e28c3dad1da"}, { new Terminator(DelimiterType.Comma), "077bd83e-2dfc-4686-bf99-ec6992c05b60"},}, new Dictionary<Terminator, ReduceInformation>{ }, "ec56d7f1-93f2-4594-85e3-ecac0c328912") },
		{ "3dfb3788-fd4a-4da1-bc2c-54ed1108602e", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Statement), "714a7905-6456-45fe-b2b4-a1d1ea20b67b"}, { new NonTerminator(NonTerminatorType.Variable), "f7849288-2cd0-4c68-aeee-abb4f910fa32"}, { Terminator.IdentifierTerminator, "403840ce-3751-4324-a4da-a2b0a7653957"}, { new NonTerminator(NonTerminatorType.ProcedureCall), "504c43d8-8335-43de-8838-5c3205046f5d"}, { new NonTerminator(NonTerminatorType.CompoundStatement), "15b27fe6-dc50-4e57-8246-bc2e8a5eb05a"}, { new Terminator(KeywordType.If), "1e290bf0-8ecb-4a2d-b5dc-bd2aa7ff6505"}, { new Terminator(KeywordType.For), "83bfcb23-a73e-4ba5-ac65-68e5a019cc6a"}, { new Terminator(KeywordType.Begin), "1fd17dc7-14a7-4500-8bc1-e768a6969eae"},}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(0, new NonTerminator(NonTerminatorType.Statement))}, { new Terminator(KeywordType.Else), new ReduceInformation(0, new NonTerminator(NonTerminatorType.Statement))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(0, new NonTerminator(NonTerminatorType.Statement))}, }, "3dfb3788-fd4a-4da1-bc2c-54ed1108602e") },
		{ "852b35d2-4d23-4fdc-852b-10e8cbbc4a52", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Expression), "85296e3b-2548-4584-9bb2-54ee8e935658"}, { new NonTerminator(NonTerminatorType.SimpleExpression), "d04891ca-bc62-431f-b1aa-67a00abdd72a"}, { new NonTerminator(NonTerminatorType.Term), "2e23b417-e8d1-4bf4-b5fb-791730c5bed2"}, { new NonTerminator(NonTerminatorType.Factor), "e2eecffb-85c1-440e-9c45-1fdd778c942a"}, { Terminator.NumberTerminator, "656892f9-1aaa-4635-84ff-c12d4e1a0c2e"}, { new NonTerminator(NonTerminatorType.Variable), "fd6b2f33-2a46-4dc5-93fc-0b7816be2582"}, { new Terminator(DelimiterType.LeftParenthesis), "079e396f-7464-432e-8666-7fe4b0ca40a7"}, { Terminator.IdentifierTerminator, "902c5106-8d85-47b7-b482-58ed10129c15"}, { new Terminator(KeywordType.Not), "b715b0a1-976c-42f9-84d7-4e3968a3369f"}, { new Terminator(OperatorType.Minus), "b8cd8bf3-bd87-4006-b6a9-4f0305e07f51"},}, new Dictionary<Terminator, ReduceInformation>{ }, "852b35d2-4d23-4fdc-852b-10e8cbbc4a52") },
		{ "db50ea31-01ea-498f-bf72-f0744d2f3cd2", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(3, new NonTerminator(NonTerminatorType.CompoundStatement))}, { new Terminator(KeywordType.Else), new ReduceInformation(3, new NonTerminator(NonTerminatorType.CompoundStatement))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(3, new NonTerminator(NonTerminatorType.CompoundStatement))}, }, "db50ea31-01ea-498f-bf72-f0744d2f3cd2") },
		{ "faddedbb-3247-4458-98ea-694e7b6ed843", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.MultiplyOperator), "2a0c4c84-0d34-48b7-ac1e-6c4263554c2c"}, { new Terminator(OperatorType.Multiply), "e576c109-5246-4139-8248-415b8e3e68de"}, { new Terminator(OperatorType.Divide), "6a567f32-04a0-49b8-bf8e-42f55ee0600f"}, { new Terminator(KeywordType.Divide), "86be9c46-a085-4183-9264-b12d73fde46e"}, { new Terminator(KeywordType.Mod), "d7708b97-7da8-492a-a64f-6196ae9527f7"}, { new Terminator(KeywordType.And), "271d9f46-4be0-495c-a90b-ae8f19ee928b"},}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.Then), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Plus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Minus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(KeywordType.Or), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, }, "faddedbb-3247-4458-98ea-694e7b6ed843") },
		{ "42a84635-03d1-42a5-9c25-cf101cde9d2b", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.Then), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Mod), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.And), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Plus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Minus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Or), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, }, "42a84635-03d1-42a5-9c25-cf101cde9d2b") },
		{ "c2e31402-c226-4545-9be0-759dfcd81339", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.Then), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, }, "c2e31402-c226-4545-9be0-759dfcd81339") },
		{ "09352cee-2c04-4475-b2f9-4f57fd76d991", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.RightParenthesis), "869a80ce-96ec-4017-8e60-a85d2632062d"}, { new Terminator(DelimiterType.Comma), "077bd83e-2dfc-4686-bf99-ec6992c05b60"},}, new Dictionary<Terminator, ReduceInformation>{ }, "09352cee-2c04-4475-b2f9-4f57fd76d991") },
		{ "f101e13b-d089-42c8-9bef-cd57e53a54a0", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.RightSquareBracket), "327c1e7c-d45a-40cd-b180-e3cc1677ad0f"}, { new Terminator(DelimiterType.Comma), "082dc96a-aa2d-444e-b2b0-8a6bca7f6ba2"},}, new Dictionary<Terminator, ReduceInformation>{ }, "f101e13b-d089-42c8-9bef-cd57e53a54a0") },
		{ "0fe43d0d-66a7-4555-a250-6b3300e008ab", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Term), "df18369a-ab42-4e49-890f-31601c600004"}, { new NonTerminator(NonTerminatorType.Factor), "e469f27a-a5b7-4a0a-aa29-0092ba56b087"}, { Terminator.NumberTerminator, "b0508395-27cc-4179-8a0f-1a5ac38d89b7"}, { new NonTerminator(NonTerminatorType.Variable), "6d5d615b-e7d3-422e-a5ba-123e617a33e0"}, { new Terminator(DelimiterType.LeftParenthesis), "617493e7-522c-4e09-b525-6b29f2c96528"}, { Terminator.IdentifierTerminator, "a8b3e8cf-a167-4e52-a876-56fc36b99e81"}, { new Terminator(KeywordType.Not), "65e2d29a-99a0-4377-95af-9b1954c5e202"}, { new Terminator(OperatorType.Minus), "dba55308-efb5-4ad7-8bee-f379f817f752"},}, new Dictionary<Terminator, ReduceInformation>{ }, "0fe43d0d-66a7-4555-a250-6b3300e008ab") },
		{ "a48ba1d6-7955-4c69-ac2a-e5cf8d58e33a", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Factor), "4b953aad-9ea9-449d-9f59-8d2a69f13f7c"}, { Terminator.NumberTerminator, "b0508395-27cc-4179-8a0f-1a5ac38d89b7"}, { new NonTerminator(NonTerminatorType.Variable), "6d5d615b-e7d3-422e-a5ba-123e617a33e0"}, { new Terminator(DelimiterType.LeftParenthesis), "617493e7-522c-4e09-b525-6b29f2c96528"}, { Terminator.IdentifierTerminator, "a8b3e8cf-a167-4e52-a876-56fc36b99e81"}, { new Terminator(KeywordType.Not), "65e2d29a-99a0-4377-95af-9b1954c5e202"}, { new Terminator(OperatorType.Minus), "dba55308-efb5-4ad7-8bee-f379f817f752"},}, new Dictionary<Terminator, ReduceInformation>{ }, "a48ba1d6-7955-4c69-ac2a-e5cf8d58e33a") },
		{ "a7246b45-ae4e-485e-87c0-43c1e81d53c1", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.RightParenthesis), "2dd63deb-7a8c-422a-b553-3de8553021e6"},}, new Dictionary<Terminator, ReduceInformation>{ }, "a7246b45-ae4e-485e-87c0-43c1e81d53c1") },
		{ "dd2f25ec-2fc5-49f5-9ad9-42cdbfadcdcf", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.ExpressionList), "c64ec36a-4379-4374-8b1b-8f6f91bc70a0"}, { new NonTerminator(NonTerminatorType.Expression), "4f59631a-5553-49da-9f0d-dda48b0d65b2"}, { new NonTerminator(NonTerminatorType.SimpleExpression), "e2a2e2c4-61e9-496f-8a60-ac108404247f"}, { new NonTerminator(NonTerminatorType.Term), "83d0456c-4817-49a5-bd20-c0d8a1c5b696"}, { new NonTerminator(NonTerminatorType.Factor), "f1db3e46-b446-4adc-93ae-5dfed0964c99"}, { Terminator.NumberTerminator, "0abfb7c0-083d-4242-a9a7-06183f7230f7"}, { new NonTerminator(NonTerminatorType.Variable), "a8c27dec-213a-4917-8b05-b0422260d1c1"}, { new Terminator(DelimiterType.LeftParenthesis), "45c4069f-f3cf-48f7-af29-8eb8303c47d6"}, { Terminator.IdentifierTerminator, "5aabfbf6-09f5-484f-9f95-ac0fa52c26c4"}, { new Terminator(KeywordType.Not), "253ff382-06bc-47a7-9645-cc04c48e1d0a"}, { new Terminator(OperatorType.Minus), "5708e399-73e5-4bcb-9aeb-f25d1e06c35c"},}, new Dictionary<Terminator, ReduceInformation>{ }, "dd2f25ec-2fc5-49f5-9ad9-42cdbfadcdcf") },
		{ "6f6e0fef-9b3f-42d1-a8eb-dbd918b57142", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightParenthesis), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(KeywordType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(KeywordType.Mod), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(KeywordType.And), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.Plus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.Minus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(KeywordType.Or), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, }, "6f6e0fef-9b3f-42d1-a8eb-dbd918b57142") },
		{ "86d71483-4b09-4a09-b5e8-e97a0ae2fbe9", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.ExpressionList), "f0d57b7c-dd48-43b6-b6a8-173cd7c71368"}, { new NonTerminator(NonTerminatorType.Expression), "a0ba5247-cbd6-46f6-b1fd-574979eb2b27"}, { new NonTerminator(NonTerminatorType.SimpleExpression), "e8b0426e-8437-45f8-a02f-3eebdfb48e84"}, { new NonTerminator(NonTerminatorType.Term), "cd4bfe7a-5a2e-41f1-a891-6359311b9403"}, { new NonTerminator(NonTerminatorType.Factor), "ceb0b15a-7ff9-4265-8766-716485412510"}, { Terminator.NumberTerminator, "06d206a2-8c81-4399-8790-c903a8acfe21"}, { new NonTerminator(NonTerminatorType.Variable), "422866b2-29d6-4004-b762-cb9ad9ddc5e7"}, { new Terminator(DelimiterType.LeftParenthesis), "4f42a0a3-3c4b-46a5-8dcc-b91007f93d5c"}, { Terminator.IdentifierTerminator, "7339bcec-b0ae-4611-83cd-08345e44324f"}, { new Terminator(KeywordType.Not), "bfdaacfe-9460-4db8-9469-712b6674efb5"}, { new Terminator(OperatorType.Minus), "eac356e4-6098-43be-9b50-3ff1a637a898"},}, new Dictionary<Terminator, ReduceInformation>{ }, "86d71483-4b09-4a09-b5e8-e97a0ae2fbe9") },
		{ "39060e84-cdeb-42ee-a24e-8a018b2b0944", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightParenthesis), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, }, "39060e84-cdeb-42ee-a24e-8a018b2b0944") },
		{ "95beb628-7860-4f11-9712-15c738b9c1cd", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightParenthesis), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, }, "95beb628-7860-4f11-9712-15c738b9c1cd") },
		{ "08a8ab6c-dd7e-46cf-9145-20af1cb47384", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightParenthesis), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Equal), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Less), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Greater), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, }, "08a8ab6c-dd7e-46cf-9145-20af1cb47384") },
		{ "4e4b9686-7296-431d-b4f9-d9feb8b73e03", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightParenthesis), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Equal), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Less), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Greater), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Mod), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.And), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Plus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Minus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Or), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, }, "4e4b9686-7296-431d-b4f9-d9feb8b73e03") },
		{ "c937afc1-3473-48bd-b95f-77b995ca08c8", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(KeywordType.Do), "0b6aad40-60a6-4144-998a-be547ee82f1f"},}, new Dictionary<Terminator, ReduceInformation>{ }, "c937afc1-3473-48bd-b95f-77b995ca08c8") },
		{ "edabf765-08f8-4d58-8722-863c2b38bf71", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.RelationOperator), "59ce931f-0bea-4ab7-8afc-68f32f4e4aea"}, { new Terminator(OperatorType.Equal), "1b49be29-a9d8-403a-b434-5c116300d15c"}, { new Terminator(OperatorType.NotEqual), "fb7de677-df97-4431-b5b7-086a8ded26ff"}, { new Terminator(OperatorType.Less), "8db4bd06-8fe0-4476-924a-3354772a5ac3"}, { new Terminator(OperatorType.LessEqual), "68ce8cad-2c90-4f71-822f-885e193cc394"}, { new Terminator(OperatorType.Greater), "f863fa1a-f90f-430f-8ee8-4cb33053ee45"}, { new Terminator(OperatorType.GreaterEqual), "d79c0f7f-9997-421d-bed4-1e0b2517774c"}, { new NonTerminator(NonTerminatorType.AddOperator), "7412aad9-254b-4fb3-9513-1d52100422dc"}, { new Terminator(OperatorType.Plus), "b2a223d6-6905-4b72-a67b-96d8604fe688"}, { new Terminator(OperatorType.Minus), "569b7d42-5c9a-4ba6-b124-8dacd7f387f4"}, { new Terminator(KeywordType.Or), "37aaac08-c0a9-420d-bfd1-1ad8615a7513"},}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.Do), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Expression))}, }, "edabf765-08f8-4d58-8722-863c2b38bf71") },
		{ "96b22d74-a3ce-42d1-980f-ab9ca7edff4d", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.MultiplyOperator), "2ad01207-64fe-4500-91d8-a8ebf7d2de07"}, { new Terminator(OperatorType.Multiply), "e576c109-5246-4139-8248-415b8e3e68de"}, { new Terminator(OperatorType.Divide), "6a567f32-04a0-49b8-bf8e-42f55ee0600f"}, { new Terminator(KeywordType.Divide), "86be9c46-a085-4183-9264-b12d73fde46e"}, { new Terminator(KeywordType.Mod), "d7708b97-7da8-492a-a64f-6196ae9527f7"}, { new Terminator(KeywordType.And), "271d9f46-4be0-495c-a90b-ae8f19ee928b"},}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.Do), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Equal), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Less), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Greater), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Plus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Minus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(KeywordType.Or), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, }, "96b22d74-a3ce-42d1-980f-ab9ca7edff4d") },
		{ "a7870627-59bc-434d-bf03-0c1d5b649d45", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.Do), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Equal), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Less), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Greater), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Mod), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.And), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Plus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Minus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Or), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, }, "a7870627-59bc-434d-bf03-0c1d5b649d45") },
		{ "906b0111-f667-467d-a74a-c2be40fb7905", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.Do), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Equal), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Less), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Greater), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, }, "906b0111-f667-467d-a74a-c2be40fb7905") },
		{ "ecc69635-01d5-4cdf-aaa8-750d1fa736f1", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.Do), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Equal), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Less), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Greater), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, }, "ecc69635-01d5-4cdf-aaa8-750d1fa736f1") },
		{ "e41a1788-a93a-4fb3-846e-a2d33bd77958", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Expression), "fbc2facf-c586-48bd-9185-7308bedfd222"}, { new NonTerminator(NonTerminatorType.SimpleExpression), "e566a9ad-f17c-46ff-981f-ee4fa81d2d3d"}, { new NonTerminator(NonTerminatorType.Term), "1f1e4a05-7d30-44f4-aa5f-5caacfbce4b2"}, { new NonTerminator(NonTerminatorType.Factor), "ff72d68f-ac20-46e8-94ed-090e841128c1"}, { Terminator.NumberTerminator, "79939c29-990e-4743-8a8a-5fccd2ec415d"}, { new NonTerminator(NonTerminatorType.Variable), "28f501cb-0b97-40c3-8a64-d214832a6789"}, { new Terminator(DelimiterType.LeftParenthesis), "f1464d64-129c-4d68-a61e-43296e963ee1"}, { Terminator.IdentifierTerminator, "9dd45908-639c-4e69-97e4-2471cdf5e189"}, { new Terminator(KeywordType.Not), "303776ee-584f-4977-b76a-9449d6a4e1ad"}, { new Terminator(OperatorType.Minus), "2401b03b-4e43-4409-989c-a4471708a795"},}, new Dictionary<Terminator, ReduceInformation>{ }, "e41a1788-a93a-4fb3-846e-a2d33bd77958") },
		{ "71f58c48-8093-40c7-b29d-0e1099642b47", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.LeftParenthesis), "9573e208-f4ec-443d-b0c1-db78f4cc013e"}, { new NonTerminator(NonTerminatorType.IdVarPart), "7f79fb52-93d9-4d81-9d37-9d509369ccff"}, { new Terminator(DelimiterType.LeftSquareBracket), "df68949f-fa42-4574-93f0-6740cb47ce6f"},}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.Do), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Equal), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Less), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Greater), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Divide), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Divide), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Mod), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.And), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Plus), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Minus), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Or), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, }, "71f58c48-8093-40c7-b29d-0e1099642b47") },
		{ "c5c60bb4-0f4e-4e7b-a222-1e9f7c12472d", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Factor), "8959fb99-fe59-41c1-815c-8dd79b37b18a"}, { Terminator.NumberTerminator, "906b0111-f667-467d-a74a-c2be40fb7905"}, { new NonTerminator(NonTerminatorType.Variable), "ecc69635-01d5-4cdf-aaa8-750d1fa736f1"}, { new Terminator(DelimiterType.LeftParenthesis), "e41a1788-a93a-4fb3-846e-a2d33bd77958"}, { Terminator.IdentifierTerminator, "71f58c48-8093-40c7-b29d-0e1099642b47"}, { new Terminator(KeywordType.Not), "c5c60bb4-0f4e-4e7b-a222-1e9f7c12472d"}, { new Terminator(OperatorType.Minus), "cb5f054d-2d9d-417c-8843-2156b2d466a7"},}, new Dictionary<Terminator, ReduceInformation>{ }, "c5c60bb4-0f4e-4e7b-a222-1e9f7c12472d") },
		{ "cb5f054d-2d9d-417c-8843-2156b2d466a7", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Factor), "7232def0-9978-4352-a557-518acfcd2f5d"}, { Terminator.NumberTerminator, "906b0111-f667-467d-a74a-c2be40fb7905"}, { new NonTerminator(NonTerminatorType.Variable), "ecc69635-01d5-4cdf-aaa8-750d1fa736f1"}, { new Terminator(DelimiterType.LeftParenthesis), "e41a1788-a93a-4fb3-846e-a2d33bd77958"}, { Terminator.IdentifierTerminator, "71f58c48-8093-40c7-b29d-0e1099642b47"}, { new Terminator(KeywordType.Not), "c5c60bb4-0f4e-4e7b-a222-1e9f7c12472d"}, { new Terminator(OperatorType.Minus), "cb5f054d-2d9d-417c-8843-2156b2d466a7"},}, new Dictionary<Terminator, ReduceInformation>{ }, "cb5f054d-2d9d-417c-8843-2156b2d466a7") },
		{ "5f59a76e-a0df-4ac7-a095-ae456f4a1564", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.AddOperator), "7bcc2d3e-3acb-48f8-92f8-11999bac7c09"}, { new Terminator(OperatorType.Plus), "b2a223d6-6905-4b72-a67b-96d8604fe688"}, { new Terminator(OperatorType.Minus), "569b7d42-5c9a-4ba6-b124-8dacd7f387f4"}, { new Terminator(KeywordType.Or), "37aaac08-c0a9-420d-bfd1-1ad8615a7513"},}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.To), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Expression))}, }, "5f59a76e-a0df-4ac7-a095-ae456f4a1564") },
		{ "475339af-ff8f-4fac-b64c-01fe36f26db0", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.MultiplyOperator), "a9d56e77-936e-4ced-953d-d412cea663ff"}, { new Terminator(OperatorType.Multiply), "e576c109-5246-4139-8248-415b8e3e68de"}, { new Terminator(OperatorType.Divide), "6a567f32-04a0-49b8-bf8e-42f55ee0600f"}, { new Terminator(KeywordType.Divide), "86be9c46-a085-4183-9264-b12d73fde46e"}, { new Terminator(KeywordType.Mod), "d7708b97-7da8-492a-a64f-6196ae9527f7"}, { new Terminator(KeywordType.And), "271d9f46-4be0-495c-a90b-ae8f19ee928b"},}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.To), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Plus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Minus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(KeywordType.Or), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, }, "475339af-ff8f-4fac-b64c-01fe36f26db0") },
		{ "aaa1c6ff-81ea-415b-a7f2-0ede345af1dd", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.To), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Mod), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.And), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Plus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Minus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Or), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, }, "aaa1c6ff-81ea-415b-a7f2-0ede345af1dd") },
		{ "f62db516-a4fe-45b8-8c43-7013e181d8d1", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.To), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, }, "f62db516-a4fe-45b8-8c43-7013e181d8d1") },
		{ "e1c60364-c243-481b-b5ad-fac016bd7b32", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.To), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, }, "e1c60364-c243-481b-b5ad-fac016bd7b32") },
		{ "d2ba8656-b05e-4fea-9aeb-3e7e5aef0cd3", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Expression), "32f99453-2d57-424a-9ffe-cbf15f943b16"}, { new NonTerminator(NonTerminatorType.SimpleExpression), "e566a9ad-f17c-46ff-981f-ee4fa81d2d3d"}, { new NonTerminator(NonTerminatorType.Term), "1f1e4a05-7d30-44f4-aa5f-5caacfbce4b2"}, { new NonTerminator(NonTerminatorType.Factor), "ff72d68f-ac20-46e8-94ed-090e841128c1"}, { Terminator.NumberTerminator, "79939c29-990e-4743-8a8a-5fccd2ec415d"}, { new NonTerminator(NonTerminatorType.Variable), "28f501cb-0b97-40c3-8a64-d214832a6789"}, { new Terminator(DelimiterType.LeftParenthesis), "f1464d64-129c-4d68-a61e-43296e963ee1"}, { Terminator.IdentifierTerminator, "9dd45908-639c-4e69-97e4-2471cdf5e189"}, { new Terminator(KeywordType.Not), "303776ee-584f-4977-b76a-9449d6a4e1ad"}, { new Terminator(OperatorType.Minus), "2401b03b-4e43-4409-989c-a4471708a795"},}, new Dictionary<Terminator, ReduceInformation>{ }, "d2ba8656-b05e-4fea-9aeb-3e7e5aef0cd3") },
		{ "e81de486-3409-4f08-98da-f145bd2fb370", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.LeftParenthesis), "8dfd88d6-8c53-4281-9d85-ed375411849f"}, { new NonTerminator(NonTerminatorType.IdVarPart), "44d05a75-4b8d-4ea5-baf5-a5af902d5511"}, { new Terminator(DelimiterType.LeftSquareBracket), "4b2d5ff9-301d-49ab-add3-750972085a18"},}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.To), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Divide), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Divide), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Mod), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.And), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Plus), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Minus), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Or), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, }, "e81de486-3409-4f08-98da-f145bd2fb370") },
		{ "0b574dc0-93f9-4835-b426-8a0d43f17059", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Factor), "6ae08d08-0fbe-451d-b3cf-f18e0b3a95ab"}, { Terminator.NumberTerminator, "f62db516-a4fe-45b8-8c43-7013e181d8d1"}, { new NonTerminator(NonTerminatorType.Variable), "e1c60364-c243-481b-b5ad-fac016bd7b32"}, { new Terminator(DelimiterType.LeftParenthesis), "d2ba8656-b05e-4fea-9aeb-3e7e5aef0cd3"}, { Terminator.IdentifierTerminator, "e81de486-3409-4f08-98da-f145bd2fb370"}, { new Terminator(KeywordType.Not), "0b574dc0-93f9-4835-b426-8a0d43f17059"}, { new Terminator(OperatorType.Minus), "b1e61a3b-133a-4ff2-bc0c-e4580ad4a90a"},}, new Dictionary<Terminator, ReduceInformation>{ }, "0b574dc0-93f9-4835-b426-8a0d43f17059") },
		{ "b1e61a3b-133a-4ff2-bc0c-e4580ad4a90a", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Factor), "060c208b-8cc7-4c14-98b0-8af53e8abb4d"}, { Terminator.NumberTerminator, "f62db516-a4fe-45b8-8c43-7013e181d8d1"}, { new NonTerminator(NonTerminatorType.Variable), "e1c60364-c243-481b-b5ad-fac016bd7b32"}, { new Terminator(DelimiterType.LeftParenthesis), "d2ba8656-b05e-4fea-9aeb-3e7e5aef0cd3"}, { Terminator.IdentifierTerminator, "e81de486-3409-4f08-98da-f145bd2fb370"}, { new Terminator(KeywordType.Not), "0b574dc0-93f9-4835-b426-8a0d43f17059"}, { new Terminator(OperatorType.Minus), "b1e61a3b-133a-4ff2-bc0c-e4580ad4a90a"},}, new Dictionary<Terminator, ReduceInformation>{ }, "b1e61a3b-133a-4ff2-bc0c-e4580ad4a90a") },
		{ "b69dc680-816c-4e2d-9c51-145d5c2a980b", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.MultiplyOperator), "a2d36e1c-1210-4175-a84b-9df1fe589223"}, { new Terminator(OperatorType.Multiply), "e576c109-5246-4139-8248-415b8e3e68de"}, { new Terminator(OperatorType.Divide), "6a567f32-04a0-49b8-bf8e-42f55ee0600f"}, { new Terminator(KeywordType.Divide), "86be9c46-a085-4183-9264-b12d73fde46e"}, { new Terminator(KeywordType.Mod), "d7708b97-7da8-492a-a64f-6196ae9527f7"}, { new Terminator(KeywordType.And), "271d9f46-4be0-495c-a90b-ae8f19ee928b"},}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.To), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Equal), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Less), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Greater), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Plus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Minus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(KeywordType.Or), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, }, "b69dc680-816c-4e2d-9c51-145d5c2a980b") },
		{ "0aedeb64-c294-4d32-8738-24b107274f8d", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.To), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Equal), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Less), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Greater), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Mod), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.And), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Plus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Minus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Or), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, }, "0aedeb64-c294-4d32-8738-24b107274f8d") },
		{ "eee81e2f-ca56-4fc9-a31b-7e8616370f67", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.To), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Equal), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Less), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Greater), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, }, "eee81e2f-ca56-4fc9-a31b-7e8616370f67") },
		{ "051cd12a-adfd-4234-89d2-238b590e7687", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.RightParenthesis), "0e4d35ac-6900-4ca0-ba4c-b632b5d7e575"}, { new Terminator(DelimiterType.Comma), "077bd83e-2dfc-4686-bf99-ec6992c05b60"},}, new Dictionary<Terminator, ReduceInformation>{ }, "051cd12a-adfd-4234-89d2-238b590e7687") },
		{ "6baa2e2e-995a-4434-b8c2-e80b1bef7a42", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.RightSquareBracket), "dfb5f63b-5ee9-471a-ba9b-a0947a66ed49"}, { new Terminator(DelimiterType.Comma), "082dc96a-aa2d-444e-b2b0-8a6bca7f6ba2"},}, new Dictionary<Terminator, ReduceInformation>{ }, "6baa2e2e-995a-4434-b8c2-e80b1bef7a42") },
		{ "1f4f1827-a7f7-41ff-968f-07d84486ed66", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.Semicolon), new ReduceInformation(3, new NonTerminator(NonTerminatorType.CompoundStatement))}, }, "1f4f1827-a7f7-41ff-968f-07d84486ed66") },
		{ "ad6e6545-4780-4630-9eb3-e455f9525849", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.Semicolon), new ReduceInformation(6, new NonTerminator(NonTerminatorType.Type))}, }, "ad6e6545-4780-4630-9eb3-e455f9525849") },
		{ "e9c44581-4124-4d63-9fb0-7b4cafb8d601", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { Terminator.NumberTerminator, "94175ffd-57ec-4010-b747-b24e868cc768"},}, new Dictionary<Terminator, ReduceInformation>{ }, "e9c44581-4124-4d63-9fb0-7b4cafb8d601") },
		{ "c0fddf91-3585-48d3-9b2f-fa5838ce5bbe", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.MultiplyOperator), "2f27ae56-216f-4e19-b722-ed4dd82cebbb"}, { new Terminator(OperatorType.Multiply), "e576c109-5246-4139-8248-415b8e3e68de"}, { new Terminator(OperatorType.Divide), "6a567f32-04a0-49b8-bf8e-42f55ee0600f"}, { new Terminator(KeywordType.Divide), "86be9c46-a085-4183-9264-b12d73fde46e"}, { new Terminator(KeywordType.Mod), "d7708b97-7da8-492a-a64f-6196ae9527f7"}, { new Terminator(KeywordType.And), "271d9f46-4be0-495c-a90b-ae8f19ee928b"},}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Plus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Minus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(KeywordType.Or), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, }, "c0fddf91-3585-48d3-9b2f-fa5838ce5bbe") },
		{ "64b11017-23c6-4007-ae3d-43b98aa96cbf", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Mod), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.And), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Plus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Minus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Or), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, }, "64b11017-23c6-4007-ae3d-43b98aa96cbf") },
		{ "b6dc85b0-25f6-4140-b85f-8fcc92436e6e", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, }, "b6dc85b0-25f6-4140-b85f-8fcc92436e6e") },
		{ "083d8249-ed1c-42c0-98c8-92237661ef9a", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.RightParenthesis), "670a2adf-00b7-4f63-94cf-912477ed0bca"}, { new Terminator(DelimiterType.Comma), "077bd83e-2dfc-4686-bf99-ec6992c05b60"},}, new Dictionary<Terminator, ReduceInformation>{ }, "083d8249-ed1c-42c0-98c8-92237661ef9a") },
		{ "1ec358e9-0a98-4c60-a070-c1529d2559bf", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.RightSquareBracket), "352d3eaf-86b5-408d-8b83-25c9b832a549"}, { new Terminator(DelimiterType.Comma), "082dc96a-aa2d-444e-b2b0-8a6bca7f6ba2"},}, new Dictionary<Terminator, ReduceInformation>{ }, "1ec358e9-0a98-4c60-a070-c1529d2559bf") },
		{ "489a9a58-71fe-42a8-a046-0e59b767b261", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.MultiplyOperator), "fda8ff9f-0d9c-4a54-9207-d10bf57748e1"}, { new Terminator(OperatorType.Multiply), "e576c109-5246-4139-8248-415b8e3e68de"}, { new Terminator(OperatorType.Divide), "6a567f32-04a0-49b8-bf8e-42f55ee0600f"}, { new Terminator(KeywordType.Divide), "86be9c46-a085-4183-9264-b12d73fde46e"}, { new Terminator(KeywordType.Mod), "d7708b97-7da8-492a-a64f-6196ae9527f7"}, { new Terminator(KeywordType.And), "271d9f46-4be0-495c-a90b-ae8f19ee928b"},}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightSquareBracket), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Plus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Minus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(KeywordType.Or), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(DelimiterType.Comma), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, }, "489a9a58-71fe-42a8-a046-0e59b767b261") },
		{ "49e98f98-fe8e-414f-b78e-507b72727731", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightSquareBracket), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Mod), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.And), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Plus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Minus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Or), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(DelimiterType.Comma), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, }, "49e98f98-fe8e-414f-b78e-507b72727731") },
		{ "ffd07de0-d88a-4227-bd15-c48495e37164", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightSquareBracket), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(DelimiterType.Comma), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, }, "ffd07de0-d88a-4227-bd15-c48495e37164") },
		{ "dc755815-5742-4ca8-a2c2-bc83ee7bef33", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.RightParenthesis), "eb5b24f6-cf71-4d86-bef6-594aadcea8ee"}, { new Terminator(DelimiterType.Comma), "077bd83e-2dfc-4686-bf99-ec6992c05b60"},}, new Dictionary<Terminator, ReduceInformation>{ }, "dc755815-5742-4ca8-a2c2-bc83ee7bef33") },
		{ "2af7807b-4b78-4fe8-889c-1623f0065ab0", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.RightSquareBracket), "47777f79-3c3b-4e78-9cc2-112c7017090e"}, { new Terminator(DelimiterType.Comma), "082dc96a-aa2d-444e-b2b0-8a6bca7f6ba2"},}, new Dictionary<Terminator, ReduceInformation>{ }, "2af7807b-4b78-4fe8-889c-1623f0065ab0") },
		{ "8da10347-87c7-4248-85fc-5c2724d54627", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.MultiplyOperator), "a37c0513-7bbf-49ad-8a65-73ce9ee54a69"}, { new Terminator(OperatorType.Multiply), "e576c109-5246-4139-8248-415b8e3e68de"}, { new Terminator(OperatorType.Divide), "6a567f32-04a0-49b8-bf8e-42f55ee0600f"}, { new Terminator(KeywordType.Divide), "86be9c46-a085-4183-9264-b12d73fde46e"}, { new Terminator(KeywordType.Mod), "d7708b97-7da8-492a-a64f-6196ae9527f7"}, { new Terminator(KeywordType.And), "271d9f46-4be0-495c-a90b-ae8f19ee928b"},}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightParenthesis), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Plus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Minus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(KeywordType.Or), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(DelimiterType.Comma), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, }, "8da10347-87c7-4248-85fc-5c2724d54627") },
		{ "92e5c3df-1503-47ec-9f00-e45f84ef5001", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightParenthesis), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Mod), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.And), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Plus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Minus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Or), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(DelimiterType.Comma), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, }, "92e5c3df-1503-47ec-9f00-e45f84ef5001") },
		{ "519d03d5-16d4-404d-bf9b-96b9e663743e", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightParenthesis), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(DelimiterType.Comma), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, }, "519d03d5-16d4-404d-bf9b-96b9e663743e") },
		{ "aa1370ba-efea-43e6-ae33-0a00afafc098", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.RightParenthesis), "fe9b513c-1e36-4556-9089-d476f10b98aa"}, { new Terminator(DelimiterType.Comma), "077bd83e-2dfc-4686-bf99-ec6992c05b60"},}, new Dictionary<Terminator, ReduceInformation>{ }, "aa1370ba-efea-43e6-ae33-0a00afafc098") },
		{ "81dd75f5-6044-4c57-9b8f-79de660a2a26", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.RightSquareBracket), "95b0dba4-bae5-4a93-9c80-9304734676c6"}, { new Terminator(DelimiterType.Comma), "082dc96a-aa2d-444e-b2b0-8a6bca7f6ba2"},}, new Dictionary<Terminator, ReduceInformation>{ }, "81dd75f5-6044-4c57-9b8f-79de660a2a26") },
		{ "bdc6514f-a58e-4634-8a6f-bcc320b035bf", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.SimpleExpression), "0795b67b-5a21-40c9-b7c7-1081ec0077e0"}, { new NonTerminator(NonTerminatorType.Term), "c645b24f-43f0-42ef-b236-fefc9f8a59e3"}, { new NonTerminator(NonTerminatorType.Factor), "6e59d416-a3a6-4e03-ae3b-117366106542"}, { Terminator.NumberTerminator, "1559bac5-de37-4c11-ba19-090208a71781"}, { new NonTerminator(NonTerminatorType.Variable), "f846fa35-cf45-4ce4-9dba-97755e4630d3"}, { new Terminator(DelimiterType.LeftParenthesis), "587ba18e-861d-49e1-b28a-b4381855be14"}, { Terminator.IdentifierTerminator, "ba9b99d4-067b-47d8-b627-a60322d8c29d"}, { new Terminator(KeywordType.Not), "da0127ed-aab2-42d4-8efa-8cf170904675"}, { new Terminator(OperatorType.Minus), "a66fbe37-bcfb-4f38-a243-3e471986c2b2"},}, new Dictionary<Terminator, ReduceInformation>{ }, "bdc6514f-a58e-4634-8a6f-bcc320b035bf") },
		{ "38b1cd92-ad64-41f6-9986-f00b145da852", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Term), "648315c9-c592-4c1d-8d80-68f89504009e"}, { new NonTerminator(NonTerminatorType.Factor), "4aa8e9d3-a1a6-4705-80c6-5c186dc595e8"}, { Terminator.NumberTerminator, "012315f4-1fde-4502-b517-f4971dda6e61"}, { new NonTerminator(NonTerminatorType.Variable), "54633641-356f-4730-9414-2206c55b142e"}, { new Terminator(DelimiterType.LeftParenthesis), "27dee30b-ae72-4e32-a279-cca6925591e0"}, { Terminator.IdentifierTerminator, "162410d2-e221-4dcc-b587-ecb26e782be7"}, { new Terminator(KeywordType.Not), "75cbf022-d2eb-4962-8b19-cbc270acb474"}, { new Terminator(OperatorType.Minus), "d8b62d2f-6f12-438e-9866-38ee91689fff"},}, new Dictionary<Terminator, ReduceInformation>{ }, "38b1cd92-ad64-41f6-9986-f00b145da852") },
		{ "aa2b1cf1-871a-4c42-9c5c-99e15f272913", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Factor), "7523711f-4646-4a87-945d-1f54799a4a78"}, { Terminator.NumberTerminator, "012315f4-1fde-4502-b517-f4971dda6e61"}, { new NonTerminator(NonTerminatorType.Variable), "54633641-356f-4730-9414-2206c55b142e"}, { new Terminator(DelimiterType.LeftParenthesis), "27dee30b-ae72-4e32-a279-cca6925591e0"}, { Terminator.IdentifierTerminator, "162410d2-e221-4dcc-b587-ecb26e782be7"}, { new Terminator(KeywordType.Not), "75cbf022-d2eb-4962-8b19-cbc270acb474"}, { new Terminator(OperatorType.Minus), "d8b62d2f-6f12-438e-9866-38ee91689fff"},}, new Dictionary<Terminator, ReduceInformation>{ }, "aa2b1cf1-871a-4c42-9c5c-99e15f272913") },
		{ "32e027bd-473f-4b6a-ac01-0a052520f552", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.RightParenthesis), "00227edf-fc74-4b10-a5dc-bf25bbb14f9f"},}, new Dictionary<Terminator, ReduceInformation>{ }, "32e027bd-473f-4b6a-ac01-0a052520f552") },
		{ "79605cae-447f-4d0b-bfb8-d6859a50ddb5", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.ExpressionList), "a1706667-9803-40c2-8b67-b3e932d6aca5"}, { new NonTerminator(NonTerminatorType.Expression), "4f59631a-5553-49da-9f0d-dda48b0d65b2"}, { new NonTerminator(NonTerminatorType.SimpleExpression), "e2a2e2c4-61e9-496f-8a60-ac108404247f"}, { new NonTerminator(NonTerminatorType.Term), "83d0456c-4817-49a5-bd20-c0d8a1c5b696"}, { new NonTerminator(NonTerminatorType.Factor), "f1db3e46-b446-4adc-93ae-5dfed0964c99"}, { Terminator.NumberTerminator, "0abfb7c0-083d-4242-a9a7-06183f7230f7"}, { new NonTerminator(NonTerminatorType.Variable), "a8c27dec-213a-4917-8b05-b0422260d1c1"}, { new Terminator(DelimiterType.LeftParenthesis), "45c4069f-f3cf-48f7-af29-8eb8303c47d6"}, { Terminator.IdentifierTerminator, "5aabfbf6-09f5-484f-9f95-ac0fa52c26c4"}, { new Terminator(KeywordType.Not), "253ff382-06bc-47a7-9645-cc04c48e1d0a"}, { new Terminator(OperatorType.Minus), "5708e399-73e5-4bcb-9aeb-f25d1e06c35c"},}, new Dictionary<Terminator, ReduceInformation>{ }, "79605cae-447f-4d0b-bfb8-d6859a50ddb5") },
		{ "d2de827c-7bb6-460d-ab0b-e8e6fb62790b", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.Equal), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.Less), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.Greater), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(KeywordType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(KeywordType.Mod), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(KeywordType.And), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.Plus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.Minus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(KeywordType.Or), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(KeywordType.Else), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, }, "d2de827c-7bb6-460d-ab0b-e8e6fb62790b") },
		{ "1c78eb36-1121-41ad-b141-80d582c7874d", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.ExpressionList), "7b0a5c49-12ff-4104-84eb-88adfa26f300"}, { new NonTerminator(NonTerminatorType.Expression), "a0ba5247-cbd6-46f6-b1fd-574979eb2b27"}, { new NonTerminator(NonTerminatorType.SimpleExpression), "e8b0426e-8437-45f8-a02f-3eebdfb48e84"}, { new NonTerminator(NonTerminatorType.Term), "cd4bfe7a-5a2e-41f1-a891-6359311b9403"}, { new NonTerminator(NonTerminatorType.Factor), "ceb0b15a-7ff9-4265-8766-716485412510"}, { Terminator.NumberTerminator, "06d206a2-8c81-4399-8790-c903a8acfe21"}, { new NonTerminator(NonTerminatorType.Variable), "422866b2-29d6-4004-b762-cb9ad9ddc5e7"}, { new Terminator(DelimiterType.LeftParenthesis), "4f42a0a3-3c4b-46a5-8dcc-b91007f93d5c"}, { Terminator.IdentifierTerminator, "7339bcec-b0ae-4611-83cd-08345e44324f"}, { new Terminator(KeywordType.Not), "bfdaacfe-9460-4db8-9469-712b6674efb5"}, { new Terminator(OperatorType.Minus), "eac356e4-6098-43be-9b50-3ff1a637a898"},}, new Dictionary<Terminator, ReduceInformation>{ }, "1c78eb36-1121-41ad-b141-80d582c7874d") },
		{ "6fdc7e31-69dc-4672-9330-620107141f23", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Equal), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Less), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Greater), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Else), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, }, "6fdc7e31-69dc-4672-9330-620107141f23") },
		{ "f2302ba9-1f2e-4b86-9151-57eadcbc32b8", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Equal), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Less), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Greater), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Else), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, }, "f2302ba9-1f2e-4b86-9151-57eadcbc32b8") },
		{ "07c1847f-9fe9-44d0-9631-2e28c3dad1da", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(4, new NonTerminator(NonTerminatorType.ProcedureCall))}, { new Terminator(KeywordType.Else), new ReduceInformation(4, new NonTerminator(NonTerminatorType.ProcedureCall))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(4, new NonTerminator(NonTerminatorType.ProcedureCall))}, }, "07c1847f-9fe9-44d0-9631-2e28c3dad1da") },
		{ "714a7905-6456-45fe-b2b4-a1d1ea20b67b", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.ElsePart), "49f5f016-7e61-4fe0-a2d3-570e305a9f3d"}, { new Terminator(KeywordType.Else), "eed27dcb-7c83-4c96-9f7e-6ad65df5e808"},}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(0, new NonTerminator(NonTerminatorType.ElsePart))}, { new Terminator(KeywordType.Else), new ReduceInformation(0, new NonTerminator(NonTerminatorType.ElsePart))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(0, new NonTerminator(NonTerminatorType.ElsePart))}, }, "714a7905-6456-45fe-b2b4-a1d1ea20b67b") },
		{ "85296e3b-2548-4584-9bb2-54ee8e935658", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(KeywordType.To), "96881fa8-667c-4ded-a1ef-ead45665aa0e"},}, new Dictionary<Terminator, ReduceInformation>{ }, "85296e3b-2548-4584-9bb2-54ee8e935658") },
		{ "869a80ce-96ec-4017-8e60-a85d2632062d", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.Then), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, }, "869a80ce-96ec-4017-8e60-a85d2632062d") },
		{ "327c1e7c-d45a-40cd-b180-e3cc1677ad0f", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.Then), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Mod), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.And), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Plus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Minus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Or), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, }, "327c1e7c-d45a-40cd-b180-e3cc1677ad0f") },
		{ "df18369a-ab42-4e49-890f-31601c600004", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.MultiplyOperator), "a48ba1d6-7955-4c69-ac2a-e5cf8d58e33a"}, { new Terminator(OperatorType.Multiply), "e576c109-5246-4139-8248-415b8e3e68de"}, { new Terminator(OperatorType.Divide), "6a567f32-04a0-49b8-bf8e-42f55ee0600f"}, { new Terminator(KeywordType.Divide), "86be9c46-a085-4183-9264-b12d73fde46e"}, { new Terminator(KeywordType.Mod), "d7708b97-7da8-492a-a64f-6196ae9527f7"}, { new Terminator(KeywordType.And), "271d9f46-4be0-495c-a90b-ae8f19ee928b"},}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightParenthesis), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Plus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Minus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(KeywordType.Or), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, }, "df18369a-ab42-4e49-890f-31601c600004") },
		{ "4b953aad-9ea9-449d-9f59-8d2a69f13f7c", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightParenthesis), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Mod), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.And), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Plus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Minus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Or), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, }, "4b953aad-9ea9-449d-9f59-8d2a69f13f7c") },
		{ "2dd63deb-7a8c-422a-b553-3de8553021e6", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightParenthesis), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, }, "2dd63deb-7a8c-422a-b553-3de8553021e6") },
		{ "c64ec36a-4379-4374-8b1b-8f6f91bc70a0", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.RightParenthesis), "9ccb7a92-a877-468c-9336-be8774db708c"}, { new Terminator(DelimiterType.Comma), "077bd83e-2dfc-4686-bf99-ec6992c05b60"},}, new Dictionary<Terminator, ReduceInformation>{ }, "c64ec36a-4379-4374-8b1b-8f6f91bc70a0") },
		{ "f0d57b7c-dd48-43b6-b6a8-173cd7c71368", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.RightSquareBracket), "4042a064-6c43-4aea-abdb-18972be4fbbc"}, { new Terminator(DelimiterType.Comma), "082dc96a-aa2d-444e-b2b0-8a6bca7f6ba2"},}, new Dictionary<Terminator, ReduceInformation>{ }, "f0d57b7c-dd48-43b6-b6a8-173cd7c71368") },
		{ "0b6aad40-60a6-4144-998a-be547ee82f1f", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Statement), "104526cf-b558-4030-a5f2-aed66488d13d"}, { new NonTerminator(NonTerminatorType.Variable), "168b14a3-c5c2-4d3b-85d6-8401a5d72d3f"}, { Terminator.IdentifierTerminator, "a9bcbf2a-3d77-41a8-a4cb-35695590c1a7"}, { new NonTerminator(NonTerminatorType.ProcedureCall), "ee7bd04c-ba1b-4c05-9f5a-92dcbd37e67b"}, { new NonTerminator(NonTerminatorType.CompoundStatement), "372dfbe4-160d-4966-bc33-aafc60a96c43"}, { new Terminator(KeywordType.If), "67ff610b-d0c2-4e7d-9505-4dceffaa4ee1"}, { new Terminator(KeywordType.For), "3bee9f71-bf83-4ee3-a95e-aeefaa9e9f45"}, { new Terminator(KeywordType.Begin), "de7a627a-1e9b-41f4-835e-c30359b633fd"},}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(0, new NonTerminator(NonTerminatorType.Statement))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(0, new NonTerminator(NonTerminatorType.Statement))}, }, "0b6aad40-60a6-4144-998a-be547ee82f1f") },
		{ "59ce931f-0bea-4ab7-8afc-68f32f4e4aea", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.SimpleExpression), "388a77cf-f480-4e83-bfc2-cac4b03d73b4"}, { new NonTerminator(NonTerminatorType.Term), "d13a7c18-266f-43f2-bef4-f58fd96ec3ef"}, { new NonTerminator(NonTerminatorType.Factor), "06db0a54-0f03-4a6e-8443-0a44e1b38be3"}, { Terminator.NumberTerminator, "784756b8-c73b-4dd6-86b6-58b58716b6e3"}, { new NonTerminator(NonTerminatorType.Variable), "ccf37d94-9c0e-47c5-86fa-111ba6434e42"}, { new Terminator(DelimiterType.LeftParenthesis), "f4268929-efb8-43fa-8b73-0480990f7cbd"}, { Terminator.IdentifierTerminator, "efef7f5d-1a51-4f9f-8673-689387940dbf"}, { new Terminator(KeywordType.Not), "62237775-1fa6-4992-b4ea-3edef6dc3190"}, { new Terminator(OperatorType.Minus), "ba340278-4883-4f89-8ff0-53cc10442705"},}, new Dictionary<Terminator, ReduceInformation>{ }, "59ce931f-0bea-4ab7-8afc-68f32f4e4aea") },
		{ "7412aad9-254b-4fb3-9513-1d52100422dc", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Term), "15b9ae9f-0dae-4c19-8845-e276cf5a1fef"}, { new NonTerminator(NonTerminatorType.Factor), "a7870627-59bc-434d-bf03-0c1d5b649d45"}, { Terminator.NumberTerminator, "906b0111-f667-467d-a74a-c2be40fb7905"}, { new NonTerminator(NonTerminatorType.Variable), "ecc69635-01d5-4cdf-aaa8-750d1fa736f1"}, { new Terminator(DelimiterType.LeftParenthesis), "e41a1788-a93a-4fb3-846e-a2d33bd77958"}, { Terminator.IdentifierTerminator, "71f58c48-8093-40c7-b29d-0e1099642b47"}, { new Terminator(KeywordType.Not), "c5c60bb4-0f4e-4e7b-a222-1e9f7c12472d"}, { new Terminator(OperatorType.Minus), "cb5f054d-2d9d-417c-8843-2156b2d466a7"},}, new Dictionary<Terminator, ReduceInformation>{ }, "7412aad9-254b-4fb3-9513-1d52100422dc") },
		{ "2ad01207-64fe-4500-91d8-a8ebf7d2de07", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Factor), "1a89936f-2f66-41b4-b432-27744983b13f"}, { Terminator.NumberTerminator, "906b0111-f667-467d-a74a-c2be40fb7905"}, { new NonTerminator(NonTerminatorType.Variable), "ecc69635-01d5-4cdf-aaa8-750d1fa736f1"}, { new Terminator(DelimiterType.LeftParenthesis), "e41a1788-a93a-4fb3-846e-a2d33bd77958"}, { Terminator.IdentifierTerminator, "71f58c48-8093-40c7-b29d-0e1099642b47"}, { new Terminator(KeywordType.Not), "c5c60bb4-0f4e-4e7b-a222-1e9f7c12472d"}, { new Terminator(OperatorType.Minus), "cb5f054d-2d9d-417c-8843-2156b2d466a7"},}, new Dictionary<Terminator, ReduceInformation>{ }, "2ad01207-64fe-4500-91d8-a8ebf7d2de07") },
		{ "fbc2facf-c586-48bd-9185-7308bedfd222", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.RightParenthesis), "be1fbd00-6c48-4ffd-adbf-9293199041b0"},}, new Dictionary<Terminator, ReduceInformation>{ }, "fbc2facf-c586-48bd-9185-7308bedfd222") },
		{ "9573e208-f4ec-443d-b0c1-db78f4cc013e", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.ExpressionList), "3426c828-02b5-48ab-ae63-c562acf003ec"}, { new NonTerminator(NonTerminatorType.Expression), "4f59631a-5553-49da-9f0d-dda48b0d65b2"}, { new NonTerminator(NonTerminatorType.SimpleExpression), "e2a2e2c4-61e9-496f-8a60-ac108404247f"}, { new NonTerminator(NonTerminatorType.Term), "83d0456c-4817-49a5-bd20-c0d8a1c5b696"}, { new NonTerminator(NonTerminatorType.Factor), "f1db3e46-b446-4adc-93ae-5dfed0964c99"}, { Terminator.NumberTerminator, "0abfb7c0-083d-4242-a9a7-06183f7230f7"}, { new NonTerminator(NonTerminatorType.Variable), "a8c27dec-213a-4917-8b05-b0422260d1c1"}, { new Terminator(DelimiterType.LeftParenthesis), "45c4069f-f3cf-48f7-af29-8eb8303c47d6"}, { Terminator.IdentifierTerminator, "5aabfbf6-09f5-484f-9f95-ac0fa52c26c4"}, { new Terminator(KeywordType.Not), "253ff382-06bc-47a7-9645-cc04c48e1d0a"}, { new Terminator(OperatorType.Minus), "5708e399-73e5-4bcb-9aeb-f25d1e06c35c"},}, new Dictionary<Terminator, ReduceInformation>{ }, "9573e208-f4ec-443d-b0c1-db78f4cc013e") },
		{ "7f79fb52-93d9-4d81-9d37-9d509369ccff", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.Do), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.Equal), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.Less), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.Greater), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(KeywordType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(KeywordType.Mod), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(KeywordType.And), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.Plus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.Minus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(KeywordType.Or), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, }, "7f79fb52-93d9-4d81-9d37-9d509369ccff") },
		{ "df68949f-fa42-4574-93f0-6740cb47ce6f", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.ExpressionList), "493ffdbc-236a-4c12-bcb3-f09564462f9c"}, { new NonTerminator(NonTerminatorType.Expression), "a0ba5247-cbd6-46f6-b1fd-574979eb2b27"}, { new NonTerminator(NonTerminatorType.SimpleExpression), "e8b0426e-8437-45f8-a02f-3eebdfb48e84"}, { new NonTerminator(NonTerminatorType.Term), "cd4bfe7a-5a2e-41f1-a891-6359311b9403"}, { new NonTerminator(NonTerminatorType.Factor), "ceb0b15a-7ff9-4265-8766-716485412510"}, { Terminator.NumberTerminator, "06d206a2-8c81-4399-8790-c903a8acfe21"}, { new NonTerminator(NonTerminatorType.Variable), "422866b2-29d6-4004-b762-cb9ad9ddc5e7"}, { new Terminator(DelimiterType.LeftParenthesis), "4f42a0a3-3c4b-46a5-8dcc-b91007f93d5c"}, { Terminator.IdentifierTerminator, "7339bcec-b0ae-4611-83cd-08345e44324f"}, { new Terminator(KeywordType.Not), "bfdaacfe-9460-4db8-9469-712b6674efb5"}, { new Terminator(OperatorType.Minus), "eac356e4-6098-43be-9b50-3ff1a637a898"},}, new Dictionary<Terminator, ReduceInformation>{ }, "df68949f-fa42-4574-93f0-6740cb47ce6f") },
		{ "8959fb99-fe59-41c1-815c-8dd79b37b18a", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.Do), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Equal), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Less), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Greater), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, }, "8959fb99-fe59-41c1-815c-8dd79b37b18a") },
		{ "7232def0-9978-4352-a557-518acfcd2f5d", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.Do), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Equal), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Less), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Greater), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, }, "7232def0-9978-4352-a557-518acfcd2f5d") },
		{ "7bcc2d3e-3acb-48f8-92f8-11999bac7c09", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Term), "796c2394-7f4e-4f1c-b59f-287f0ac39eaa"}, { new NonTerminator(NonTerminatorType.Factor), "aaa1c6ff-81ea-415b-a7f2-0ede345af1dd"}, { Terminator.NumberTerminator, "f62db516-a4fe-45b8-8c43-7013e181d8d1"}, { new NonTerminator(NonTerminatorType.Variable), "e1c60364-c243-481b-b5ad-fac016bd7b32"}, { new Terminator(DelimiterType.LeftParenthesis), "d2ba8656-b05e-4fea-9aeb-3e7e5aef0cd3"}, { Terminator.IdentifierTerminator, "e81de486-3409-4f08-98da-f145bd2fb370"}, { new Terminator(KeywordType.Not), "0b574dc0-93f9-4835-b426-8a0d43f17059"}, { new Terminator(OperatorType.Minus), "b1e61a3b-133a-4ff2-bc0c-e4580ad4a90a"},}, new Dictionary<Terminator, ReduceInformation>{ }, "7bcc2d3e-3acb-48f8-92f8-11999bac7c09") },
		{ "a9d56e77-936e-4ced-953d-d412cea663ff", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Factor), "f635cd09-f3a0-400b-a2a6-6adb18d59ed6"}, { Terminator.NumberTerminator, "f62db516-a4fe-45b8-8c43-7013e181d8d1"}, { new NonTerminator(NonTerminatorType.Variable), "e1c60364-c243-481b-b5ad-fac016bd7b32"}, { new Terminator(DelimiterType.LeftParenthesis), "d2ba8656-b05e-4fea-9aeb-3e7e5aef0cd3"}, { Terminator.IdentifierTerminator, "e81de486-3409-4f08-98da-f145bd2fb370"}, { new Terminator(KeywordType.Not), "0b574dc0-93f9-4835-b426-8a0d43f17059"}, { new Terminator(OperatorType.Minus), "b1e61a3b-133a-4ff2-bc0c-e4580ad4a90a"},}, new Dictionary<Terminator, ReduceInformation>{ }, "a9d56e77-936e-4ced-953d-d412cea663ff") },
		{ "32f99453-2d57-424a-9ffe-cbf15f943b16", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.RightParenthesis), "4752c35b-9a6d-4cd2-8979-a3c54fef279b"},}, new Dictionary<Terminator, ReduceInformation>{ }, "32f99453-2d57-424a-9ffe-cbf15f943b16") },
		{ "8dfd88d6-8c53-4281-9d85-ed375411849f", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.ExpressionList), "efe17470-b085-48dd-a681-46f566b47368"}, { new NonTerminator(NonTerminatorType.Expression), "4f59631a-5553-49da-9f0d-dda48b0d65b2"}, { new NonTerminator(NonTerminatorType.SimpleExpression), "e2a2e2c4-61e9-496f-8a60-ac108404247f"}, { new NonTerminator(NonTerminatorType.Term), "83d0456c-4817-49a5-bd20-c0d8a1c5b696"}, { new NonTerminator(NonTerminatorType.Factor), "f1db3e46-b446-4adc-93ae-5dfed0964c99"}, { Terminator.NumberTerminator, "0abfb7c0-083d-4242-a9a7-06183f7230f7"}, { new NonTerminator(NonTerminatorType.Variable), "a8c27dec-213a-4917-8b05-b0422260d1c1"}, { new Terminator(DelimiterType.LeftParenthesis), "45c4069f-f3cf-48f7-af29-8eb8303c47d6"}, { Terminator.IdentifierTerminator, "5aabfbf6-09f5-484f-9f95-ac0fa52c26c4"}, { new Terminator(KeywordType.Not), "253ff382-06bc-47a7-9645-cc04c48e1d0a"}, { new Terminator(OperatorType.Minus), "5708e399-73e5-4bcb-9aeb-f25d1e06c35c"},}, new Dictionary<Terminator, ReduceInformation>{ }, "8dfd88d6-8c53-4281-9d85-ed375411849f") },
		{ "44d05a75-4b8d-4ea5-baf5-a5af902d5511", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.To), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(KeywordType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(KeywordType.Mod), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(KeywordType.And), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.Plus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.Minus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(KeywordType.Or), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, }, "44d05a75-4b8d-4ea5-baf5-a5af902d5511") },
		{ "4b2d5ff9-301d-49ab-add3-750972085a18", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.ExpressionList), "cfea8326-915f-4f23-a2cd-77739a67f337"}, { new NonTerminator(NonTerminatorType.Expression), "a0ba5247-cbd6-46f6-b1fd-574979eb2b27"}, { new NonTerminator(NonTerminatorType.SimpleExpression), "e8b0426e-8437-45f8-a02f-3eebdfb48e84"}, { new NonTerminator(NonTerminatorType.Term), "cd4bfe7a-5a2e-41f1-a891-6359311b9403"}, { new NonTerminator(NonTerminatorType.Factor), "ceb0b15a-7ff9-4265-8766-716485412510"}, { Terminator.NumberTerminator, "06d206a2-8c81-4399-8790-c903a8acfe21"}, { new NonTerminator(NonTerminatorType.Variable), "422866b2-29d6-4004-b762-cb9ad9ddc5e7"}, { new Terminator(DelimiterType.LeftParenthesis), "4f42a0a3-3c4b-46a5-8dcc-b91007f93d5c"}, { Terminator.IdentifierTerminator, "7339bcec-b0ae-4611-83cd-08345e44324f"}, { new Terminator(KeywordType.Not), "bfdaacfe-9460-4db8-9469-712b6674efb5"}, { new Terminator(OperatorType.Minus), "eac356e4-6098-43be-9b50-3ff1a637a898"},}, new Dictionary<Terminator, ReduceInformation>{ }, "4b2d5ff9-301d-49ab-add3-750972085a18") },
		{ "6ae08d08-0fbe-451d-b3cf-f18e0b3a95ab", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.To), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, }, "6ae08d08-0fbe-451d-b3cf-f18e0b3a95ab") },
		{ "060c208b-8cc7-4c14-98b0-8af53e8abb4d", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.To), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, }, "060c208b-8cc7-4c14-98b0-8af53e8abb4d") },
		{ "0e4d35ac-6900-4ca0-ba4c-b632b5d7e575", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.To), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Equal), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Less), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Greater), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, }, "0e4d35ac-6900-4ca0-ba4c-b632b5d7e575") },
		{ "dfb5f63b-5ee9-471a-ba9b-a0947a66ed49", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.To), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Equal), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Less), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Greater), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Mod), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.And), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Plus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Minus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Or), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, }, "dfb5f63b-5ee9-471a-ba9b-a0947a66ed49") },
		{ "94175ffd-57ec-4010-b747-b24e868cc768", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightSquareBracket), new ReduceInformation(5, new NonTerminator(NonTerminatorType.Period))}, { new Terminator(DelimiterType.Comma), new ReduceInformation(5, new NonTerminator(NonTerminatorType.Period))}, }, "94175ffd-57ec-4010-b747-b24e868cc768") },
		{ "670a2adf-00b7-4f63-94cf-912477ed0bca", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, }, "670a2adf-00b7-4f63-94cf-912477ed0bca") },
		{ "352d3eaf-86b5-408d-8b83-25c9b832a549", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Mod), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.And), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Plus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Minus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Or), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, }, "352d3eaf-86b5-408d-8b83-25c9b832a549") },
		{ "eb5b24f6-cf71-4d86-bef6-594aadcea8ee", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightSquareBracket), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(DelimiterType.Comma), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, }, "eb5b24f6-cf71-4d86-bef6-594aadcea8ee") },
		{ "47777f79-3c3b-4e78-9cc2-112c7017090e", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightSquareBracket), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Mod), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.And), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Plus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Minus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Or), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(DelimiterType.Comma), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, }, "47777f79-3c3b-4e78-9cc2-112c7017090e") },
		{ "fe9b513c-1e36-4556-9089-d476f10b98aa", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightParenthesis), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(DelimiterType.Comma), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, }, "fe9b513c-1e36-4556-9089-d476f10b98aa") },
		{ "95b0dba4-bae5-4a93-9c80-9304734676c6", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightParenthesis), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Mod), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.And), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Plus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Minus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Or), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(DelimiterType.Comma), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, }, "95b0dba4-bae5-4a93-9c80-9304734676c6") },
		{ "0795b67b-5a21-40c9-b7c7-1081ec0077e0", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.AddOperator), "2f700e55-441e-4692-9d37-82c941a5a1bf"}, { new Terminator(OperatorType.Plus), "b2a223d6-6905-4b72-a67b-96d8604fe688"}, { new Terminator(OperatorType.Minus), "569b7d42-5c9a-4ba6-b124-8dacd7f387f4"}, { new Terminator(KeywordType.Or), "37aaac08-c0a9-420d-bfd1-1ad8615a7513"},}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Expression))}, { new Terminator(KeywordType.Else), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Expression))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Expression))}, }, "0795b67b-5a21-40c9-b7c7-1081ec0077e0") },
		{ "c645b24f-43f0-42ef-b236-fefc9f8a59e3", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.MultiplyOperator), "a5ecd9a4-ae45-4220-bf8d-ed67dbb1efdd"}, { new Terminator(OperatorType.Multiply), "e576c109-5246-4139-8248-415b8e3e68de"}, { new Terminator(OperatorType.Divide), "6a567f32-04a0-49b8-bf8e-42f55ee0600f"}, { new Terminator(KeywordType.Divide), "86be9c46-a085-4183-9264-b12d73fde46e"}, { new Terminator(KeywordType.Mod), "d7708b97-7da8-492a-a64f-6196ae9527f7"}, { new Terminator(KeywordType.And), "271d9f46-4be0-495c-a90b-ae8f19ee928b"},}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Plus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Minus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(KeywordType.Or), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(KeywordType.Else), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, }, "c645b24f-43f0-42ef-b236-fefc9f8a59e3") },
		{ "6e59d416-a3a6-4e03-ae3b-117366106542", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Mod), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.And), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Plus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Minus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Or), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Else), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, }, "6e59d416-a3a6-4e03-ae3b-117366106542") },
		{ "1559bac5-de37-4c11-ba19-090208a71781", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Else), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, }, "1559bac5-de37-4c11-ba19-090208a71781") },
		{ "f846fa35-cf45-4ce4-9dba-97755e4630d3", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Else), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, }, "f846fa35-cf45-4ce4-9dba-97755e4630d3") },
		{ "587ba18e-861d-49e1-b28a-b4381855be14", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Expression), "ae377318-d2f3-4325-9df8-9938f1eda4e8"}, { new NonTerminator(NonTerminatorType.SimpleExpression), "e566a9ad-f17c-46ff-981f-ee4fa81d2d3d"}, { new NonTerminator(NonTerminatorType.Term), "1f1e4a05-7d30-44f4-aa5f-5caacfbce4b2"}, { new NonTerminator(NonTerminatorType.Factor), "ff72d68f-ac20-46e8-94ed-090e841128c1"}, { Terminator.NumberTerminator, "79939c29-990e-4743-8a8a-5fccd2ec415d"}, { new NonTerminator(NonTerminatorType.Variable), "28f501cb-0b97-40c3-8a64-d214832a6789"}, { new Terminator(DelimiterType.LeftParenthesis), "f1464d64-129c-4d68-a61e-43296e963ee1"}, { Terminator.IdentifierTerminator, "9dd45908-639c-4e69-97e4-2471cdf5e189"}, { new Terminator(KeywordType.Not), "303776ee-584f-4977-b76a-9449d6a4e1ad"}, { new Terminator(OperatorType.Minus), "2401b03b-4e43-4409-989c-a4471708a795"},}, new Dictionary<Terminator, ReduceInformation>{ }, "587ba18e-861d-49e1-b28a-b4381855be14") },
		{ "ba9b99d4-067b-47d8-b627-a60322d8c29d", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.LeftParenthesis), "0ac65a24-4b49-46b9-9669-fa7451f75b6f"}, { new NonTerminator(NonTerminatorType.IdVarPart), "65e8488c-1040-45e8-a990-04c1de7e2159"}, { new Terminator(DelimiterType.LeftSquareBracket), "2b9577b1-5a25-44ab-b363-c6a6b6fe3afa"},}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Divide), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Divide), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Mod), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.And), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Plus), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Minus), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Or), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Else), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, }, "ba9b99d4-067b-47d8-b627-a60322d8c29d") },
		{ "da0127ed-aab2-42d4-8efa-8cf170904675", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Factor), "61925788-3300-4c5a-85cb-2d14ad0c2e9f"}, { Terminator.NumberTerminator, "1559bac5-de37-4c11-ba19-090208a71781"}, { new NonTerminator(NonTerminatorType.Variable), "f846fa35-cf45-4ce4-9dba-97755e4630d3"}, { new Terminator(DelimiterType.LeftParenthesis), "587ba18e-861d-49e1-b28a-b4381855be14"}, { Terminator.IdentifierTerminator, "ba9b99d4-067b-47d8-b627-a60322d8c29d"}, { new Terminator(KeywordType.Not), "da0127ed-aab2-42d4-8efa-8cf170904675"}, { new Terminator(OperatorType.Minus), "a66fbe37-bcfb-4f38-a243-3e471986c2b2"},}, new Dictionary<Terminator, ReduceInformation>{ }, "da0127ed-aab2-42d4-8efa-8cf170904675") },
		{ "a66fbe37-bcfb-4f38-a243-3e471986c2b2", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Factor), "024fa528-b762-4ca7-9664-a2e7de6e6d09"}, { Terminator.NumberTerminator, "1559bac5-de37-4c11-ba19-090208a71781"}, { new NonTerminator(NonTerminatorType.Variable), "f846fa35-cf45-4ce4-9dba-97755e4630d3"}, { new Terminator(DelimiterType.LeftParenthesis), "587ba18e-861d-49e1-b28a-b4381855be14"}, { Terminator.IdentifierTerminator, "ba9b99d4-067b-47d8-b627-a60322d8c29d"}, { new Terminator(KeywordType.Not), "da0127ed-aab2-42d4-8efa-8cf170904675"}, { new Terminator(OperatorType.Minus), "a66fbe37-bcfb-4f38-a243-3e471986c2b2"},}, new Dictionary<Terminator, ReduceInformation>{ }, "a66fbe37-bcfb-4f38-a243-3e471986c2b2") },
		{ "648315c9-c592-4c1d-8d80-68f89504009e", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.MultiplyOperator), "aa2b1cf1-871a-4c42-9c5c-99e15f272913"}, { new Terminator(OperatorType.Multiply), "e576c109-5246-4139-8248-415b8e3e68de"}, { new Terminator(OperatorType.Divide), "6a567f32-04a0-49b8-bf8e-42f55ee0600f"}, { new Terminator(KeywordType.Divide), "86be9c46-a085-4183-9264-b12d73fde46e"}, { new Terminator(KeywordType.Mod), "d7708b97-7da8-492a-a64f-6196ae9527f7"}, { new Terminator(KeywordType.And), "271d9f46-4be0-495c-a90b-ae8f19ee928b"},}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Equal), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Less), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Greater), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Plus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Minus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(KeywordType.Or), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(KeywordType.Else), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, }, "648315c9-c592-4c1d-8d80-68f89504009e") },
		{ "7523711f-4646-4a87-945d-1f54799a4a78", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Equal), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Less), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Greater), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Mod), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.And), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Plus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Minus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Or), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Else), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, }, "7523711f-4646-4a87-945d-1f54799a4a78") },
		{ "00227edf-fc74-4b10-a5dc-bf25bbb14f9f", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Equal), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Less), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Greater), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Else), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, }, "00227edf-fc74-4b10-a5dc-bf25bbb14f9f") },
		{ "a1706667-9803-40c2-8b67-b3e932d6aca5", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.RightParenthesis), "fd6e4f30-8b9e-4243-8ce9-2859e7ae376c"}, { new Terminator(DelimiterType.Comma), "077bd83e-2dfc-4686-bf99-ec6992c05b60"},}, new Dictionary<Terminator, ReduceInformation>{ }, "a1706667-9803-40c2-8b67-b3e932d6aca5") },
		{ "7b0a5c49-12ff-4104-84eb-88adfa26f300", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.RightSquareBracket), "33481e80-ce4c-490f-b0a4-67f43726cf62"}, { new Terminator(DelimiterType.Comma), "082dc96a-aa2d-444e-b2b0-8a6bca7f6ba2"},}, new Dictionary<Terminator, ReduceInformation>{ }, "7b0a5c49-12ff-4104-84eb-88adfa26f300") },
		{ "49f5f016-7e61-4fe0-a2d3-570e305a9f3d", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(5, new NonTerminator(NonTerminatorType.Statement))}, { new Terminator(KeywordType.Else), new ReduceInformation(5, new NonTerminator(NonTerminatorType.Statement))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(5, new NonTerminator(NonTerminatorType.Statement))}, }, "49f5f016-7e61-4fe0-a2d3-570e305a9f3d") },
		{ "eed27dcb-7c83-4c96-9f7e-6ad65df5e808", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Statement), "3ed202f4-bc2a-43e0-9d91-a6425e3b5c57"}, { new NonTerminator(NonTerminatorType.Variable), "f7849288-2cd0-4c68-aeee-abb4f910fa32"}, { Terminator.IdentifierTerminator, "403840ce-3751-4324-a4da-a2b0a7653957"}, { new NonTerminator(NonTerminatorType.ProcedureCall), "504c43d8-8335-43de-8838-5c3205046f5d"}, { new NonTerminator(NonTerminatorType.CompoundStatement), "15b27fe6-dc50-4e57-8246-bc2e8a5eb05a"}, { new Terminator(KeywordType.If), "1e290bf0-8ecb-4a2d-b5dc-bd2aa7ff6505"}, { new Terminator(KeywordType.For), "83bfcb23-a73e-4ba5-ac65-68e5a019cc6a"}, { new Terminator(KeywordType.Begin), "1fd17dc7-14a7-4500-8bc1-e768a6969eae"},}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(0, new NonTerminator(NonTerminatorType.Statement))}, { new Terminator(KeywordType.Else), new ReduceInformation(0, new NonTerminator(NonTerminatorType.Statement))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(0, new NonTerminator(NonTerminatorType.Statement))}, }, "eed27dcb-7c83-4c96-9f7e-6ad65df5e808") },
		{ "96881fa8-667c-4ded-a1ef-ead45665aa0e", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Expression), "b851545e-dcc9-4aa6-8754-83420ef5cabf"}, { new NonTerminator(NonTerminatorType.SimpleExpression), "edabf765-08f8-4d58-8722-863c2b38bf71"}, { new NonTerminator(NonTerminatorType.Term), "96b22d74-a3ce-42d1-980f-ab9ca7edff4d"}, { new NonTerminator(NonTerminatorType.Factor), "a7870627-59bc-434d-bf03-0c1d5b649d45"}, { Terminator.NumberTerminator, "906b0111-f667-467d-a74a-c2be40fb7905"}, { new NonTerminator(NonTerminatorType.Variable), "ecc69635-01d5-4cdf-aaa8-750d1fa736f1"}, { new Terminator(DelimiterType.LeftParenthesis), "e41a1788-a93a-4fb3-846e-a2d33bd77958"}, { Terminator.IdentifierTerminator, "71f58c48-8093-40c7-b29d-0e1099642b47"}, { new Terminator(KeywordType.Not), "c5c60bb4-0f4e-4e7b-a222-1e9f7c12472d"}, { new Terminator(OperatorType.Minus), "cb5f054d-2d9d-417c-8843-2156b2d466a7"},}, new Dictionary<Terminator, ReduceInformation>{ }, "96881fa8-667c-4ded-a1ef-ead45665aa0e") },
		{ "9ccb7a92-a877-468c-9336-be8774db708c", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightParenthesis), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, }, "9ccb7a92-a877-468c-9336-be8774db708c") },
		{ "4042a064-6c43-4aea-abdb-18972be4fbbc", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightParenthesis), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Mod), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.And), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Plus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Minus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Or), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, }, "4042a064-6c43-4aea-abdb-18972be4fbbc") },
		{ "104526cf-b558-4030-a5f2-aed66488d13d", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(8, new NonTerminator(NonTerminatorType.Statement))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(8, new NonTerminator(NonTerminatorType.Statement))}, }, "104526cf-b558-4030-a5f2-aed66488d13d") },
		{ "388a77cf-f480-4e83-bfc2-cac4b03d73b4", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.AddOperator), "7588b849-d2f3-43c0-9ef3-8eb0cbdfeebe"}, { new Terminator(OperatorType.Plus), "b2a223d6-6905-4b72-a67b-96d8604fe688"}, { new Terminator(OperatorType.Minus), "569b7d42-5c9a-4ba6-b124-8dacd7f387f4"}, { new Terminator(KeywordType.Or), "37aaac08-c0a9-420d-bfd1-1ad8615a7513"},}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.Do), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Expression))}, }, "388a77cf-f480-4e83-bfc2-cac4b03d73b4") },
		{ "d13a7c18-266f-43f2-bef4-f58fd96ec3ef", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.MultiplyOperator), "3cf82f86-e590-49fe-9fd6-3f65d227dbd3"}, { new Terminator(OperatorType.Multiply), "e576c109-5246-4139-8248-415b8e3e68de"}, { new Terminator(OperatorType.Divide), "6a567f32-04a0-49b8-bf8e-42f55ee0600f"}, { new Terminator(KeywordType.Divide), "86be9c46-a085-4183-9264-b12d73fde46e"}, { new Terminator(KeywordType.Mod), "d7708b97-7da8-492a-a64f-6196ae9527f7"}, { new Terminator(KeywordType.And), "271d9f46-4be0-495c-a90b-ae8f19ee928b"},}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.Do), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Plus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Minus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(KeywordType.Or), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, }, "d13a7c18-266f-43f2-bef4-f58fd96ec3ef") },
		{ "06db0a54-0f03-4a6e-8443-0a44e1b38be3", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.Do), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Mod), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.And), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Plus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Minus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Or), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, }, "06db0a54-0f03-4a6e-8443-0a44e1b38be3") },
		{ "784756b8-c73b-4dd6-86b6-58b58716b6e3", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.Do), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, }, "784756b8-c73b-4dd6-86b6-58b58716b6e3") },
		{ "ccf37d94-9c0e-47c5-86fa-111ba6434e42", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.Do), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, }, "ccf37d94-9c0e-47c5-86fa-111ba6434e42") },
		{ "f4268929-efb8-43fa-8b73-0480990f7cbd", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Expression), "ac8f11f1-7fe0-4b8e-a6a9-fda3b3414999"}, { new NonTerminator(NonTerminatorType.SimpleExpression), "e566a9ad-f17c-46ff-981f-ee4fa81d2d3d"}, { new NonTerminator(NonTerminatorType.Term), "1f1e4a05-7d30-44f4-aa5f-5caacfbce4b2"}, { new NonTerminator(NonTerminatorType.Factor), "ff72d68f-ac20-46e8-94ed-090e841128c1"}, { Terminator.NumberTerminator, "79939c29-990e-4743-8a8a-5fccd2ec415d"}, { new NonTerminator(NonTerminatorType.Variable), "28f501cb-0b97-40c3-8a64-d214832a6789"}, { new Terminator(DelimiterType.LeftParenthesis), "f1464d64-129c-4d68-a61e-43296e963ee1"}, { Terminator.IdentifierTerminator, "9dd45908-639c-4e69-97e4-2471cdf5e189"}, { new Terminator(KeywordType.Not), "303776ee-584f-4977-b76a-9449d6a4e1ad"}, { new Terminator(OperatorType.Minus), "2401b03b-4e43-4409-989c-a4471708a795"},}, new Dictionary<Terminator, ReduceInformation>{ }, "f4268929-efb8-43fa-8b73-0480990f7cbd") },
		{ "efef7f5d-1a51-4f9f-8673-689387940dbf", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.LeftParenthesis), "4011bcd2-6538-44d0-aa99-9ee948e6b2d2"}, { new NonTerminator(NonTerminatorType.IdVarPart), "a0fc66e2-aa44-4e5a-b418-7a46f55c9a1b"}, { new Terminator(DelimiterType.LeftSquareBracket), "119d2e5e-6d07-44bd-b592-9864d7389981"},}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.Do), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Divide), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Divide), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Mod), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.And), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Plus), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Minus), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Or), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, }, "efef7f5d-1a51-4f9f-8673-689387940dbf") },
		{ "62237775-1fa6-4992-b4ea-3edef6dc3190", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Factor), "19b21fc1-ab76-4637-b60c-1db8adfb2aa1"}, { Terminator.NumberTerminator, "784756b8-c73b-4dd6-86b6-58b58716b6e3"}, { new NonTerminator(NonTerminatorType.Variable), "ccf37d94-9c0e-47c5-86fa-111ba6434e42"}, { new Terminator(DelimiterType.LeftParenthesis), "f4268929-efb8-43fa-8b73-0480990f7cbd"}, { Terminator.IdentifierTerminator, "efef7f5d-1a51-4f9f-8673-689387940dbf"}, { new Terminator(KeywordType.Not), "62237775-1fa6-4992-b4ea-3edef6dc3190"}, { new Terminator(OperatorType.Minus), "ba340278-4883-4f89-8ff0-53cc10442705"},}, new Dictionary<Terminator, ReduceInformation>{ }, "62237775-1fa6-4992-b4ea-3edef6dc3190") },
		{ "ba340278-4883-4f89-8ff0-53cc10442705", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Factor), "3e00b747-3315-4aa7-bc54-1f5c87783637"}, { Terminator.NumberTerminator, "784756b8-c73b-4dd6-86b6-58b58716b6e3"}, { new NonTerminator(NonTerminatorType.Variable), "ccf37d94-9c0e-47c5-86fa-111ba6434e42"}, { new Terminator(DelimiterType.LeftParenthesis), "f4268929-efb8-43fa-8b73-0480990f7cbd"}, { Terminator.IdentifierTerminator, "efef7f5d-1a51-4f9f-8673-689387940dbf"}, { new Terminator(KeywordType.Not), "62237775-1fa6-4992-b4ea-3edef6dc3190"}, { new Terminator(OperatorType.Minus), "ba340278-4883-4f89-8ff0-53cc10442705"},}, new Dictionary<Terminator, ReduceInformation>{ }, "ba340278-4883-4f89-8ff0-53cc10442705") },
		{ "15b9ae9f-0dae-4c19-8845-e276cf5a1fef", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.MultiplyOperator), "2ad01207-64fe-4500-91d8-a8ebf7d2de07"}, { new Terminator(OperatorType.Multiply), "e576c109-5246-4139-8248-415b8e3e68de"}, { new Terminator(OperatorType.Divide), "6a567f32-04a0-49b8-bf8e-42f55ee0600f"}, { new Terminator(KeywordType.Divide), "86be9c46-a085-4183-9264-b12d73fde46e"}, { new Terminator(KeywordType.Mod), "d7708b97-7da8-492a-a64f-6196ae9527f7"}, { new Terminator(KeywordType.And), "271d9f46-4be0-495c-a90b-ae8f19ee928b"},}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.Do), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Equal), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Less), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Greater), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Plus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Minus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(KeywordType.Or), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, }, "15b9ae9f-0dae-4c19-8845-e276cf5a1fef") },
		{ "1a89936f-2f66-41b4-b432-27744983b13f", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.Do), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Equal), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Less), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Greater), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Mod), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.And), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Plus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Minus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Or), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, }, "1a89936f-2f66-41b4-b432-27744983b13f") },
		{ "be1fbd00-6c48-4ffd-adbf-9293199041b0", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.Do), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Equal), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Less), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Greater), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, }, "be1fbd00-6c48-4ffd-adbf-9293199041b0") },
		{ "3426c828-02b5-48ab-ae63-c562acf003ec", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.RightParenthesis), "c16b82fe-6e72-4e58-b646-4f9b8c696287"}, { new Terminator(DelimiterType.Comma), "077bd83e-2dfc-4686-bf99-ec6992c05b60"},}, new Dictionary<Terminator, ReduceInformation>{ }, "3426c828-02b5-48ab-ae63-c562acf003ec") },
		{ "493ffdbc-236a-4c12-bcb3-f09564462f9c", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.RightSquareBracket), "92a32c49-f3dc-4975-9fd8-d4c0efff00eb"}, { new Terminator(DelimiterType.Comma), "082dc96a-aa2d-444e-b2b0-8a6bca7f6ba2"},}, new Dictionary<Terminator, ReduceInformation>{ }, "493ffdbc-236a-4c12-bcb3-f09564462f9c") },
		{ "796c2394-7f4e-4f1c-b59f-287f0ac39eaa", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.MultiplyOperator), "a9d56e77-936e-4ced-953d-d412cea663ff"}, { new Terminator(OperatorType.Multiply), "e576c109-5246-4139-8248-415b8e3e68de"}, { new Terminator(OperatorType.Divide), "6a567f32-04a0-49b8-bf8e-42f55ee0600f"}, { new Terminator(KeywordType.Divide), "86be9c46-a085-4183-9264-b12d73fde46e"}, { new Terminator(KeywordType.Mod), "d7708b97-7da8-492a-a64f-6196ae9527f7"}, { new Terminator(KeywordType.And), "271d9f46-4be0-495c-a90b-ae8f19ee928b"},}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.To), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Plus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Minus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(KeywordType.Or), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, }, "796c2394-7f4e-4f1c-b59f-287f0ac39eaa") },
		{ "f635cd09-f3a0-400b-a2a6-6adb18d59ed6", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.To), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Mod), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.And), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Plus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Minus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Or), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, }, "f635cd09-f3a0-400b-a2a6-6adb18d59ed6") },
		{ "4752c35b-9a6d-4cd2-8979-a3c54fef279b", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.To), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, }, "4752c35b-9a6d-4cd2-8979-a3c54fef279b") },
		{ "efe17470-b085-48dd-a681-46f566b47368", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.RightParenthesis), "21a4c2c2-7306-4c0f-83ee-6ac2178084b4"}, { new Terminator(DelimiterType.Comma), "077bd83e-2dfc-4686-bf99-ec6992c05b60"},}, new Dictionary<Terminator, ReduceInformation>{ }, "efe17470-b085-48dd-a681-46f566b47368") },
		{ "cfea8326-915f-4f23-a2cd-77739a67f337", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.RightSquareBracket), "415cb287-0f72-44b8-ba43-568b757aa408"}, { new Terminator(DelimiterType.Comma), "082dc96a-aa2d-444e-b2b0-8a6bca7f6ba2"},}, new Dictionary<Terminator, ReduceInformation>{ }, "cfea8326-915f-4f23-a2cd-77739a67f337") },
		{ "2f700e55-441e-4692-9d37-82c941a5a1bf", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Term), "7ef38f54-fe98-4fcc-a9bd-beab32123ba4"}, { new NonTerminator(NonTerminatorType.Factor), "6e59d416-a3a6-4e03-ae3b-117366106542"}, { Terminator.NumberTerminator, "1559bac5-de37-4c11-ba19-090208a71781"}, { new NonTerminator(NonTerminatorType.Variable), "f846fa35-cf45-4ce4-9dba-97755e4630d3"}, { new Terminator(DelimiterType.LeftParenthesis), "587ba18e-861d-49e1-b28a-b4381855be14"}, { Terminator.IdentifierTerminator, "ba9b99d4-067b-47d8-b627-a60322d8c29d"}, { new Terminator(KeywordType.Not), "da0127ed-aab2-42d4-8efa-8cf170904675"}, { new Terminator(OperatorType.Minus), "a66fbe37-bcfb-4f38-a243-3e471986c2b2"},}, new Dictionary<Terminator, ReduceInformation>{ }, "2f700e55-441e-4692-9d37-82c941a5a1bf") },
		{ "a5ecd9a4-ae45-4220-bf8d-ed67dbb1efdd", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Factor), "3d691087-ad63-4f0f-a50a-3a9205410797"}, { Terminator.NumberTerminator, "1559bac5-de37-4c11-ba19-090208a71781"}, { new NonTerminator(NonTerminatorType.Variable), "f846fa35-cf45-4ce4-9dba-97755e4630d3"}, { new Terminator(DelimiterType.LeftParenthesis), "587ba18e-861d-49e1-b28a-b4381855be14"}, { Terminator.IdentifierTerminator, "ba9b99d4-067b-47d8-b627-a60322d8c29d"}, { new Terminator(KeywordType.Not), "da0127ed-aab2-42d4-8efa-8cf170904675"}, { new Terminator(OperatorType.Minus), "a66fbe37-bcfb-4f38-a243-3e471986c2b2"},}, new Dictionary<Terminator, ReduceInformation>{ }, "a5ecd9a4-ae45-4220-bf8d-ed67dbb1efdd") },
		{ "ae377318-d2f3-4325-9df8-9938f1eda4e8", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.RightParenthesis), "ad50edde-2192-482c-b4f9-477fe5d2c19e"},}, new Dictionary<Terminator, ReduceInformation>{ }, "ae377318-d2f3-4325-9df8-9938f1eda4e8") },
		{ "0ac65a24-4b49-46b9-9669-fa7451f75b6f", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.ExpressionList), "701f351c-1d0b-4a57-8eca-ee87badea333"}, { new NonTerminator(NonTerminatorType.Expression), "4f59631a-5553-49da-9f0d-dda48b0d65b2"}, { new NonTerminator(NonTerminatorType.SimpleExpression), "e2a2e2c4-61e9-496f-8a60-ac108404247f"}, { new NonTerminator(NonTerminatorType.Term), "83d0456c-4817-49a5-bd20-c0d8a1c5b696"}, { new NonTerminator(NonTerminatorType.Factor), "f1db3e46-b446-4adc-93ae-5dfed0964c99"}, { Terminator.NumberTerminator, "0abfb7c0-083d-4242-a9a7-06183f7230f7"}, { new NonTerminator(NonTerminatorType.Variable), "a8c27dec-213a-4917-8b05-b0422260d1c1"}, { new Terminator(DelimiterType.LeftParenthesis), "45c4069f-f3cf-48f7-af29-8eb8303c47d6"}, { Terminator.IdentifierTerminator, "5aabfbf6-09f5-484f-9f95-ac0fa52c26c4"}, { new Terminator(KeywordType.Not), "253ff382-06bc-47a7-9645-cc04c48e1d0a"}, { new Terminator(OperatorType.Minus), "5708e399-73e5-4bcb-9aeb-f25d1e06c35c"},}, new Dictionary<Terminator, ReduceInformation>{ }, "0ac65a24-4b49-46b9-9669-fa7451f75b6f") },
		{ "65e8488c-1040-45e8-a990-04c1de7e2159", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(KeywordType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(KeywordType.Mod), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(KeywordType.And), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.Plus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.Minus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(KeywordType.Or), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(KeywordType.Else), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, }, "65e8488c-1040-45e8-a990-04c1de7e2159") },
		{ "2b9577b1-5a25-44ab-b363-c6a6b6fe3afa", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.ExpressionList), "de6c6542-dcfe-430c-a8eb-29e7fe90cb06"}, { new NonTerminator(NonTerminatorType.Expression), "a0ba5247-cbd6-46f6-b1fd-574979eb2b27"}, { new NonTerminator(NonTerminatorType.SimpleExpression), "e8b0426e-8437-45f8-a02f-3eebdfb48e84"}, { new NonTerminator(NonTerminatorType.Term), "cd4bfe7a-5a2e-41f1-a891-6359311b9403"}, { new NonTerminator(NonTerminatorType.Factor), "ceb0b15a-7ff9-4265-8766-716485412510"}, { Terminator.NumberTerminator, "06d206a2-8c81-4399-8790-c903a8acfe21"}, { new NonTerminator(NonTerminatorType.Variable), "422866b2-29d6-4004-b762-cb9ad9ddc5e7"}, { new Terminator(DelimiterType.LeftParenthesis), "4f42a0a3-3c4b-46a5-8dcc-b91007f93d5c"}, { Terminator.IdentifierTerminator, "7339bcec-b0ae-4611-83cd-08345e44324f"}, { new Terminator(KeywordType.Not), "bfdaacfe-9460-4db8-9469-712b6674efb5"}, { new Terminator(OperatorType.Minus), "eac356e4-6098-43be-9b50-3ff1a637a898"},}, new Dictionary<Terminator, ReduceInformation>{ }, "2b9577b1-5a25-44ab-b363-c6a6b6fe3afa") },
		{ "61925788-3300-4c5a-85cb-2d14ad0c2e9f", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Else), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, }, "61925788-3300-4c5a-85cb-2d14ad0c2e9f") },
		{ "024fa528-b762-4ca7-9664-a2e7de6e6d09", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Else), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, }, "024fa528-b762-4ca7-9664-a2e7de6e6d09") },
		{ "fd6e4f30-8b9e-4243-8ce9-2859e7ae376c", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Equal), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Less), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Greater), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Else), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, }, "fd6e4f30-8b9e-4243-8ce9-2859e7ae376c") },
		{ "33481e80-ce4c-490f-b0a4-67f43726cf62", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Equal), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Less), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Greater), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Mod), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.And), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Plus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Minus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Or), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Else), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, }, "33481e80-ce4c-490f-b0a4-67f43726cf62") },
		{ "3ed202f4-bc2a-43e0-9d91-a6425e3b5c57", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(2, new NonTerminator(NonTerminatorType.ElsePart))}, { new Terminator(KeywordType.Else), new ReduceInformation(2, new NonTerminator(NonTerminatorType.ElsePart))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(2, new NonTerminator(NonTerminatorType.ElsePart))}, }, "3ed202f4-bc2a-43e0-9d91-a6425e3b5c57") },
		{ "b851545e-dcc9-4aa6-8754-83420ef5cabf", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(KeywordType.Do), "ec89d2c8-3c95-433e-b6c6-a635fc98dbb6"},}, new Dictionary<Terminator, ReduceInformation>{ }, "b851545e-dcc9-4aa6-8754-83420ef5cabf") },
		{ "7588b849-d2f3-43c0-9ef3-8eb0cbdfeebe", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Term), "e6f79bff-9f27-4b9d-91ae-7528965e1569"}, { new NonTerminator(NonTerminatorType.Factor), "06db0a54-0f03-4a6e-8443-0a44e1b38be3"}, { Terminator.NumberTerminator, "784756b8-c73b-4dd6-86b6-58b58716b6e3"}, { new NonTerminator(NonTerminatorType.Variable), "ccf37d94-9c0e-47c5-86fa-111ba6434e42"}, { new Terminator(DelimiterType.LeftParenthesis), "f4268929-efb8-43fa-8b73-0480990f7cbd"}, { Terminator.IdentifierTerminator, "efef7f5d-1a51-4f9f-8673-689387940dbf"}, { new Terminator(KeywordType.Not), "62237775-1fa6-4992-b4ea-3edef6dc3190"}, { new Terminator(OperatorType.Minus), "ba340278-4883-4f89-8ff0-53cc10442705"},}, new Dictionary<Terminator, ReduceInformation>{ }, "7588b849-d2f3-43c0-9ef3-8eb0cbdfeebe") },
		{ "3cf82f86-e590-49fe-9fd6-3f65d227dbd3", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Factor), "fc33e10e-31dd-4bf5-b2c5-fa308ae42160"}, { Terminator.NumberTerminator, "784756b8-c73b-4dd6-86b6-58b58716b6e3"}, { new NonTerminator(NonTerminatorType.Variable), "ccf37d94-9c0e-47c5-86fa-111ba6434e42"}, { new Terminator(DelimiterType.LeftParenthesis), "f4268929-efb8-43fa-8b73-0480990f7cbd"}, { Terminator.IdentifierTerminator, "efef7f5d-1a51-4f9f-8673-689387940dbf"}, { new Terminator(KeywordType.Not), "62237775-1fa6-4992-b4ea-3edef6dc3190"}, { new Terminator(OperatorType.Minus), "ba340278-4883-4f89-8ff0-53cc10442705"},}, new Dictionary<Terminator, ReduceInformation>{ }, "3cf82f86-e590-49fe-9fd6-3f65d227dbd3") },
		{ "ac8f11f1-7fe0-4b8e-a6a9-fda3b3414999", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.RightParenthesis), "9d9f9e17-3d6f-47c1-9674-2ddc47b05109"},}, new Dictionary<Terminator, ReduceInformation>{ }, "ac8f11f1-7fe0-4b8e-a6a9-fda3b3414999") },
		{ "4011bcd2-6538-44d0-aa99-9ee948e6b2d2", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.ExpressionList), "bd8826e0-0ad5-43bf-a8d9-2b7ec3785625"}, { new NonTerminator(NonTerminatorType.Expression), "4f59631a-5553-49da-9f0d-dda48b0d65b2"}, { new NonTerminator(NonTerminatorType.SimpleExpression), "e2a2e2c4-61e9-496f-8a60-ac108404247f"}, { new NonTerminator(NonTerminatorType.Term), "83d0456c-4817-49a5-bd20-c0d8a1c5b696"}, { new NonTerminator(NonTerminatorType.Factor), "f1db3e46-b446-4adc-93ae-5dfed0964c99"}, { Terminator.NumberTerminator, "0abfb7c0-083d-4242-a9a7-06183f7230f7"}, { new NonTerminator(NonTerminatorType.Variable), "a8c27dec-213a-4917-8b05-b0422260d1c1"}, { new Terminator(DelimiterType.LeftParenthesis), "45c4069f-f3cf-48f7-af29-8eb8303c47d6"}, { Terminator.IdentifierTerminator, "5aabfbf6-09f5-484f-9f95-ac0fa52c26c4"}, { new Terminator(KeywordType.Not), "253ff382-06bc-47a7-9645-cc04c48e1d0a"}, { new Terminator(OperatorType.Minus), "5708e399-73e5-4bcb-9aeb-f25d1e06c35c"},}, new Dictionary<Terminator, ReduceInformation>{ }, "4011bcd2-6538-44d0-aa99-9ee948e6b2d2") },
		{ "a0fc66e2-aa44-4e5a-b418-7a46f55c9a1b", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.Do), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(KeywordType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(KeywordType.Mod), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(KeywordType.And), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.Plus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.Minus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(KeywordType.Or), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, }, "a0fc66e2-aa44-4e5a-b418-7a46f55c9a1b") },
		{ "119d2e5e-6d07-44bd-b592-9864d7389981", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.ExpressionList), "59946f05-e479-4422-b708-317b6245a1ab"}, { new NonTerminator(NonTerminatorType.Expression), "a0ba5247-cbd6-46f6-b1fd-574979eb2b27"}, { new NonTerminator(NonTerminatorType.SimpleExpression), "e8b0426e-8437-45f8-a02f-3eebdfb48e84"}, { new NonTerminator(NonTerminatorType.Term), "cd4bfe7a-5a2e-41f1-a891-6359311b9403"}, { new NonTerminator(NonTerminatorType.Factor), "ceb0b15a-7ff9-4265-8766-716485412510"}, { Terminator.NumberTerminator, "06d206a2-8c81-4399-8790-c903a8acfe21"}, { new NonTerminator(NonTerminatorType.Variable), "422866b2-29d6-4004-b762-cb9ad9ddc5e7"}, { new Terminator(DelimiterType.LeftParenthesis), "4f42a0a3-3c4b-46a5-8dcc-b91007f93d5c"}, { Terminator.IdentifierTerminator, "7339bcec-b0ae-4611-83cd-08345e44324f"}, { new Terminator(KeywordType.Not), "bfdaacfe-9460-4db8-9469-712b6674efb5"}, { new Terminator(OperatorType.Minus), "eac356e4-6098-43be-9b50-3ff1a637a898"},}, new Dictionary<Terminator, ReduceInformation>{ }, "119d2e5e-6d07-44bd-b592-9864d7389981") },
		{ "19b21fc1-ab76-4637-b60c-1db8adfb2aa1", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.Do), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, }, "19b21fc1-ab76-4637-b60c-1db8adfb2aa1") },
		{ "3e00b747-3315-4aa7-bc54-1f5c87783637", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.Do), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, }, "3e00b747-3315-4aa7-bc54-1f5c87783637") },
		{ "c16b82fe-6e72-4e58-b646-4f9b8c696287", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.Do), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Equal), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Less), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Greater), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, }, "c16b82fe-6e72-4e58-b646-4f9b8c696287") },
		{ "92a32c49-f3dc-4975-9fd8-d4c0efff00eb", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.Do), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Equal), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Less), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Greater), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Mod), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.And), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Plus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Minus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Or), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, }, "92a32c49-f3dc-4975-9fd8-d4c0efff00eb") },
		{ "21a4c2c2-7306-4c0f-83ee-6ac2178084b4", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.To), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, }, "21a4c2c2-7306-4c0f-83ee-6ac2178084b4") },
		{ "415cb287-0f72-44b8-ba43-568b757aa408", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.To), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Mod), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.And), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Plus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Minus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Or), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, }, "415cb287-0f72-44b8-ba43-568b757aa408") },
		{ "7ef38f54-fe98-4fcc-a9bd-beab32123ba4", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.MultiplyOperator), "a5ecd9a4-ae45-4220-bf8d-ed67dbb1efdd"}, { new Terminator(OperatorType.Multiply), "e576c109-5246-4139-8248-415b8e3e68de"}, { new Terminator(OperatorType.Divide), "6a567f32-04a0-49b8-bf8e-42f55ee0600f"}, { new Terminator(KeywordType.Divide), "86be9c46-a085-4183-9264-b12d73fde46e"}, { new Terminator(KeywordType.Mod), "d7708b97-7da8-492a-a64f-6196ae9527f7"}, { new Terminator(KeywordType.And), "271d9f46-4be0-495c-a90b-ae8f19ee928b"},}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Plus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Minus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(KeywordType.Or), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(KeywordType.Else), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, }, "7ef38f54-fe98-4fcc-a9bd-beab32123ba4") },
		{ "3d691087-ad63-4f0f-a50a-3a9205410797", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Mod), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.And), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Plus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Minus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Or), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Else), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, }, "3d691087-ad63-4f0f-a50a-3a9205410797") },
		{ "ad50edde-2192-482c-b4f9-477fe5d2c19e", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Else), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, }, "ad50edde-2192-482c-b4f9-477fe5d2c19e") },
		{ "701f351c-1d0b-4a57-8eca-ee87badea333", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.RightParenthesis), "3bc44837-79e1-419a-ab32-0aba4cd6ac3e"}, { new Terminator(DelimiterType.Comma), "077bd83e-2dfc-4686-bf99-ec6992c05b60"},}, new Dictionary<Terminator, ReduceInformation>{ }, "701f351c-1d0b-4a57-8eca-ee87badea333") },
		{ "de6c6542-dcfe-430c-a8eb-29e7fe90cb06", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.RightSquareBracket), "029ea866-e0ea-4e11-8041-11cd93ae09ad"}, { new Terminator(DelimiterType.Comma), "082dc96a-aa2d-444e-b2b0-8a6bca7f6ba2"},}, new Dictionary<Terminator, ReduceInformation>{ }, "de6c6542-dcfe-430c-a8eb-29e7fe90cb06") },
		{ "ec89d2c8-3c95-433e-b6c6-a635fc98dbb6", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Statement), "8575da38-e1a2-446f-9853-62682170976d"}, { new NonTerminator(NonTerminatorType.Variable), "f7849288-2cd0-4c68-aeee-abb4f910fa32"}, { Terminator.IdentifierTerminator, "403840ce-3751-4324-a4da-a2b0a7653957"}, { new NonTerminator(NonTerminatorType.ProcedureCall), "504c43d8-8335-43de-8838-5c3205046f5d"}, { new NonTerminator(NonTerminatorType.CompoundStatement), "15b27fe6-dc50-4e57-8246-bc2e8a5eb05a"}, { new Terminator(KeywordType.If), "1e290bf0-8ecb-4a2d-b5dc-bd2aa7ff6505"}, { new Terminator(KeywordType.For), "83bfcb23-a73e-4ba5-ac65-68e5a019cc6a"}, { new Terminator(KeywordType.Begin), "1fd17dc7-14a7-4500-8bc1-e768a6969eae"},}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(0, new NonTerminator(NonTerminatorType.Statement))}, { new Terminator(KeywordType.Else), new ReduceInformation(0, new NonTerminator(NonTerminatorType.Statement))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(0, new NonTerminator(NonTerminatorType.Statement))}, }, "ec89d2c8-3c95-433e-b6c6-a635fc98dbb6") },
		{ "e6f79bff-9f27-4b9d-91ae-7528965e1569", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.MultiplyOperator), "3cf82f86-e590-49fe-9fd6-3f65d227dbd3"}, { new Terminator(OperatorType.Multiply), "e576c109-5246-4139-8248-415b8e3e68de"}, { new Terminator(OperatorType.Divide), "6a567f32-04a0-49b8-bf8e-42f55ee0600f"}, { new Terminator(KeywordType.Divide), "86be9c46-a085-4183-9264-b12d73fde46e"}, { new Terminator(KeywordType.Mod), "d7708b97-7da8-492a-a64f-6196ae9527f7"}, { new Terminator(KeywordType.And), "271d9f46-4be0-495c-a90b-ae8f19ee928b"},}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.Do), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Plus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Minus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(KeywordType.Or), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, }, "e6f79bff-9f27-4b9d-91ae-7528965e1569") },
		{ "fc33e10e-31dd-4bf5-b2c5-fa308ae42160", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.Do), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Mod), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.And), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Plus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Minus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Or), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, }, "fc33e10e-31dd-4bf5-b2c5-fa308ae42160") },
		{ "9d9f9e17-3d6f-47c1-9674-2ddc47b05109", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.Do), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, }, "9d9f9e17-3d6f-47c1-9674-2ddc47b05109") },
		{ "bd8826e0-0ad5-43bf-a8d9-2b7ec3785625", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.RightParenthesis), "1c920879-022f-49a3-8469-e91ffe7038b3"}, { new Terminator(DelimiterType.Comma), "077bd83e-2dfc-4686-bf99-ec6992c05b60"},}, new Dictionary<Terminator, ReduceInformation>{ }, "bd8826e0-0ad5-43bf-a8d9-2b7ec3785625") },
		{ "59946f05-e479-4422-b708-317b6245a1ab", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.RightSquareBracket), "4fe2aacc-e63a-48a3-884e-13cb5ca9b8e2"}, { new Terminator(DelimiterType.Comma), "082dc96a-aa2d-444e-b2b0-8a6bca7f6ba2"},}, new Dictionary<Terminator, ReduceInformation>{ }, "59946f05-e479-4422-b708-317b6245a1ab") },
		{ "3bc44837-79e1-419a-ab32-0aba4cd6ac3e", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Else), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, }, "3bc44837-79e1-419a-ab32-0aba4cd6ac3e") },
		{ "029ea866-e0ea-4e11-8041-11cd93ae09ad", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Mod), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.And), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Plus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Minus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Or), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Else), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, }, "029ea866-e0ea-4e11-8041-11cd93ae09ad") },
		{ "8575da38-e1a2-446f-9853-62682170976d", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(8, new NonTerminator(NonTerminatorType.Statement))}, { new Terminator(KeywordType.Else), new ReduceInformation(8, new NonTerminator(NonTerminatorType.Statement))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(8, new NonTerminator(NonTerminatorType.Statement))}, }, "8575da38-e1a2-446f-9853-62682170976d") },
		{ "1c920879-022f-49a3-8469-e91ffe7038b3", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.Do), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, }, "1c920879-022f-49a3-8469-e91ffe7038b3") },
		{ "4fe2aacc-e63a-48a3-884e-13cb5ca9b8e2", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.Do), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Mod), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.And), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Plus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Minus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Or), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, }, "4fe2aacc-e63a-48a3-884e-13cb5ca9b8e2") },
	};

    private GeneratedGrammarParser()
    {
        foreach(GeneratedTransformer transformer in s_transformers.Values)
        {
            transformer.ConstructShiftTable(s_transformers);
        }
    }

    private static GeneratedGrammarParser s_instance = new GeneratedGrammarParser();

    public static GeneratedGrammarParser Instance => s_instance;

	public ITransformer BeginTransformer => s_transformers["e697e771-c2e7-47fc-96a0-7070dbb2ce33"];
	public NonTerminator Begin => new NonTerminator(NonTerminatorType.StartNonTerminator);
}
